/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2015. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.ESignOperationDialog");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.ValidationTextBox");
dojo.require("dijit.form.SimpleTextarea");

dojo.require("com.ibm.asq.common.web.client.internal.admin");
dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");
dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.util.SelectUtility");
dojo.require("com.ibm.asq.common.web.ui.internal.util.GlobalEvents");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.asq.common.web.ui.internal.util.BidiUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.TextareaBidi");


(function(){
    var _textContent = dojo.isIE ? "innerText" : "textContent";
    var ImageCache = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
    var SelectUtility = com.ibm.asq.common.web.ui.internal.util.SelectUtility;
    var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
    var adminService = new com.ibm.asq.common.web.client.internal.admin();
    var CommonUtility = com.ibm.asq.common.web.ui.internal.util.Utility;
    var BidiUtil = com.ibm.asq.common.web.ui.internal.util.BidiUtil;
    
    var FIELD_STYLE = { width:"100%", display:"block", marginLeft:"10px" };
    var SELECT_FIELD_STYLE = { width:"100%", display:"block", marginLeft:"10px", padding:"2px" };

    dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.ESignOperationDialog", [dijit._Widget, dijit._Templated], {
        templatePath: dojo.moduleUrl("com.ibm.asq.common.web", "ui/internal/widgets/templates/ESignOperationDialog.html"),

		width: "450px", 
        title: MessageUtil.getMessage(MessageBundle.COMMON, "labelESignRequired"), 
        msgDefaultHeading: MessageUtil.getMessage(MessageBundle.COMMON, "msgEsigDefaultHeading"), 
        msgDefaultApprovalAction: MessageUtil.getMessage(MessageBundle.COMMON, "msgEsigDefaultApprovalActionTemplate"),
        msgDefaultLockAction: MessageUtil.getMessage(MessageBundle.COMMON, "msgEsigDefaultLockActionTemplate"),
        esignImgSrc: ImageCache.ESIG_IMAGE,
        
        labelWhat: MessageUtil.getMessage(MessageBundle.COMMON, "labelWhat"),
        labelAction:  MessageUtil.getMessage(MessageBundle.COMMON, "labelAction"),
		labelUserId: MessageUtil.getMessage(MessageBundle.COMMON, "labelFormLabel", MessageUtil.getMessage(MessageBundle.COMMON, "labelUserId")),
		labelPassword: MessageUtil.getMessage(MessageBundle.COMMON, "labelFormLabel", MessageUtil.getMessage(MessageBundle.COMMON, "labelPassword")),
		labelComment: MessageUtil.getMessage(MessageBundle.COMMON, "labelFormLabel", MessageUtil.getMessage(MessageBundle.COMMON, "labelComment")),
		labelPredefinedComments: MessageUtil.getMessage(MessageBundle.COMMON, "labelSelectAdditionalInfo"),
        labelSinglePredefinedComments: MessageUtil.getMessage(MessageBundle.COMMON, "labelAdditionalInfo"),
		
        msgUserIdIsRequired: MessageUtil.getMessage(MessageBundle.COMMON, "msgUserIdIsRequired"),
        msgPasswordIsRequired: MessageUtil.getMessage(MessageBundle.COMMON, "msgPasswordIsRequired"),
        msgCommentIsRequired: MessageUtil.getMessage(MessageBundle.COMMON, "msgCommentIsRequired"),
        msgPredefinedCommentsIsRequired: MessageUtil.getMessage(MessageBundle.COMMON, "msgAdditionalInfoIsRequired"),
        msgMissingRequiredField: MessageUtil.getMessage(MessageBundle.COMMON, "msgMissingRequiredField"),
        msgIdMismatch: MessageUtil.getMessage(MessageBundle.COMMON, "msgIdMismatch"),

        constructor: function(args){
			//callback functions
            this.onSign = function(){};
			this.onCancel = function(){};
        },
		
		postCreate: function() {
            // get the server time just to trigger the login in in case the session is timed out
            var responseHandler = new ServiceResponseHandler(this, "_initializeDialog", "_initializeDialog");
            adminService.getServerTime(responseHandler);
		},
		
	    _initializeDialog: function() {
		    // check if there is E-Sign config data
            var hasConfigData = false;
            if (this.esignConfigData) {
                // check if it is empty
                for (var key in this.esignConfigData) {
                    hasConfigData = true;
                    break;
                }
            }
            
            // get the current E-Sig signer user id
            this.esigUserId = dojo.getObject(GlobalEvents.ESigSignerUserId);
            var currentUserId = "";  // user has to fill in the id
            if (this.esigUserId) {
                // user signed before, no need to ask for user id again
                currentUserId = this.esigUserId; // pre-fill the user id of the current signer
            }
            
            // set the dialog heading messages
            if (hasConfigData) {
                var heading = this.esignConfigData.heading ? this.esignConfigData.heading : this.msgDefaultHeading;
                this.esignMsgHeading[_textContent] = heading;
                this.actionMsg = this._composeActionMsg();
		    } else {
                // use default heading message if there is no config data
                dojo.style(this.esignMsgHeading, "display", "none");
                this.actionMsg = this.esignRequiredMessage;
		    }
            this.esignMsgAction[_textContent] = this.actionMsg;
		    
            // create the user id
		    this._userIdWidget = new dijit.form.ValidationTextBox({
                value: currentUserId,
                style: FIELD_STYLE,
                textDir: BidiUtil.getWidgetBaseTextDir(),
                onBlur: dojo.hitch(this, function() {
                    this._validateUserId(true);
                }),
                onKeyUp: dojo.hitch(this, function(event) {
                    if (this._keyHandler(event)) {   // validate fields without updating the error node for OK button enablement
                        this._validateUserId(true);  // validate this field to update the error node
                    }
                }),
                "aria-labelledby": this.id + "_userId"
            });
            this.userIdNode.appendChild(this._userIdWidget.domNode);
	        
            // create the password
            this._passwordWidget = new dijit.form.ValidationTextBox({
                type: "password",
                style: FIELD_STYLE,
                onBlur: dojo.hitch(this, function() {
                    this._validatePassword(true);
                }),
                onKeyUp: dojo.hitch(this, function(event) {
                    if (this._keyHandler(event)) {
                        this._validatePassword(true);
                    }
                }),
                "aria-labelledby": this.id + "_password"
            });
            this.passwordNode.appendChild(this._passwordWidget.domNode);
            
            // create the optional predefined comments drop down
            if (this.esignConfigData && this.esignConfigData.requiresPredefinedComment == "true") {
                this._createPredefinedCommentsDropdown(this.esignConfigData.predefinedComments);
                if (this.PredefinedCommentsDropdown) {
                    // there is predefined comments
                    if (this.PredefinedCommentsDropdown.isSingle) {
                        // domNode is just a <span> label for only one choice, change label to show the only choice
                        this.predefinedCommentsLabel[_textContent] = this.labelSinglePredefinedComments;
                        dojo.style(this.PredefinedCommentsDropdown.domNode, FIELD_STYLE);
                    } else {
                        // domNode is a <select>
                        dojo.style(this.PredefinedCommentsDropdown.domNode, SELECT_FIELD_STYLE);
                    }
                    this.predefinedCommentsNode.appendChild(this.PredefinedCommentsDropdown.domNode);
                    dojo.style(this.predefinedComments, "display", "block");
                }
            }
            
            // create the optional comments
            if (!hasConfigData || (this.esignConfigData && this.esignConfigData.requiresComment == "true")) {
                this._commentWidget = new com.ibm.asq.common.web.ui.internal.widgets.SimpleTextareaBidi({
                    cols: 70,
                    rows: 5,
                    style: FIELD_STYLE,
                    onBlur: dojo.hitch(this, function() {
                        this._validateComment(true);
                    }),
                    onKeyUp: dojo.hitch(this, function(event) {
                        if (this._keyHandler(event)) {
                            this._validateComment(true);
                        }
                    }),
                    "aria-labelledby": this.id + "_comments"
                });
                this.commentsNode.appendChild(this._commentWidget.domNode);
                dojo.style(this.comments, "display", "block");
            }
            
            // display the dialog
		    var dialogArgs = {
				primaryTitle: this.title,
				okCallback: dojo.hitch(this, "_onSign"), 
				cancelCallback: dojo.hitch(this, "_onCancel"), 
		        isClosable: false
		    };                
		    this._dialog = new com.ibm.asq.common.web.ui.internal.widgets.SelectionDialog(dialogArgs);
		    this._dialog.setWidth(this.width);
			this._dialog.getOkButton().disabled = true;
		    this._dialog.contentContainer.appendChild(this.domNode);
			this._dialog.open();			
			setTimeout(dojo.hitch(this, function() {
			    if (this.esigUserId) {
			        // user id is pre-filled with current signer
    			    this._passwordWidget.focus();
			    } else {
			        // no current signer, user need to specify the user id
                    this._userIdWidget.focus();
			    }
			}), 250); // focus on password field					
		},
		
		// compose the action heading message
		//
		_composeActionMsg: function() {
		    if (this.actionMsgOverride) {
		        return this.actionMsgOverride;
		    }
            var actionTemplate = this.esignConfigData.actionTemplate;
		    if (!actionTemplate) {
		        // use default message that can be for lock/unlock or formal review
		        actionTemplate = this.esignConfigData.isFormalReview ? this.msgDefaultApprovalAction : this.msgDefaultLockAction;
		    }
		    var actionMsg = "";
		    var approvalActions = this.esignConfigData.approvalActions || [];
		    for (var i = 0; i < approvalActions.length; i++) {
		        // it is really unusual that the user has multiple approval actions so
		        // we just repeat the message for each approval
		        var name = approvalActions[i].name || "";  // there may not be any name if the e-sign is not for formal review
                var action = approvalActions[i].action;
                var msg = actionTemplate.replace(/\${action}/g, action);
                msg = msg.replace(/\${name}/g, name);
                actionMsg = (actionMsg == "") ? msg : actionMsg + " \n" + msg;
		    }
		    return actionMsg;
		},
		
		// create the predefined comments drop-down
		//
		_createPredefinedCommentsDropdown: function(predefinedComments) {
		    // split the "\n" separated JSON string of predefined comments to an array
            var comments = predefinedComments ? predefinedComments.split("\n") : [];
            var commentsCount = comments.length;
		    if (commentsCount < 1) {
		        // there is no comment defined, nothing to show
                this.PredefinedCommentsDropdown = null;
		    } else {
                this.PredefinedCommentsDropdown = {};
		        var dom;
		        if (commentsCount == 1) {
		            // only one comment is defined, use a label instead of a drop-down
		            dom = dojo.create("span", { className: "value-field"});
		            dom[_textContent] = comments[0];
		        } else {
		            // create the select widget with option comments
	                dom = dojo.create("select", { className: "value-field", "aria-labelledby" : this.id + "_predefined_comments" });
	                this.connect(dom, "onchange", dojo.hitch(this, function() {
	                    this._validatePredefinedComment(true);
	                }));
	                 this.connect(dom, "onkeyup", dojo.hitch(this, function(event) {
	                	var key = event.keyCode;
	                	var dojoKeys =  dojo.keys;
	                	if(key == dojoKeys.UP_ARROW || key == dojoKeys.DOWN_ARROW){
	                		this._validatePredefinedComment(true);
	                	}
	                }));
	                // use blank for no default value
                    var blank = new Option("", "");
                    SelectUtility.add(dom, blank);
    	            for (var i = 0; i < commentsCount; i++) {
	                    var option = new Option(comments[i], comments[i]);
	                    SelectUtility.add(dom, option);
    	            }
	            }

	            this.PredefinedCommentsDropdown = {
		            isSingle: commentsCount == 1,
		            domNode: dom
		        };
		    }
		},
		
        // validate that predefined comment field is not empty if it is required
        //
		_validatePredefinedComment: function(showError) {
		    if (this.PredefinedCommentsDropdown) {
		        if (this.PredefinedCommentsDropdown.isSingle) {
                    return this.PredefinedCommentsDropdown.domNode[_textContent];
		        } else {
		            var select = this.PredefinedCommentsDropdown.domNode;
	                var isEmpty = (select.value == "");
	                if (showError) {
                        dojo.style(this.predefinedCommentsErrorNode, "visibility", isEmpty ? "visible" : "hidden");
	                }
	                // set the enablement of the OK button
                    var textFieldsOK = this._validateTextFields();
	                this._dialog.getOkButton().disabled = (!textFieldsOK || isEmpty) ? true : false;
	                return isEmpty ? null : select.value;
		        }
		    } else {
                // it is fine that the optional predefined comments widget is not created
		        return true;
		    }
		},
		
		// check if the specified widget is empty to show the error icon
		// 
		_checkEmpty: function(widget, errorNode) {
		    var value = dojo.trim(widget.get('value'));
            var isEmpty = value.length == 0;
            if (errorNode) {
                dojo.style(errorNode, "visibility", isEmpty ? "visible" : "hidden");
            }
            return isEmpty ? null : value;
		},
		
		// validate that the user id field is not empty
		//
		_validateUserId: function(showError) {
            return this._checkEmpty(this._userIdWidget, showError ? this.userIdErrorNode : null);
		},
		
        // validate that the password field is not empty
        //
		_validatePassword: function(showError) {
            return this._checkEmpty(this._passwordWidget, showError ? this.passwordErrorNode : null);
		},
        
        // validate that comment field is not empty if it is required
        //
		_validateComment: function(showError) {
		    if (this._commentWidget) {
                return this._checkEmpty(this._commentWidget, showError ? this.commentsErrorNode : null);
		    } else {
		        // it is fine that the optional comment widget is not created
		        return true;
		    }
		},
        
		_validateTextFields: function() {
            var userIdOK = this._validateUserId();
            var passwordOK = this._validatePassword();
            var commentOK = this._validateComment();
            return (userIdOK && passwordOK && commentOK) ? true : false;
		},
		
        _keyHandler: function(event) {
            if (event && CommonUtility.isNavigationKey(event)) {
                // ignore cursor navigation such as the tab and arrow keys 
                dojo.stopEvent(event);
                return false;
            } else {
                // set the enablement of the OK button
                var textFieldsOK = this._validateTextFields();
                var predefinedCommentOK = this._validatePredefinedComment();
                this._dialog.getOkButton().disabled = (textFieldsOK && predefinedCommentOK) ? false : true;
                return true;
            }
        },
        
		_onCancel: function() {
            if (dojo.isFunction(this.onCancel)) {
				this.onCancel();
			}
		},
		
		_onSign: function() {
			// ensure all inputs have values
		    var predefinedComment = this._validatePredefinedComment();
		    var userId = this._validateUserId();
		    var comment = this._validateComment();
			if (userId && this._validatePassword() && predefinedComment && comment) {
	            if (dojo.isFunction(this.onSign)) {
                    if (predefinedComment === true) {
                        // predefined comment is not required
                        predefinedComment = "";
                    }
                    if (comment === true) {
                        // comment is not required
                        comment = "";
                    }
                    
                    // check the user id
                    var loggedInUserId = com.ibm.team.repository.web.client.session.getAuthenticatedContributor().userId;
                    if (loggedInUserId == userId) {
                        // id's match
                        if (!this.esigUserId) {
                            // no current E-Sig signer user id, signed for the first time
                            // save the user id as the current E-Sig signer
                            dojo.setObject(GlobalEvents.ESigSignerUserId, loggedInUserId);
                        }
                    } else {
                        // id's doesn't match, post error
                        this._status[_textContent] = this.msgIdMismatch;
                        dojo.style(this._status, "display", "block");
                        this._userIdWidget.focus();
                        return false;
                    }
	                this.onSign(this._passwordWidget.get('value'), comment, predefinedComment, this.actionMsg);
	            }
	            return true;
			} else {
			    this._status[_textContent] = this.msgMissingRequiredField;
	            dojo.style(this._status, "display", "block");
				return false;
			}
		}
		
    });
      
})();
