/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2015. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.rdm.web.viewlet.SingleQueryViewlet");

dojo.require("com.ibm.rdm.web.artifactgrid.ArtifactConstants");
dojo.require("com.ibm.rdm.web.common.ResourceLinksHandler");
dojo.require("com.ibm.rdm.web.common.ArtifactIcons");
dojo.require("com.ibm.rdm.web.model.types.RRMTypes");
dojo.require("com.ibm.rdm.web.model.types.TypesManager");
dojo.require("com.ibm.rdm.web.query.SavedFilterUtils");
dojo.require("com.ibm.rdm.web.query.ViewServiceClient");
dojo.require("com.ibm.rdm.web.query.stringMatchUtils");
dojo.require("com.ibm.rdm.web.server.client.RDMClient");
dojo.require("com.ibm.rdm.web.suspicion.SuspectLinksUtil");
dojo.require("com.ibm.rdm.web.suspicion.SuspicionStatus");
dojo.require("com.ibm.rdm.web.suspicion.model.types");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("com.ibm.rdm.web.viewlet.ArtifactQueryPreferenceProvider");
dojo.require("com.ibm.rdm.web.viewlet.ModulePreferenceProvider");
dojo.require("com.ibm.rdm.web.viewlet.RRCViewlet");
dojo.require("com.ibm.rdm.web.util.vvcutils");
dojo.require("jazz.util.i18n");

dojo.requireLocalization("com.ibm.rdm.web.viewlet", "SingleQueryViewlet");
dojo.requireLocalization("com.ibm.rdm.web.common", "common");

(function(){

var RRCViewlet = com.ibm.rdm.web.viewlet.RRCViewlet;
var I18nUtil = jazz.util.i18n;
var ArtifactQueryPreferenceProvider = com.ibm.rdm.web.viewlet.ArtifactQueryPreferenceProvider;
var ModulePreferenceProvider = com.ibm.rdm.web.viewlet.ModulePreferenceProvider;
var SavedFilterUtils = com.ibm.rdm.web.query.SavedFilterUtils;
var SuspectLinksUtil = com.ibm.rdm.web.suspicion.SuspectLinksUtil;
var ViewServiceClient = com.ibm.rdm.web.query.ViewServiceClient;
var RRMTypes = com.ibm.rdm.web.model.types.RRMTypes;
var ResourceLinksHandler = com.ibm.rdm.web.common.ResourceLinksHandler;
var ArtifactIcons = com.ibm.rdm.web.common.ArtifactIcons;
var domutils = com.ibm.rdm.web.util.domutils;
var stringMatchUtils = com.ibm.rdm.web.query.stringMatchUtils;
var VvcUtils = com.ibm.rdm.web.util.vvcutils;
var DEFAULT_QUERY = "defaultQuery";
var DEFAULT_MODULE = "defaultModule";

var messages = dojo.i18n.getLocalization("com.ibm.rdm.web.viewlet", "SingleQueryViewlet");
var commonMessages = dojo.i18n.getLocalization("com.ibm.rdm.web.common", "common");



dojo.declare(
	"com.ibm.rdm.web.viewlet.SingleQueryViewlet",
	RRCViewlet,
{
	templatePath: dojo.moduleUrl("com.ibm.rdm.web.viewlet", "templates/SingleQueryViewlet.html"),      
	projectScopeURI: null, /* Project scope for this viewlet */
	moduleScopeURI: null,	/* Module scope for this viewlet */
	
	_attributeTypes: null,
	_linkTypes: null,
	_customAttributes: null,
	_pagingWidget: null,
	_resourceLinks: [],

 
	init: function(){
		this.inherited(arguments);
		
		if (this.getSite().getFilterBox) {
			this.getSite().getFilterBox().onChange = dojo.hitch(this, this._beginFilterTimeout);
		}
		
		this._pageToDisplay = 1;
		
		//Initialize paging widget
		this.addPaginationWidget();
	},
					
	getItemsPerPage: function(){
		return this.getPreference("itemsToShow");
	},
	
	refresh: function(){
		this.inherited(arguments);
		if (this.isInitialized()) {
			this._destroyResourceLinks();
			
			// Message that prompts the user to set a project in the preferences setting.
			if (!this.getScopeItem() || this.getPreference("query") === -1) {
				//clear contents and show msg
				this._showMessage(this._createEditSettingsMessage(false));
				this.getSite().doneRefresh();
				return;
			}

			//get the scoped projectURI 
			this.projectScopeURI = (this.getScopeItem()) ? com.ibm.rdm.web.FRONTING_SERVICE_JFS_PROJECT_AREAS + "/" + this.getScopeItem().itemId : null;

			// if the viewlet has a default query and it has not been changed, initialize the title and preference provider
			var currentlySelectedQuery = this.getPreference("query");
			if (currentlySelectedQuery && currentlySelectedQuery !== "-1") {
				//mark the following query uri for pre-selection in the saved query chooser
				this.getPreferenceProvider("query").initDefault(currentlySelectedQuery); 
			}

			var currentlySelectedModule = this.getPreference("module");
			if (currentlySelectedModule && currentlySelectedModule !== "-1") {
				this.moduleScopeURI = currentlySelectedModule;
			}			
			
			//proceed to get the query results if both project and query are defined.
			if(this.projectScopeURI && currentlySelectedQuery && currentlySelectedQuery !== "-1"){
				//Process the new query            
				this.processSavedQuery(currentlySelectedQuery);
			}else{
				//Not a valid query, prompt user to select a valid query
				this._showMessage(this._createEditSettingsMessage(false));
			}
			com.ibm.rdm.web.common.UpdateContext.refreshHasOccurred();
		}
		else {
			this.waitForInit(dojo.hitch(this, this.refresh));
		}
	},
		   
	settingChanging: function(settingId, currentSettings){
		
		//Check if a new project scope has been selected, if so return the id so that the computed
		//preferences for query URI or module is computed.
		if (settingId == "scopeItem") {
			this.projectScopeURI = (currentSettings.scopeItemId) ? com.ibm.rdm.web.FRONTING_SERVICE_JFS_PROJECT_AREAS + "/" + currentSettings.scopeItemId : null;
			this.getPreferenceProvider("query").setProjectScope(this.projectScopeURI);	
			this.getPreferenceProvider("module").setProjectScope(this.projectScopeURI);
			//Indicate that the defaultQuery and default module value is no good to use since we have switched project scope
			this._clearDefaultQuery();
			this._clearDefaultModule();
			return["query", "module"];
		}
		else if (settingId == "module") {
			// Set the view's scope to that module
			this.moduleScopeURI = currentSettings.preferences.module;
			this.getPreferenceProvider("query").setModuleScope(this.moduleScopeURI);

			// Whenever the module has changed, the view field should be cleared.
			this._clearDefaultQuery();
			return["query"];
		}
		return [];
	},
	
	settingsChanged: function(/* object */oldSettings, /* object */ newSettings){
				
		//Test if this is the first time we are setting the query preference           
		if (oldSettings.preferences.query === "-1") {
			if (this.getSite().getFilterBox) {
				this.getSite().getFilterBox().clear();
			}
			
			if (newSettings.preferences.query !== "-1") {					
				//Always the first page of results when changing context
				this._pageToDisplay=1;
				//Process the new query            
				this.processSavedQuery(newSettings.preferences.query);
				//Save the query into a user preference
				this.getContributorState().set(DEFAULT_QUERY, newSettings.preferences.query);
			}
			//Not a valid query, prompt user to select a valid query
			this._showMessage(this._createEditSettingsMessage(false));
		}
		else 
			if (oldSettings.preferences.query != newSettings.preferences.query) {
				if (newSettings.preferences.query !== "-1") {
					if (this.getSite().getFilterBox) {
						this.getSite().getFilterBox().clear();
					}
					//Always the first page of results when changing context
					this._pageToDisplay=1;
					//Process the new query            
					this.processSavedQuery(newSettings.preferences.query);
					this.getContributorState().set(DEFAULT_QUERY, newSettings.preferences.query);
				}
				//Not a valid query, prompt user to select a valid query
				this._showMessage(this._createEditSettingsMessage(false));

				// Update the title to show no query has been selected.
				this.getSite().setTitle(messages.requirementsView, true);		
				this.getSite().setSecondaryTitle(' ');
			}
			else 
				if (oldSettings.preferences.itemsToShow != newSettings.preferences.itemsToShow) {
					this.refresh();
				}
				
		
		// getTextLabel is not CustomPreferenceProvider API
		// but is provided by the WorkItemQueryPreferenceProvider
		// always persist the query title (when it's available) on setting changes in case it has changed
		if (this.getPreferenceProvider("query").getTextLabel(newSettings.preferences.query)) {
			// If a module has been specified, then use its name to qualify the query.
			var moduleName = null;
			if (this.moduleScopeURI) {
				moduleName = this.getPreferenceProvider("module").getTextLabel(newSettings.preferences.module);	
			}			
			
			//Set the title to the new query name
			var queryName = this.getPreferenceProvider("query").getTextLabel(newSettings.preferences.query);
			this.getSite().setTitle(moduleName ? moduleName + " : " + queryName : queryName, true);			
		}          
	},
	
	processSavedQuery: function(queryURI){
		var loadArtifactIconsDeferred =
			ArtifactIcons.loadTypes(this.projectScopeURI);

		// Display the Loading message.
		this._showMessage(document.createTextNode(commonMessages.loading));
		
		// Process the requested saved query.
		this._parseSavedQuery(queryURI).addCallback(this, function(queryParams){
			this.runQuery(queryParams).addCallback(this, function(queryResults){
				loadArtifactIconsDeferred.then(
					dojo.hitch(
						this,
						function(){
							this.displayResults(queryResults);
						}
					)
				);
			});
		});
	   
	},
	
	getTitleHyperlinkURL: function(){
		var queryId = this.getPreference("query");
		if (queryId != "-1") {
			return this._getQueryHref(queryId);
		}
		// if the viewlet has a default query and it has not been changed, initialize the title and preference provider
		var currentlySelectedQuery = this.getContributorState().get(DEFAULT_QUERY);
		if (currentlySelectedQuery && currentlySelectedQuery !== "-1") {
			return this._getQueryHref(currentlySelectedQuery);
		}
		return null;
	},
	
	_getQueryHref: function(uri) {
		var filterURI = com.ibm.rdm.web.FRONTING_SERVICE_CONTEXT_ROOT + "web";
		
		// If it's a module view, then construct the url to take you there instead.
		if (this.moduleScopeURI) {
			filterURI += "#action=com.ibm.rdm.web.pages.showArtifact&artifactURI=" + encodeURIComponent(this.moduleScopeURI);
		}
		else {
			//Return URL to open query in project dashboard             		
			filterURI += "#action=com.ibm.rdm.web.pages.showProjectDashboard&projectURI=" + encodeURIComponent(this.projectScopeURI);
		}
		
		filterURI += "&filterId=";
		filterURI += uri.substring(uri.lastIndexOf('/') + 1);
		if (com.ibm.rdm.web.CURRENT_PROJECT) {
			filterURI = VvcUtils.replaceConfigurationInActionUriWithCurrent(filterURI);
		}
		return filterURI;
	},

	
	displayResults: function(queryResults){
		//Parse the query results
		var results = ViewServiceClient.parseViewContentsResponse(queryResults, this.projectScopeURI, {}, false);
		var resources = results.resources;
		//Save the current page information
		this._pageToDisplay = results.currPage;
		this._lastPage = results.lastPage;
		
		//Display the result set count as a secondary title of the viewlet
		var totalCount = (results.count)?results.count:0;
		var resultCountLabel = "(" + totalCount + ")";

		this.getSite().setSecondaryTitle(
			dojo.string.substitute(
				messages.inProjectWithCount,
				[this.getScopeItem().name, totalCount]
			)
		);
		
		//Clean the results node
		domutils.removeChildNodes(this._queryResultSectionNode);
		
		//Display or hide the pagination based on the amount of pages found	
		if (this._lastPage == 0 || this._lastPage == 1) {
			this._pageToDisplay = 1;
			this._lastPage = 0;
			dojo.style(this._pagingWidget.domNode, 'visibility', 'hidden');
		}
		else {
			dojo.style(this._pagingWidget.domNode, 'visibility', 'visible');
			this._pagingWidget.refresh();
		}
		
		if (results.resources.length > 0) { //Make sure the query returned results
			// Create a table and parent it appropriately.
			var table = document.createElement("table");
			if (!dojo.isIE) {
				table.setAttribute("cellSpacing", "0");
			}
			table.className = "search-results-table";
			var tbody = document.createElement("tbody");
			
			
			for (var i = 0; i < resources.length; i++) {
				var resourceEntryElement = document.createElement("tr");
				this._createUserDashResourceTableRow(resourceEntryElement, resources[i]);
				tbody.appendChild(resourceEntryElement);
			}
			table.appendChild(tbody);
			this._queryResultSectionNode.appendChild(table);
		} else {
			//Indicated no matches were found
			//display Loading message
			this._showMessage(document.createTextNode(commonMessages.noResultsFilter));
		}
	},
		 
	runQuery: function(queryParams){
		var viewServiceRequestParams = this._getViewServiceRequestParams(queryParams);
		return ViewServiceClient.getViewContents(viewServiceRequestParams);
	},
	
	getPreferenceProvider: function(/* string */preferenceId){
		if (!this._preferenceProviders) {
			this._preferenceProviders = {};
		}
		
		if (!this._preferenceProviders[preferenceId]) {
			if (preferenceId === "query") {
				this._preferenceProviders[preferenceId] = new ArtifactQueryPreferenceProvider({
					"prefId": preferenceId,
					"messages": commonMessages
				});
			}
			else if (preferenceId === "module") {
				this._preferenceProviders[preferenceId] = new ModulePreferenceProvider({
					"prefId": preferenceId,
					"messages": commonMessages
				});
			}
		}
		return this._preferenceProviders[preferenceId];
	},
	
	addPaginationWidget: function(){
		var that = this;
		//Add paging
		this._pagingWidget = new com.ibm.rdm.web.grid.PagingWidget({}, this._paginationWidgetNode);
		
		//Pass callback parameters to paging widget
		this._pagingWidget.addPaging({
			getTotalPagesFunction: function(){
				if (that._lastPage) {
					return that._lastPage;
				} else { 
					return -1;
				}
			},
			getCurrentPageFunction: function(){
				return that._pageToDisplay || 1;
			},
			gotoPageFunction: function(pageNum){
				that._pageToDisplay = pageNum;
				that.getSelectedPageResults();
			}
		});
		
		//Do not display the paging widget until is needed
		dojo.style(this._pagingWidget.domNode, 'visibility', 'hidden');
	},
	
	getSelectedPageResults: function(){        
		this.processSavedQuery(this.getPreference("query"));
	},
				 
	destroy: function(){
		this._destroyResourceLinks();
		if (this._pagingWidget) {
			this._pagingWidget.destroyRecursive();
		}
		this.inherited(arguments);
	},

	_destroyResourceLinks: function() {
		for (var i = 0; i < this._resourceLinks.length; i++) {
			this._resourceLinks[i].destroyRecursive();
		}
		this._resourceLinks = [];
	},

	_getModuleName: function(moduleURI){
		var moduleNameDeferred = new dojo.Deferred();
		var moduleDeferred = com.ibm.rdm.web.model.resources.ResourceManager.get({
			resourceURI: moduleURI
		});
		moduleDeferred.addCallback(this, function(module) {
			var moduleID = module.getID();			
			var moduleName = module.getDisplayTitle() + " (" + moduleID + ")";			
			moduleNameDeferred.callback(moduleName);
		});
		
		return moduleNameDeferred;
	},		
	
	_parseSavedQuery: function(queryURI){
	
		var deferred = new dojo.Deferred();
	 
		if (queryURI /*savedQueryDTO */) {
			
			var deferredArray = [];
						
			// Get the module name if module specified.
			if (this.moduleScopeURI) {
				var moduleNameDeferred = this._getModuleName(this.moduleScopeURI);
				moduleNameDeferred.addCallback(this, function(moduleName){
					this.getPreferenceProvider("module").setModuleName(this.moduleScopeURI, moduleName);						
				});
				
				deferredArray.push(moduleNameDeferred);
			}			
			
			var customAttributesDeferred = com.ibm.rdm.web.model.types.TypesManager.getAttributeDefinitionsForProject({projectURI: this.projectScopeURI});
			customAttributesDeferred.addCallback(this, function(response){
				this._customAttributes = {};
				for (var customAttribute in response){
					this._customAttributes[response[customAttribute].attributeDefinitionURI] = response[customAttribute];
				}
			});
			deferredArray.push(customAttributesDeferred);
			
			var attributeTypesDeferred = com.ibm.rdm.web.model.types.TypesManager.getAttributeTypesForProject({projectURI: this.projectScopeURI});
			attributeTypesDeferred.addCallback(this, function(response){
				this._attributeTypes = {};
				for (var attributeType in response){
					this._attributeTypes[response[attributeType].attributeTypeURI] = response[attributeType];
				}
			});
			deferredArray.push(attributeTypesDeferred);
			
			var linkTypesDeferred = com.ibm.rdm.web.model.types.TypesManager.getLinkTypesForProject({
				projectURI: this.projectScopeURI
			});
			linkTypesDeferred.addCallback(this, function(linkTypesArray){
				this._linkTypes = {};
				for (var i = 0; i < linkTypesArray.length; i++) {
					var linkType = linkTypesArray[i];
					this._linkTypes[linkType.getResourceURI()] = linkType;
				}
				return linkTypesArray;
			});
			deferredArray.push(linkTypesDeferred);

			var deferredList = new dojo.DeferredList(deferredArray);
			deferredList.addCallback(this, function(defListResponse){
				var savedQueryDeferred = com.ibm.rdm.web.server.client.RDMClient.doGet({
					uri: queryURI,
					xhrArgs: {
						handleAs: "json",
						headers: {
							Accept: "application/json"
						}
					}
				});
				savedQueryDeferred.addCallback(this, function(savedFilter){
					var queryName = this._getSavedQueryNameFromResponse(savedFilter.doc,queryURI);
					//update the viewlet title in case the query was renamed
					var moduleName = null;
					if (this.moduleScopeURI) {
						moduleName = this.getPreferenceProvider("module").getTextLabel(this.moduleScopeURI);	
					}
					
					this.getSite().setTitle(moduleName ? moduleName + " : " + queryName : queryName);
					
					this.getPreferenceProvider("query").setQueryName(queryURI, queryName);
					var parsedFilter = SavedFilterUtils.parseFilterFromJson(
						savedFilter.doc,
						queryURI,
						this.projectScopeURI,
						this._customAttributes,
						this._attributeTypes,
						this._linkTypes
					);
					deferred.callback(parsedFilter);
				});
				savedQueryDeferred.addErrback(this, function(error){
					com.ibm.rdm.web.common.MessageHandler.showError(7236);
					deferred.callback(null);
				});
			});
		}
		else{
			deferred.callback(null);
		}
		
		return deferred;
	},
	
	_getViewServiceRequestParams: function(queryParams){
		
		
		//Process any arguments passed to the search filter
		// Add an attribute for Artifact ID or Title 
		var attributeConditionInfos = []; 
		var fullTextCondition = null;
  
		
		if (this.getSite().getFilterBox().value != "") {
		
			var stringSearch = this.getSite().getFilterBox().value;
			if (!isNaN(stringSearch)) {
				//Search By ID 
				var attributeURI = "http://purl.org/dc/terms/identifier";
				attributeConditionInfos.push({
					attributeDefinitionURI: attributeURI,
					attributeType: RRMTypes.ATTRIBUTE_TYPES.IntegerType,
					attributeValues: ["is", stringSearch],
					isEnum: false
				});
			}
			else {
				//Process LUCENE search  
				// Add lucene search clause
				fullTextCondition = {
					searchText: stringSearch,
					searchField: ViewServiceClient.LUCENE_SEARCH_FIELDS.FullContent
				};
				
			}
		}
				
	
		//Gather all of the query parameters from the parsed saved query filter and return a viewServiceRequestParams object to pass
		//to the ViewServiceClient
		
		var queryAttributes = [RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Identifier, 
							   RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Title, 
							   RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.ArtifactFormat, 
							   "http://www.ibm.com/xmlns/rdm/rdf/wrappedResourceMimeType",
							   "http://www.ibm.com/xmlns/rdm/rdf/wrappedResourceURI",
							   RRMTypes.WELL_KNOWN_OBJECT_TYPE];
		
		
		var pageSize = this.getItemsPerPage();
		
		dojo.some(queryParams.columns, function(column) {
			if (column.colURI == com.ibm.rdm.web.artifactgrid.ArtifactConstants.SUSPICION_COL) {
				queryAttributes.push(com.ibm.rdm.web.suspicion.model.types.SUSPECT_ENTRIES);
				return false;
			}
		});
		
		if(attributeConditionInfos.length > 0){
			for(var i=0; i < attributeConditionInfos.length; i++){
				queryParams.attributeConditionInfos.push(attributeConditionInfos[i]);
			}
		}
			
		var viewServiceRequestParams = {
			projectURI: this.projectScopeURI,
			moduleURI: (this.moduleScopeURI) ? this.moduleScopeURI : null,
			baselineURI: null,
			collectionURI: null,
			clmFilters: (
				queryParams.calm && queryParams.calm.length > 0 ?
				queryParams.calm :
				null
			),
			folderURIs: queryParams.folderURIs,
			tagOperator: queryParams.tagOperator,
			tagURIs: queryParams.tagURIs,
			suspicionProfileURIs: queryParams.suspicionProfileURIs,
			artifactTypeConditionInfo: queryParams.artifactTypeConditionInfo,
			attributeConditionInfos: queryParams.attributeConditionInfos,
			linkedArtifactConditionInfos: queryParams.linkedArtifactConditionInfos,
			//Identify the query order and the page number.
			groupBy: queryParams.groupBy,
			orderBy: queryParams.orderBy,
			pageRequested: this._pageToDisplay,
			pageSize: pageSize,
			requestedAttributeUris: queryAttributes,
			fullTextCondition: fullTextCondition,
			capQueryExecutionTime: true,			
			isCoreView: (this.moduleScopeURI) ? false : true,
			disableModuleReuseMode: (this.moduleScopeURI) ? false : true,
			disableModulefactBasePropertiesUnionisCoreView: (this.moduleScopeURI) ? true : false			
		};
		
		return viewServiceRequestParams;
	},
	
	
	_createUserDashResourceTableRow: function(tr, resource){
		//Get the resource information
		var resourceURI = resource.getResourceURI();
		var identifier = resource.getID();
		var resourceName = resource.getTitle();
		//Add the current identifier to the matching result
		resourceName += " (" + identifier + ") ";
			 
		var entityType = null;
		
		if (resource.getEntityType() == RRMTypes.getArtifactFormats(this.projectScopeURI)["WrapperResource"]) {
			entityType = resource.getWrappedResourceMimeType();
		}
		else {
			entityType = resource.getEntityType();
		}
		
		var imageSrc;
		try {
			imageSrc = ArtifactIcons.getSmallImageURI(
				this.projectScopeURI,
				resource.getType()
			);
		} catch (e) {
			imageSrc = com.ibm.rdm.web.common.MimeTypeRegistry.getImageURI(
				entityType, this.projectScopeURI
			);
		}
		
		var imgNode = document.createElement("img");
		
		var link = ResourceLinksHandler.createLink({
			uri: resourceURI,
			label: resourceName,
			viewlet: this
		});
		this._resourceLinks.push(dijit.byNode(link));
		
		var suspicionTD = document.createElement("td");
		var imgTD = document.createElement("td");
		var resourceNameTD = document.createElement("td");
		
		imgNode.src = imageSrc;
		
		var typeURI = null;
		var resourceTypes = resource.getTypes();
		for (var uri in resourceTypes){
			if (uri && !typeURI){
				typeURI = uri;
			}
		}
		ArtifactIcons.addAltTitleAttributesToIcon(imgNode, resource.getProjectURI(), typeURI);
		
		dojo.addClass(imgTD, "resource-icon");
		dojo.addClass(suspicionTD, "suspicion-td");
		 
		imgTD.appendChild(imgNode);
		resourceNameTD.appendChild(link);
		
		var suspectEntryUris = resource.suspectEntries;
		if (suspectEntryUris && suspectEntryUris.length > 0) {
			var suspicionStatusWidget = new com.ibm.rdm.web.suspicion.SuspicionStatus({
				source: SuspectLinksUtil.EVENT_SOURCE.viewlet,
				projectUri: resource.projectURI,
				suspectEntryUris: suspectEntryUris,
				targetResourceUri: resource.resourceURI
			});
			suspicionTD.appendChild(suspicionStatusWidget.domNode);
		}
		
		tr.appendChild(suspicionTD);
		tr.appendChild(imgTD);
		tr.appendChild(resourceNameTD);
	},
	
	
	_getSavedQueryNameFromResponse: function(queryDocResponse, queryURI){
		var name ="";
		
		if(queryDocResponse[queryURI]){
			var queryInfo = queryDocResponse[queryURI];
			if(queryInfo["http://purl.org/dc/terms/title"]){
				var encodedQueryName = queryInfo["http://purl.org/dc/terms/title"][0].value;
				name = (encodedQueryName.indexOf("\\Q") === 1) ?
					stringMatchUtils.javaRegexpEncodedInputTextToInputText(encodedQueryName) :
					encodedQueryName;
			}
		}		
		return name;
	},
					
			
	_createEditSettingsMessage: function(/* boolean */isDeleted){
		var result = document.createElement("span");
		var links = I18nUtil.addMessage(
			result,
			(
				isDeleted ?
				commonMessages.selectNewQuery :
				commonMessages.selectQuery
			)
		);
		this.connect(links[0], "onclick", this._doPreferences);
		
		// Hide the paging widget since there is nothing to page
		dojo.style(this._pagingWidget.domNode, 'visibility', 'hidden');
		
		return result;
	},
	
	/*
	 * Shows a message (a DOM node) in the viewlet content area, replacing any other
	 * content that was there.
	 */
	_showMessage: function(/* Node */node){
		if (node) {
			 //Clean the results node
			domutils.removeChildNodes(this._queryResultSectionNode);
			this._queryResultSectionNode.appendChild(node);
		}           
	},
	
	_doPreferences: function(event){
		dojo.stopEvent(event);
		this.getSite().toggleSettings();
	},
	
	_clearDefaultQuery: function(){
		this.getContributorState().set(DEFAULT_QUERY, "-1");
	},
	
	_clearDefaultModule: function(){
		this.getContributorState().set(DEFAULT_MODULE, "-1");
	},

	_beginFilterTimeout: function(value) {

		if (this._timeoutCounter) {
			clearTimeout(this._timeoutCounter);
		}
		
		this._timeoutCounter = setTimeout(dojo.hitch(this, this.refresh), 1000);
	}
});
	  
})();
