/***************************************************************************************************************************************************************
 * Licensed Materials - Property of IBM Copyright IBM Corp. 2008, 2015. All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 **************************************************************************************************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.project.SavedFiltersSection");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit._base");
dojo.require("dijit.form.Form");
dojo.require("dijit.Menu");
dojo.require("dijit.form.Button");
dojo.require("dijit.Toolbar");
dojo.require("dijit.form.DropDownButton");
dojo.require("dijit.MenuItem");

dojo.require("jazz.util.html");

dojo.require("com.ibm.rdm.web.calm.CalmConstants");
dojo.require("com.ibm.rdm.web.calm.CalmLinkUtils");
dojo.require("com.ibm.rdm.web.calm.CalmRegistry");
dojo.require("com.ibm.rdm.web.common.MessageForm");
dojo.require("com.ibm.rdm.web.common.MessageHandler");
dojo.require("com.ibm.rdm.web.common.PermalinkDialog");
dojo.require("com.ibm.rdm.web.common.ProjectCache");
dojo.require("com.ibm.rdm.web.external.CsvExport");
dojo.require("com.ibm.rdm.web.model.types.RRMTypes");
dojo.require("com.ibm.rdm.web.notifications.ChangeNotifier");
dojo.require("com.ibm.rdm.web.notifications.EventTopics");
dojo.require("com.ibm.rdm.web.project.DashboardConstants");
dojo.require("com.ibm.rdm.web.project.FilterSection");
dojo.require("com.ibm.rdm.web.project.SavedFilter");
dojo.require("com.ibm.rdm.web.query.NewSavedFilterForm");
dojo.require("com.ibm.rdm.web.query.SavedFilterUtils");
dojo.require("com.ibm.rdm.web.query.stringMatchUtils");
dojo.require("com.ibm.rdm.web.server.client.RDMClient");
dojo.require("com.ibm.rdm.web.server.client.utils.LinkUtils");
dojo.require("com.ibm.rdm.web.tags.TagUtil");
dojo.require("com.ibm.rdm.web.tags.TagsCompactItemSelectionContainer");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("com.ibm.rdm.web.server.XhrUtils");
dojo.require("com.ibm.rdm.web.common.UpdateContext");
dojo.require("com.ibm.rdm.web.common.StatusDisplay");
dojo.require("com.ibm.rdm.web.query.filters.PartiallyCollapsedFilter");
dojo.require("com.ibm.rdm.web.pages.viewStateManager");
dojo.require("com.ibm.rdm.web.calm.CalmFilters");
dojo.require("com.ibm.rdm.web.server.Headers");
dojo.require("com.ibm.rdm.web.links.LinkTypeUtils");
dojo.require("dojox.collections.Dictionary");
dojo.require("dojo.i18n");
dojo.require("com.ibm.rdm.web.util.bidiutils");
dojo.requireLocalization("com.ibm.rdm.web.query", "NewSavedFilterForm");
dojo.requireLocalization("com.ibm.rdm.web.project", "TagsSection");
dojo.requireLocalization("com.ibm.rdm.web.project", "SavedFiltersSection");

var domutils = com.ibm.rdm.web.util.domutils;
var ProjectCache = com.ibm.rdm.web.common.ProjectCache;
var ChangeNotifier = com.ibm.rdm.web.notifications.ChangeNotifier;
var EventTopics = com.ibm.rdm.web.notifications.EventTopics;
var CsvExport = com.ibm.rdm.web.external.CsvExport;
var TagUtil = com.ibm.rdm.web.tags.TagUtil;
var stringMatchUtils = com.ibm.rdm.web.query.stringMatchUtils;
var SavedFilterUtils = com.ibm.rdm.web.query.SavedFilterUtils;
var RRMTypes = com.ibm.rdm.web.model.types.RRMTypes;
var BidiUtils = com.ibm.rdm.web.util.bidiutils;
var LinkUtils = com.ibm.rdm.web.server.client.utils.LinkUtils;
var CalmLinkUtils = com.ibm.rdm.web.calm.CalmLinkUtils;
var XhrUtils = com.ibm.rdm.web.server.XhrUtils;
var StatusDisplay = com.ibm.rdm.web.common.StatusDisplay;
var MessageHandler = com.ibm.rdm.web.common.MessageHandler;
var PartiallyCollapsedFilter = com.ibm.rdm.web.query.filters.PartiallyCollapsedFilter;
var viewStateManager = com.ibm.rdm.web.pages.viewStateManager;
var CalmFilters = com.ibm.rdm.web.calm.CalmFilters;
var Headers = com.ibm.rdm.web.server.Headers;
var VvcUtils = com.ibm.rdm.web.util.vvcutils;

var MIN_WIDTH_TO_USE_FOR_SEARCH_INPUT_CONTROL = 50;
var TEXT_INPUT_LEFT_SIDE_GAP = 3;
var RIGHT_HAND_SIDE_DIV_LEFT_SIDE_GAP = 6;
var TWISTY_MARGIN = 10;

dojo.declare(
	"com.ibm.rdm.web.project.SavedFiltersSection",
	com.ibm.rdm.web.project.FilterSection,
{
	// summary:
	//		Presents a section view on the left in the project dashboard to
	//		enable users to run saved filters.

	templatePath: dojo.moduleUrl("com.ibm.rdm.web", "project/templates/SavedFiltersSection.html"),
	widgetsInTemplate: true,

	// /////////////////////////////////////////////////////////////////////////////////
	// Public fields
	projectURI: null,
	configurationURI: null,
	resourceURI: null,
	baselineURI: null,
	viewType: null,
	//This is the class given as dashboardTabPage on contruction - not necessarily a DashboardTabPage!
	dashboardTabPage: null,
	currentlySelectedKeys: null,
	allowReportCreation: true,
	readOnly: false,
	allowToggleExpansion:true,
	preventCollapse: false,
	filterSelectionHasChangedNotify: null,
	callbackScope: null,
	skipCache: false, // Do not skip cache by default
	isPublicModify: false,
	isPrivateModify: false,
	// isEditableOnlyView: [const] Boolean
	//		This field is used to indicate if only saved filters the user may edit
	//		should be displayed.
	isEditableOnlyView: false,
	projectOperationsDeferred: null,

	// initialStateNeedsToBeApplied: Boolean
	//		When this widget is started up, this field ends up getting set to true
	//		if and only if either an initial view needs to be applied or a set of
	//		ad-hoc filters need to be applied.
	initialStateNeedsToBeApplied: false,

	// isContextArtifactPicker: Boolean
	//		When this widget is used in the context of artifact picking, UX has specified
	//		that the Views section within the picker dialog should attempt to deal with
	//		"dirty views", so you must pass this as 'true' during construction if
	//		this Views section is to be used in that manner.
	isContextArtifactPicker: false,

	// /////////////////////////////////////////////////////////////////////////////////
	// Private fields

	_savedFilterDiv: null,
	_publicSavedFiltersContainer: null,
	_privateSavedFiltersContainer: null,
	_sectionFiltersDeferredList: null,
	_savedFilter: null,
	_filtersChangedListeners: [],
	_customAttributes: null,
	_attributeTypes: null,
	_linkTypes: null,
	_savedFilters: null,
	_savedFilterNames: null,
	_newFilterMessages: null,
	_tagsSectionMessages: null,
	_messages: null,
	_saveButton: null,
	_filterText: null,
	_searchButton: null,
	_listDisplayMenuItem: null,
	_cloudDisplayMenuItem: null,
	_displayTypeDropDown: null,
	_displayTypeMenu: null,
	_loaded: false,
	_startupDeferred: null,
	_globalScopeView: [],
	_projectAreaLinks: null,
	_partiallyCollapsedFilterContainer: null,
	_partiallyCollapsedFilter: null,
	_doNotUpdateLocalStorage: null,
	_pageOrUriForViewStateManagerCalls: null,
	_initialApplyOfLastFiltersHasBeenConsidered: false,
	_initialViewsLoadedDeferred: null,
	_lastExpansionStateWasExpanded: false,
	
	//For lazy loading, track if we should populate the section when we expand
	_populateOnExpansion: false,
	
	
	// This attribute will ONLY hold a value when a refresh operation is in
	// progress. (Note: This attribute is null'd out in the constructor.)
	_refreshInfo: {
		// The callback to invoke when the refresh operation has completed (i.e.
		// after new values have been added to the DOM that reflect a refreshed
		// filters list).
		refreshCompletedCallback: null
	},

	// /////////////////////////////////////////////////////////////////////////////////
	// API

	constructor: function() {
		// We override the following field due to the fact that we DO want to
		// support showing this section as "partially collapsed" (i.e.
		// collapsed but with a "partially collapsed filter" still showing
		// to enable the user to see and/or remove the currently selected
		// view).
		this._doesSupportDisplayingAsPartiallyCollapsed = true;

		this._refreshInfo = null;
		this._savedFilter = null;
		this._savedFilters = new dojox.collections.Dictionary();
		this._savedFilterNames = {};
		this._filtersChangedListeners = [];
		this._projectAreaLinks = [];
		this._partiallyCollapsedFilter = null;
	},

	postCreate: function() {
		this._initPageOrUriForViewStateStateManagerCalls();
		this.fixedClearTitle = this._filterSectionMessages.defaultView;
		this.inherited("postCreate", arguments);
		
		var getCreateCollectionFunctionIfApplicable = dojo.hitch(this, function(){
			// Return our _createCollection function if and only if we're not
			// in read only mode AND we're not in a module based view.
			return (
				(
					!this.readOnly
					&& (
						!this.viewType
						|| this.viewType !== com.ibm.rdm.web.project.DashboardConstants.VIEW_TYPES.MODULE
						)
				) ?
				this._createCollection :
				null
			);
		});

		// Note: the following two assignments contain onItemMenuOpen/onItemMenuClose
		// callbacks.  These callbacks are used to "turn off" mouse hover treatment
		// in the "other" view section whenever the drop down menu for a particular
		// view has been opened.

		this._publicSavedFiltersContainer =
			new com.ibm.rdm.web.tags.TagsCompactItemSelectionContainer({
				projectURI: this.projectURI,
				onItemDirty: dojo.hitch(this, function(itemKey){
					this._addDirtyViewEntryToLocalStorage(itemKey);
				}),
				onItemNotDirty: dojo.hitch(this, this._removeDirtyViewEntryFromLocalStorage),
				viewType: com.ibm.rdm.web.tags.TagsCompactItemSelectionContainer.viewType.SavedView,
				scrollParent: this._contentPaneDiv,
				onItemSelected: dojo.hitch(this, function(itemKey){
					this._setCurrentStateForLocalStorage(itemKey);
					this._onSavedFilterSelectionChanged(itemKey);
				}),
				onItemUnselected: dojo.hitch(this, function(itemKey){
					if (this._publicSavedFiltersContainer.isItemDirty(itemKey)) {
						this._publicSavedFiltersContainer.hideDirtyItemToolbar(itemKey);
					}
					this._onSavedFilterSelectionChanged(itemKey);
				}),
				onItemRefresh: dojo.hitch(this, function(itemKey){
					this._onSavedFilterRefresh(this._publicSavedFiltersContainer, itemKey);
				}),
				onItemSave: dojo.hitch(this, function(itemKey){
					this._onSaveFilterAfterSelectingItemIfAppropriate(
						this._publicSavedFiltersContainer, this._privateSavedFiltersContainer, itemKey
					);
				}),
				callbackScope: this,
				isMultiSelectionEnabled: false,
				textItemCssClassName: "public-tag-name",
				onTagEdit: this._onEditFilter,
				onGet: (!this.readOnly) ? this.getCSV : null,
				onTagDelete: this._onDeleteFilter,
				onReport: (this.allowReportCreation)? this._onReport :null,
				onPermalink: this._onPermalink,
				onCollectionCreate: getCreateCollectionFunctionIfApplicable(),
				modifyAllowed: this.isPublicModify,
				listViewType: true,
				onItemMenuOpen: dojo.hitch(this, function(viewInformation) {
					this._privateSavedFiltersContainer.set("isItemTrackingDisabled", true);
					if (this._projectAreaLinks && this._projectAreaLinks.length > 0) {
						var savedFilter = this._savedFilters.item(viewInformation.viewURI);
						if (savedFilter && savedFilter.specificModule) {
							this._publicSavedFiltersContainer.addLinkMenuItem();
						}
					}					
				}),
				onItemMenuClose: dojo.hitch(this, function(viewInformation) {
					this._privateSavedFiltersContainer.set("isItemTrackingDisabled", false);
				})
		});
		this._toDestroy.push(this._publicSavedFiltersContainer);
		
		this._privateSavedFiltersContainer =
			new com.ibm.rdm.web.tags.TagsCompactItemSelectionContainer({
				projectURI: this.projectURI,
				onItemDirty: dojo.hitch(this, function(itemKey){
					this._addDirtyViewEntryToLocalStorage(itemKey);
				}),
				onItemNotDirty: dojo.hitch(this, this._removeDirtyViewEntryFromLocalStorage),
				viewType: com.ibm.rdm.web.tags.TagsCompactItemSelectionContainer.viewType.SavedView,
				scrollParent: this._contentPaneDiv,
				onItemSelected: dojo.hitch(this, function(itemKey){
					this._setCurrentStateForLocalStorage(itemKey);
					this._onSavedFilterSelectionChanged(itemKey);
				}),
				onItemUnselected: dojo.hitch(this, function(itemKey){
					if (this._privateSavedFiltersContainer.isItemDirty(itemKey)) {
						this._privateSavedFiltersContainer.hideDirtyItemToolbar(itemKey);
					}
					this._onSavedFilterSelectionChanged(itemKey);
				}),
				onItemRefresh: dojo.hitch(this, function(itemKey){
					this._onSavedFilterRefresh(this._privateSavedFiltersContainer, itemKey);
				}),
				onItemSave: dojo.hitch(this, function(itemKey){
					this._onSaveFilterAfterSelectingItemIfAppropriate(
						this._privateSavedFiltersContainer, this._publicSavedFiltersContainer, itemKey
					);
				}),
				callbackScope: this,
				isMultiSelectionEnabled: false,
				textItemCssClassName: "private-tag-name",
				onTagEdit: this._onEditFilter,
				onGet: (!this.readOnly) ? this.getCSV : null,
				onTagDelete: this._onDeleteFilter,
				onReport: (this.allowReportCreation)? this._onReport :null,
				onPermalink: this._onPermalink,
				onCollectionCreate: getCreateCollectionFunctionIfApplicable(),
				modifyAllowed: this.isPrivateModify,
				listViewType: true,
				onItemMenuOpen: dojo.hitch(this, function(viewInformation) {
					this._publicSavedFiltersContainer.set("isItemTrackingDisabled", true);
				}),
				onItemMenuClose: dojo.hitch(this, function(viewInformation) {
					this._publicSavedFiltersContainer.set("isItemTrackingDisabled", false);
				})
		});
		this._toDestroy.push(this._privateSavedFiltersContainer);

		this._partiallyCollapsedFilterContainer =
			this._createPartiallyCollapsedFilterContainer();
		this._toDestroy.push(this._partiallyCollapsedFilterContainer);
		
		this._savedFilterDiv = document.createElement("div");

		dojo.addClass(this._listDisplayMenuItem.iconNode, "menuItemSelectedBullet");
		
		this.projectOperationsDeferred = ProjectCache.getProjectPermissions(this.projectURI);
		this.projectOperationsDeferred.addCallback(this, function(projectOperations){
			this.isPublicModify = false;
			this.isPrivateModify = false;
			var isSnapshot = VvcUtils.isCurrentConfigurationSnapshot();
			if (!isSnapshot && !this.readOnly){
				this.isPublicModify = projectOperations.item(ProjectCache.MODIFY_PUBLIC_VIEW) && 
					projectOperations.item(ProjectCache.MODIFY_RESOURCE);
				this.isPrivateModify = projectOperations.item(ProjectCache.MODIFY_PRIVATE_QUERY);
				this._resetContentInPartiallyCollapsedFilter();
			}
			
			if (this.isPublicModify && !this.baselineURI && !isSnapshot){
				this._publicSavedFiltersContainer.enableModify();
				this._publicSavedFiltersContainer.enableCollection();
				this._privateSavedFiltersContainer.enableCollection();
				
				this._saveButton.set("disabled", false);
				this._saveButton.set("iconClass", "save");
			}
			if (this.isPrivateModify && !this.baselineURI && !isSnapshot){
				this._privateSavedFiltersContainer.enableModify();
				
				this._saveButton.set("disabled", false);
				this._saveButton.set("iconClass", "save");
			}
			
			//If the user is choosing an existing filter to modify, check permissions before loading saved filters.
			if(this.isEditableOnlyView){
				this.isPublicModify = projectOperations.item(ProjectCache.MODIFY_PUBLIC_VIEW) && 
										projectOperations.item(ProjectCache.MODIFY_RESOURCE);
				this.isPrivateModify = projectOperations.item(ProjectCache.MODIFY_PRIVATE_QUERY);
			
				if (this.isPublicModify && !this.baselineURI && !isSnapshot){
					this._savedFilterDiv.appendChild(this._publicSavedFiltersContainer.domNode);
					
					if (this.isPrivateModify){
						var separatorDiv = document.createElement("div");
						separatorDiv.className = "tag-name-separator";
						this._savedFilterDiv.appendChild(separatorDiv);
						this._savedFilterDiv.appendChild(this._privateSavedFiltersContainer.domNode);
					}
				}else {
					if (this.isPrivateModify && !this.baselineURI && !isSnapshot){
						this._savedFilterDiv.appendChild(this._privateSavedFiltersContainer.domNode);
					}
				}
			//Else load both shared and personal filters
			} else {
				this._savedFilterDiv.appendChild(this._publicSavedFiltersContainer.domNode);
				
				var separatorDiv = document.createElement("div");
				separatorDiv.className = "tag-name-separator";
				this._savedFilterDiv.appendChild(separatorDiv);
				this._savedFilterDiv.appendChild(this._privateSavedFiltersContainer.domNode);
			}
			
			//Regardless, append the savedFilters list.
			this._rootNodeForExpandedContent.appendChild(this._savedFilterDiv);

			dojo.addClass(this._savedFilterDiv, "tags-container");
			
			return projectOperations;
		});

		this.connect(this._searchButton, "onclick", this._onFilter);
		this.connect(this._filterText, "onkeypress", this._checkKey);
		BidiUtils.applyTextDirSetting(this._filterText, this._filterText.value);
		if (BidiUtils.getTextDir() === BidiUtils.TEXT_DIR_CONTEXTUAL) { 
			this.connect(this._filterText, "onkeyup", function(){
				BidiUtils.applyTextDirSetting(this._filterText, this._filterText.value);
			});
		}
		 
		this._sectionFiltersDeferredList = [];
		if (!this.readOnly){
			var addListener = ChangeNotifier.addListener({
				eventTopics: [EventTopics.FILTER_ADDED],
				callbackFunction: this._onFilterAddedNotify,
				callbackScope: this,
				projectURI: this.projectURI,
				resourceUriScope: this.resourceURI
			});
			this._filtersChangedListeners.push(addListener);
			var deleteListener = ChangeNotifier.addListener({
				eventTopics: [EventTopics.FILTER_DELETED],
				callbackFunction: this._onFilterDeletedNotify,
				callbackScope: this,
				projectURI: this.projectURI,
				resourceUriScope: this.resourceURI
			});
			this._filtersChangedListeners.push(deleteListener);
			var updateListener = ChangeNotifier.addListener({
				eventTopics: [EventTopics.FILTER_EDITED],
				callbackFunction: this._onFilterEditedNotify,
				callbackScope: this,
				projectURI: this.projectURI,
				resourceUriScope: this.resourceURI
			});
			this._filtersChangedListeners.push(updateListener);
		}
		
		this._addCreateLinkIfApplicable();

		this._initialViewsLoadedDeferred = new dojo.Deferred();

		var stateObject = this._getCurrentStateFromLocalStorage();
		if (stateObject.viewURI) {
			this.currentlySelectedKeys = [stateObject.viewURI];
			this.initialStateNeedsToBeApplied = true;
		}
		else if (stateObject.filters) {
			this.initialStateNeedsToBeApplied = true;
		}

		this.startup();
	},

	setBaseline: function(baselineURI) {
		this.baselineURI = baselineURI;
	},
	
	selectFilter: function(filterURI, /*Boolean?*/avoidShowingItemFloatingToolbar){
		if (filterURI){
			this._publicSavedFiltersContainer.selectItem(filterURI, avoidShowingItemFloatingToolbar);
			this._privateSavedFiltersContainer.selectItem(filterURI, avoidShowingItemFloatingToolbar);
		}
	},

	selectInitialFiltersIfAppropriate: function() {
		// summary:
		//		Consult local storage and try to resurrect whatever view (be it
		//		dirty or not) was last selected.  If no view was last selected,
		//		then instead try to resurrect whatever ad-hoc filters were last
		//		applied.
		// returns:
		//		true if a view or at least on ad-hoc filter was selected.

		var aViewOrAtLeastOneFilterWasApplied = false;
		if (!this._initialApplyOfLastFiltersHasBeenConsidered) {
			this._initialApplyOfLastFiltersHasBeenConsidered = true;
			var stateObject = this._getCurrentStateFromLocalStorage();
			if (stateObject.viewURI) {
				var currentViewURI = stateObject.viewURI;
				if (currentViewURI) {
					aViewOrAtLeastOneFilterWasApplied = true;
					window.setTimeout(
						dojo.hitch(this, function(){
							this._initialViewsLoadedDeferred.then(
								dojo.hitch(this, function(){
									this.selectFilter(currentViewURI);
									this._clearAndRepopulateDisplayedViews();
									window.setTimeout(
										dojo.hitch(this, function(){
											this._scrollAsNeededToShowView(
												currentViewURI, this._contentPaneDiv
											);
										}),
										250
									);
								})
							);
						}),
						1
					);
				}
			}
			else if (stateObject.filters) {
				aViewOrAtLeastOneFilterWasApplied = true;
				var filters = stateObject.filters;
				window.setTimeout(
					dojo.hitch(this, function(){
						this._initialViewsLoadedDeferred.then(
							dojo.hitch(this, function(){
								this.dashboardTabPage.recordFilterChange();
								this.dashboardTabPage.setSavedFilterValues(filters);
							})
						);
					}),
					1
				);
			}
		}
		return aViewOrAtLeastOneFilterWasApplied;
	},

	showSelectedFilterAsDirty: function() {
		if (!this.isContextArtifactPicker) {
			var isExpanded = this._isSectionExpanded;
			var selectedViewURI = this._getSelectedViewUri();
			if (selectedViewURI) {
				var view = this._savedFilters.item(selectedViewURI);
				this._publicSavedFiltersContainer.showSelectedItemAsDirty(
					!isExpanded			// value for avoidShowingItemHoverToolbar
				);
				this._privateSavedFiltersContainer.showSelectedItemAsDirty(
					!isExpanded			// value for avoidShowingItemHoverToolbar
				);
				this._partiallyCollapsedFilterContainer.showSelectedItemAsDirty(
					isExpanded			// value for avoidShowingItemHoverToolbar
				);
				this._setCurrentStateForLocalStorage(selectedViewURI);
			} else {
				this._setCurrentStateForLocalStorage();
			}
			
			this._partiallyCollapsedFilter.disableRemoveFilter();
		}
	},
	
	postMixInProperties: function() {
		this.inherited("postMixInProperties", arguments);
		this._newFilterMessages = dojo.i18n.getLocalization("com.ibm.rdm.web.query", "NewSavedFilterForm");
		this._tagsSectionMessages = dojo.i18n.getLocalization("com.ibm.rdm.web.project", "TagsSection");
		this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.project", "SavedFiltersSection");
	},
	
	startup: function() {
		if (!this._loaded) {
			this._loaded = true;
			this._startupDeferred = this.inherited("startup", arguments);
		}
		return this._startupDeferred;
	},

	uninitialize: function() {
		if (this._displayTypeMenu) {
			this._displayTypeMenu.destroyRecursive();
		}
		if (this._displayTypeDropDown) {
			this._displayTypeDropDown.destroyRecursive();
		}
		for (var i = 0; i < this._filtersChangedListeners.length; i++) {
			var listener = this._filtersChangedListeners[i];
			ChangeNotifier.removeListener(listener);
		}
		this.inherited(arguments);
	},

	getSelectedSavedFilters: function() {
		// summary:
		//		Return an array of strings where each element of the array
		//		represents the resource URI for a selected filter.
		if (this._publicSavedFiltersContainer.getSelectedKeys().length > 0) {
			return this._publicSavedFiltersContainer.getSelectedKeys();
		}
		else if (this._privateSavedFiltersContainer.getSelectedKeys().length > 0) {
			return this._privateSavedFiltersContainer.getSelectedKeys();
		}
		else {
			return [];
		}
	},
	
	getCSV: function (uri){
		var filterDeferred = this._getFilterAsRDF(uri, true);
		// We've retrieved the savedFilter in it's RDF form, here is where we will need to manipulate it...
		filterDeferred.addCallback(this, function(response){
			this._onExportFilter(response);
		});
	},
	
	getSelectedSavedFilter: function() {
		// summary:
		//		Return an array of strings where each element of the array
		//		represents the resource URI for a selected filter.
		return this.getSelectedSavedFilters();
	},

	refresh: function(refreshCompletedCallback) {
		if (this._loaded) {
			this._startRefresh(refreshCompletedCallback);
		}
		else if (refreshCompletedCallback){
			var callback = refreshCompletedCallback;
			callback();
		}
	},

	// reset all filters in reapply
	// usually called in the context of refresh
	reApplyFilters: function(/*Function?*/onComplete) {
		this._onSavedFilterSelectionChanged(this.currentlySelectedKeys, undefined, undefined, onComplete);
	},
	
	unselectAll: function() {
		this._savedFilter = null;
		this._publicSavedFiltersContainer.clearSelection();
		this._privateSavedFiltersContainer.clearSelection();
		this.currentlySelectedKeys = null;
		this._filterSelectionHasChanged();
		this._setCurrentStateForLocalStorage();
	},
	
	/*
	 * unselect all filters without refreshing the dashboard
	 */
	clearSelection: function() {
		this._savedFilter = null;
		this._publicSavedFiltersContainer.clearSelection();
		this._privateSavedFiltersContainer.clearSelection();
		this.currentlySelectedKeys = null;

		this._resetContentInPartiallyCollapsedFilter();
		if (!this.isSectionExpanded()) {
			// Tell the container that it can shrink this section and grow others
			// to fill the vertical space that used to be used by this section's
			// this._partiallyCollapsedFilter.
			this.filterSectionsContainer.filterSectionCollapsedHeightHasChanged(this);
		}
	},

	setWidth: function(width) {
		this.inherited(arguments);
		if (this._isSectionExpanded) {
			this._narrowOrWidenSearchInputControlAsAppropriate(width);
		}
		this._publicSavedFiltersContainer.setWidth(width);
		this._privateSavedFiltersContainer.setWidth(width);
		this._setPartiallyCollapsedFilterWidth(width);
	},

	setExpansionStateAndHeight: function(expansionState, height) {
		this.inherited(arguments);
		if (this._isSectionExpanded) {
			this._narrowOrWidenSearchInputControlAsAppropriate();
		}
	},
	
	isLoaded: function() {
		return this._loaded;
	},
	
	focusOnFilterTextField: function() {
		if (this._filterText) {
			this._filterText.focus();
		}
	},

	// /////////////////////////////////////////////////////////////////////////////////
	// Helpers

	// Superclass override functions ...
	
	_updateExpansionState: function(isInitializing) {
		// summary:
		//		This is an override of a superclass function.
		this.inherited("_updateExpansionState", arguments);
		if (this._isSectionExpanded) {
			this._toolBarDiv.style.display = "";
		} else {
			this._toolBarDiv.style.display = "none";
		}
		if (this._isSectionExpanded && this._populateOnExpansion) {
			this._populateOnExpansion = false;
			this._populateExpandedFilterContents();
		}
		if (this._isSectionExpanded && !isInitializing) {
			if (!this._loaded) {
				// section is being shown for first time, populate
				// the content
				this.startup();
			}
		}
	},

	_createPartiallyCollapsedFilterContainer: function() {
		var doNothingCallback = function(){};

		var partiallyCollapsedFilterContainer =
			new com.ibm.rdm.web.tags.TagsCompactItemSelectionContainer({
				projectURI: this.projectURI,
				viewType: com.ibm.rdm.web.tags.TagsCompactItemSelectionContainer.viewType.PartiallyCollapsedSavedView,
				scrollParent: this._contentPaneDiv,
				onItemSelected: doNothingCallback,
				onItemUnselected: doNothingCallback,
				onItemRefresh: dojo.hitch(this, function(itemKey){
					this._onSavedFilterRefresh(partiallyCollapsedFilterContainer, itemKey);
				}),
				onItemSave: dojo.hitch(this, function(itemKey){
					this._onSavedFilterSave(partiallyCollapsedFilterContainer, itemKey);
				}),
				callbackScope: this,
				isMultiSelectionEnabled: false,
				textItemCssClassName: "private-tag-name",
				modifyAllowed: false,
				listViewType: true,
				onItemMenuOpen: doNothingCallback,
				onItemMenuClose: doNothingCallback
			});

		// Ignore any clicks that the user might perform on the view name
		// that appears in this partially collapsed filter.
		partiallyCollapsedFilterContainer.disableChangingSelection();

		return partiallyCollapsedFilterContainer;
	},

	_resetContentInPartiallyCollapsedFilter: function() {
		if (this._partiallyCollapsedFilter) {
			var selectedViewURI = this._getSelectedViewUri();
			if (selectedViewURI) {
				var filter = this._savedFilters.item(selectedViewURI),
					tooltip = this._getFilterTooltipText(filter),
					isDirty = this._getDirtyViewJsonIfPresent(selectedViewURI);
				this._partiallyCollapsedFilterContainer.removeAllItems();
				var span = this._partiallyCollapsedFilterContainer.addTextItemWithTooltip(
					stringMatchUtils.decodeHtmlEntities(filter.title), filter.uri, tooltip, 0
				);
				var isShared = (
					filter.scope === com.ibm.rdm.web.project.SavedFilter.scope.PUBLIC
					|| filter.scope === com.ibm.rdm.web.project.SavedFilter.scope.COLLECTION_PUBLIC
					|| filter.scope === com.ibm.rdm.web.project.SavedFilter.scope.MODULE_PUBLIC
				);
				dojo.addClass(span, isShared ? "public-tag-name" : "private-tag-name");
				this._partiallyCollapsedFilterContainer.setSelectedKeys([filter.uri]);
				if (isDirty) {
					this._partiallyCollapsedFilterContainer.setItemAsDirty(selectedViewURI);
				}
				if (isShared) {
					if (this.isPublicModify) {
						this._partiallyCollapsedFilterContainer.enableModify();
					} else {
						this._partiallyCollapsedFilterContainer.disableModify();
					}
				} else {
					if (this.isPrivateModify) {
						this._partiallyCollapsedFilterContainer.enableModify();
					} else {
						this._partiallyCollapsedFilterContainer.disableModify();
					}
				}
			}
		}
	},

	_populateContents: function() {
		var deferred;
		if (this._isSectionExpanded || this.initialStateNeedsToBeApplied) {
			//Go ahead and get all the info
			deferred = this._populateExpandedFilterContents();
			this._populateOnExpansion = false;
		} else {
			this._populateOnExpansion = true;
			deferred = this._endRefreshIfNeeded();
		}
		this._populatePartiallyCollapsedFilterContents();
		return deferred;
	},

	_setPartiallyCollapsedFilterWidth: function(/*Integer?*/width) {
		if (this._partiallyCollapsedFilter) {
			// Tell the partially collapsed filter to use a new explicit width.
			// We need to do this in order to have the eraser appear at the correct
			// right hand side position even when the underlying view span's width
			// exceeds the new width.
			var widthToUse = width || this._lastWidth || dojo.position(this.domNode).w;
			if (widthToUse) {
				// Note: The '17' in the following is to account for things like margin
				// and padding that are assoicated with how the partially collapsed
				// is displayed.
				this._partiallyCollapsedFilter.setWidth(widthToUse-17);
			}
		}
	},

	_populatePartiallyCollapsedFilterContents: function() {
		if (!this._partiallyCollapsedFilter) {
			var partiallyCollapsedFilter = new PartiallyCollapsedFilter({
				viewType: PartiallyCollapsedFilter.viewType.Custom,
				onRemoveFilter: dojo.hitch(this, this._handleClearFilters)
			});
			partiallyCollapsedFilter.set("contents", this._partiallyCollapsedFilterContainer.domNode);
			this._rootNodeForPartiallyCollapsedContent.appendChild(
				partiallyCollapsedFilter.domNode);
			this._setNodeVisibility(this._rootNodeForPartiallyCollapsedContent, false);
			this._toDestroy.push(partiallyCollapsedFilter);

			this._setPartiallyCollapsedFilterWidth();

			// Note by SPC: I'm adding the following dojo.addClass calls to enable
			// supporting having the partially collapsed View section's view name
			// display with the appropriate color (i.e. blue for a shared view and
			// green for a private view).
			dojo.addClass(partiallyCollapsedFilter.domNode, "tags-container");
			dojo.addClass(partiallyCollapsedFilter.domNode, "partially-collapsed-filter");

			this._partiallyCollapsedFilter = partiallyCollapsedFilter;
		}
	},

	_populateExpandedFilterContents: function() {
		var customAttributesDeferred = com.ibm.rdm.web.model.types.TypesManager.getAttributeDefinitionsForProject({projectURI: this.projectURI, projectSnapshotURI: this.baselineURI});
		customAttributesDeferred.addCallback(this, function(response){
			this._customAttributes = {};
			for (var customAttribute in response){
				this._customAttributes[response[customAttribute].attributeDefinitionURI] = response[customAttribute];
			}
		});
		
		var attributeTypesDeferred = com.ibm.rdm.web.model.types.TypesManager.getAttributeTypesForProject({projectURI: this.projectURI, projectSnapshotURI: this.baselineURI});
		attributeTypesDeferred.addCallback(this, function(response){
			this._attributeTypes = {};
			for (var attributeType in response){
				this._attributeTypes[response[attributeType].attributeTypeURI] = response[attributeType];
			}
		});

		var linkTypesDeferred = com.ibm.rdm.web.model.types.TypesManager.getLinkTypesForProject({
			projectURI: this.projectURI,
			projectSnapshotURI: this.baselineURI
		});
		linkTypesDeferred.addCallback(this, function(linkTypesArray){
			this._linkTypes = {};
			for (var i = 0; i < linkTypesArray.length; i++) {
				var linkType = linkTypesArray[i];
				this._linkTypes[linkType.getResourceURI()] = linkType;
			}
			return linkTypesArray;
		});

		var publicItemKeyWithFocus = this._getFocussedItemKey(this._publicSavedFiltersContainer),
			privateItemKeyWithFocus = this._getFocussedItemKey(this._privateSavedFiltersContainer);
		this._clearLastResultsIfNeeded();
		this._setContentToLoadingMessage();
		var deferredArray = [];
		
		var skipCache = false;
		if (this._refreshInfo || this.skipCache){
			skipCache = true;
		}
		
		var viewsDeferred = this._getViews(SavedFilterUtils.getScope, skipCache);
		deferredArray.push(viewsDeferred);
		
		viewsDeferred.addCallback(this, function(viewsMap) {
			var entry = null,
				thisFilter = null,
				filter = null;
			
			for (entry in viewsMap){
				thisFilter = viewsMap[entry];
				filter = this._createFilter(thisFilter);
				this._savedFilters.add(filter.uri, filter);
				this._savedFilterNames[filter.title] = viewsMap[entry].resourceURI;
			}
		});
		
		var modDeferred = this._getModuleViews(
				com.ibm.rdm.web.project.SavedFilter.scope.MODULE, skipCache);
		deferredArray.push(modDeferred);
		
		modDeferred.then(dojo.hitch(this, function(moduleMap){
			var entry = null,
				thisFilter = null,
				filter = null;
			
			for (entry in moduleMap){
				thisFilter = moduleMap[entry];
				filter = this._createFilter(thisFilter, true);
				this._savedFilters.add(filter.uri, filter);
				this._savedFilterNames[filter.title] = moduleMap[entry].resourceURI;
			}
		}));
					
		var deferredList = new dojo.DeferredList(deferredArray);
		deferredList.then(dojo.hitch(this, function(){
			this._setContentToSavedFiltersResults();
			this._populateSections(publicItemKeyWithFocus, privateItemKeyWithFocus);
			if (this._initialViewsLoadedDeferred.fired === -1) {
				this._initialViewsLoadedDeferred.resolve();
			}
			this._endRefreshIfNeeded();
		}));

		return deferredList;
	},

	_getFocussedItemKey: function(compactItemSelectionContainer) {
		var focussedItemSpan = compactItemSelectionContainer.get("lastSpanNodeWithFocus");

		if (focussedItemSpan) {
			return compactItemSelectionContainer.getItemKeyForSpan(focussedItemSpan);
		} else {
			return null;
		}
	},
	
	_getViews: function(func, skipCache) {
		var scope = func(this.viewType);
		var queryURI = SavedFilterUtils.getQueryURI(scope, this.baselineURI);
		return ProjectCache.getSavedFilters(queryURI, this.projectURI, skipCache, this.configurationURI);
	},
	
	_getModuleViews: function(scope, skipCache) {
		if (!this.viewType || this.viewType !== com.ibm.rdm.web.project.DashboardConstants.VIEW_TYPES.MODULE) {
			var modPubDeferred = new dojo.Deferred();
			modPubDeferred.callback({});
			return modPubDeferred;
		}
		var queryURI = SavedFilterUtils.getQueryURI(scope, this.baselineURI, this.resourceURI);
		
		return ProjectCache.getSavedFilters(queryURI, this.projectURI, skipCache);
	},
	
	_createFilter: function(filter, specificModule){
		return {
			title: (filter.title.indexOf("\\Q") === 1) ?
				stringMatchUtils.javaRegexpEncodedInputTextToInputText(filter.title) :
				filter.title,
			description: (filter.description.indexOf("\\Q") === 1) ?
				stringMatchUtils.javaRegexpEncodedInputTextToInputText(filter.description) :
				filter.description,
			scope: filter.scope,
			specificModule: specificModule ? true : false,
			uri: filter.resourceURI.split('?')[0], // Strip out query params if they exist
			etag: filter.etag
		};
	},

	_getFilterTooltipText: function(filter) {
		var filterTooltip = filter.title +  " (";

		if (filter.scope === com.ibm.rdm.web.project.SavedFilter.scope.PUBLIC ||
			filter.scope === com.ibm.rdm.web.project.SavedFilter.scope.COLLECTION_PUBLIC ||
			filter.scope === com.ibm.rdm.web.project.SavedFilter.scope.MODULE_PUBLIC) {
			filterTooltip += this._newFilterMessages["public"];
		} else {
			filterTooltip += this._newFilterMessages["private"];
		}
		if (!filter.specificModule && this.viewType !== null){
			if (this.viewType === com.ibm.rdm.web.project.DashboardConstants.VIEW_TYPES.MODULE){
				filterTooltip += this._newFilterMessages["allModules"];
				this._globalScopeView.push(filter.uri);
			}
		}
		
		filterTooltip += ")";
		return stringMatchUtils.decodeHtmlEntities(filterTooltip);
	},
	
	_populateSections: function(publicItemKeyWithFocus, privateItemKeyWithFocus) {
		// make sure any previous results are cleared
		this._publicSavedFiltersContainer.removeAllItems();
		this._privateSavedFiltersContainer.removeAllItems();

		var filters = this._getSortedFilters();
		var filterLength = filters.length;
		var publicFilterCounter=0;
		var privateFilterCounter=0;
		var tabIndex;
		this._doNotUpdateLocalStorage = true;
		var doRestoreOriginalPublicFocusItem =
				publicItemKeyWithFocus && this._savedFilters.item(publicItemKeyWithFocus),
			doRestoreOriginalPrivateFocusItem =
				privateItemKeyWithFocus && this._savedFilters.item(privateItemKeyWithFocus);
		for (var i = 0; i < filterLength; i++ ) {
			var filter = filters[i];
			var filterItem = null;
			var filterTooltip = this._getFilterTooltipText(filter);
			if (filter.description != " ") {
				filterTooltip += (" - " + jazz.util.html.unescape(filter.description));
			}
			if (filter.scope === com.ibm.rdm.web.project.SavedFilter.scope.PUBLIC || 
					filter.scope === com.ibm.rdm.web.project.SavedFilter.scope.COLLECTION_PUBLIC || 
					filter.scope === com.ibm.rdm.web.project.SavedFilter.scope.MODULE_PUBLIC) {
				// Tab control access goes to either the last focussed item, or if not set, the first item.
				tabIndex = (
					doRestoreOriginalPublicFocusItem ?
					(filter.uri === publicItemKeyWithFocus ? 0 : -1) :
					(publicFilterCounter === 0 ? 0 : -1)
				);
				filterItem = this._publicSavedFiltersContainer.addTextItemWithTooltip(stringMatchUtils.decodeHtmlEntities(filter.title), filter.uri, filterTooltip, tabIndex);
				if (this._getDirtyViewJsonIfPresent(filter.uri)) {
					this._publicSavedFiltersContainer.setItemAsDirty(filterItem);
				}
				publicFilterCounter++;
			} else {
				// Tab control access goes to either the last focussed item, or if not set, the first item.
				tabIndex = (
					doRestoreOriginalPrivateFocusItem ?
					(filter.uri === privateItemKeyWithFocus ? 0 : -1) :
					(privateFilterCounter === 0 ? 0 : -1)
				);
				filterItem = this._privateSavedFiltersContainer.addTextItemWithTooltip(stringMatchUtils.decodeHtmlEntities(filter.title), filter.uri, filterTooltip, tabIndex);
				if (this._getDirtyViewJsonIfPresent(filter.uri)) {
					this._privateSavedFiltersContainer.setItemAsDirty(filterItem);
				}
				privateFilterCounter++;
			}
		}
		this._doNotUpdateLocalStorage = false;
	},

	_handleClearFilters: function() {
		// summary:
		//		This is an override of a superclass function.

		this._publicSavedFiltersContainer.clearSelection();
		this._privateSavedFiltersContainer.clearSelection();
		this.currentlySelectedKeys = null;
		this._removeDirtyViewIfPresent(this._publicSavedFiltersContainer);
		this._removeDirtyViewIfPresent(this._privateSavedFiltersContainer);
		this._savedFilterSelectionHasChanged();
	},

	_isAtLeastOneFilterSpecified: function() {
		// summary:
		//		This is an override of a superclass function.

		return (this.getSelectedSavedFilters().length > 0);
	},
	
	_updateClearFilterState: function() {
		this._clearFilterButton.set("enabled", true);
	},
	
	// Normal (non-override) functions ...

	_narrowOrWidenSearchInputControlAsAppropriate: function() {
		
		var self = this;
		
		var getWidthForSearchInput = function(){
			var widthOfRightHandDiv = dojo.marginBox(self._rightHandSideDiv).w;
			var totalInnerDropDownButtonWidth = self._getTotalSizeOfInnerDropDownButtonsInToolbar();				
			var potentialInputSize = widthOfRightHandDiv - totalInnerDropDownButtonWidth;
			var widthToUseForSearchInput = Math.max(MIN_WIDTH_TO_USE_FOR_SEARCH_INPUT_CONTROL, 
												potentialInputSize - TEXT_INPUT_LEFT_SIDE_GAP - TWISTY_MARGIN);
			
			return widthToUseForSearchInput;
		};
		
		var widthToUseForSearchInput = getWidthForSearchInput();
		var isOperatorDropDownOnSecondRow = this._displayTypeDropDown.domNode.parentNode === this._secondRowDiv;
		
		this._searchWrapper.style.width = (widthToUseForSearchInput + "px");
		this._rightHandSideDiv.style.left =
			((TEXT_INPUT_LEFT_SIDE_GAP + widthToUseForSearchInput + RIGHT_HAND_SIDE_DIV_LEFT_SIDE_GAP)
			 + "px");
	},
	
	_getTotalSizeOfInnerDropDownButtonsInToolbar: function(){
		// Utility method to determine the entire pixel width size
		// required to display the elements in the saved filters section header
		// (1) Display Type
		// (2) Saved Filter icon
		
		var totalWidthSize=0;
		var toolbarChildren = this._saveFilterstoolbar.getChildren();
		for(var i=0; i < toolbarChildren.length; i++){
			var child = toolbarChildren[i];
			totalWidthSize += dojo.marginBox(child.domNode).w;
		}
		return totalWidthSize;	
	},

	_clearAndRepopulateDisplayedViews: function() {
		// summary:
		//		This method is called whenever the user switches between list
		//		and cloud view.
		// description:
		//		Clear the contents of this section and redraw it using whatever
		//		layout scheme is currently in use ("list" or "cloud").
		//		In addition, make sure that the the last selected view is
		//		preserved (i.e. appears as selected after the redraw) and that
		//		its "dirtiness" is preserved (i.e. if the selected view was
		//		dirty before calling this method, it should still appear as
		//		dirty after this function returns).

		var selectedPublicKey = null,
			selectedPrivateKey = null,
			isSelectedKeyDirty = false;

		var selectItemAndMaybeShowAsDirty = function(
			compactItemSelectionContainer, itemKey, isDirty)
		{
			compactItemSelectionContainer.setSelectedKeys([itemKey]);
			if (isDirty) {
				compactItemSelectionContainer.showSelectedItemAsDirty();
			}
		};

		if (this._publicSavedFiltersContainer.getSelectedKeys().length > 0) {
			selectedPublicKey = this._publicSavedFiltersContainer.getSelectedKeys()[0];
			isSelectedKeyDirty = this._publicSavedFiltersContainer.isItemDirty(selectedPublicKey);
		}
		else if (this._privateSavedFiltersContainer.getSelectedKeys().length > 0) {
			selectedPrivateKey = this._privateSavedFiltersContainer.getSelectedKeys()[0];
			isSelectedKeyDirty = this._privateSavedFiltersContainer.isItemDirty(selectedPrivateKey);
		}

		this._populateSections();

		if (selectedPublicKey) {
			selectItemAndMaybeShowAsDirty(
				this._publicSavedFiltersContainer, selectedPublicKey, isSelectedKeyDirty
			);
		}
		else if (selectedPrivateKey) {
			selectItemAndMaybeShowAsDirty(
				this._privateSavedFiltersContainer, selectedPrivateKey, isSelectedKeyDirty
			);
		}
	},
	
	_onCloudDisplayClick: function(event) {
		if (!dojo.hasClass(this._cloudDisplayMenuItem.iconNode, "menuItemSelectedBullet")) {
			dojo.removeClass(this._listDisplayMenuItem.iconNode, "menuItemSelectedBullet");
			dojo.addClass(this._cloudDisplayMenuItem.iconNode, "menuItemSelectedBullet");
			this._publicSavedFiltersContainer.setDisplayMode(com.ibm.rdm.web.tags.display.DISPLAY_CLOUD);
			this._privateSavedFiltersContainer.setDisplayMode(com.ibm.rdm.web.tags.display.DISPLAY_CLOUD);
			this._clearAndRepopulateDisplayedViews();
		}
	},

	_onListDisplayClick: function(event) {
		if (!dojo.hasClass(this._listDisplayMenuItem.iconNode, "menuItemSelectedBullet")) {
			dojo.removeClass(this._cloudDisplayMenuItem.iconNode, "menuItemSelectedBullet");
			dojo.addClass(this._listDisplayMenuItem.iconNode, "menuItemSelectedBullet");
			this._publicSavedFiltersContainer.setDisplayMode(com.ibm.rdm.web.tags.display.DISPLAY_LIST);
			this._privateSavedFiltersContainer.setDisplayMode(com.ibm.rdm.web.tags.display.DISPLAY_LIST);
			this._clearAndRepopulateDisplayedViews();
		}
	},

	_onSavedFilterSelectionChanged: function(itemKey, editMode, avoidClearingSelection, /*Function?*/onComplete) {
		// summary:
		//		This function is called whenever an item becomes selected or
		//		unselected.
		
		if (this.dashboardTabPage) {
			this.dashboardTabPage.recordFilterChange();
		}
		if (!avoidClearingSelection) {
			if (this._publicSavedFiltersContainer.containsKey(itemKey)){
				this._privateSavedFiltersContainer.clearSelection();
			}
			else if (this._privateSavedFiltersContainer.containsKey(itemKey)){
				this._publicSavedFiltersContainer.clearSelection();
			}
		}
			
		this._savedFilter = null;
		if (!this.preventCollapse) {
			this._savedFilterSelectionHasChanged(onComplete, editMode);
		}
		else{
			var currentItem = this._savedFilters.item(itemKey);
			if (this.filterSelectionHasChangedNotify && this.callbackScope){
				var callbackFunction = dojo.hitch(this.callbackScope, this.filterSelectionHasChangedNotify);
				callbackFunction(currentItem);
				if (onComplete) {
					onComplete();
				}
			}
		}
	},

	_onSavedFilterRefresh: function(compactItemSelectionContainer, itemKey) {
		var setItemAsNotDirtyIfPresent = function(itemKey, container){
			if (container.isItemPresent(itemKey)) {
				container.setItemAsNotDirty(itemKey);
			}
		};
		setItemAsNotDirtyIfPresent(itemKey, this._publicSavedFiltersContainer);
		setItemAsNotDirtyIfPresent(itemKey, this._privateSavedFiltersContainer);
		setItemAsNotDirtyIfPresent(itemKey, this._partiallyCollapsedFilterContainer);
		this._partiallyCollapsedFilter.enableRemoveFilter();
		this._onSavedFilterSelectionChanged(itemKey, false, true);
	},

	_removeDirtyViewIfPresent: function(compactItemSelectionContainer, /*String?*/itemKeyToKeep) {
		if (!this.isContextArtifactPicker) {
			var dirtyItemKeys = compactItemSelectionContainer.getDirtyKeys();
			dojo.forEach(
				dirtyItemKeys,
				function(itemKey){
					if (itemKey !== itemKeyToKeep) {
						compactItemSelectionContainer.setItemAsNotDirty(itemKey);
					}
				}
			);
		}
	},

	_initPageOrUriForViewStateStateManagerCalls: function() {
		var valueToUse = "",
			baseURIValue = this.baselineURI ? this.baselineURI : this.projectURI,
			viewTypes = com.ibm.rdm.web.project.DashboardConstants.VIEW_TYPES;

		switch (this.viewType) {
		case viewTypes.ARTIFACTS:
			valueToUse = baseURIValue + "#artifacts";
			break;
		case viewTypes.REQUIREMENTS:
			valueToUse = baseURIValue + "#requirements";
			break;
		case viewTypes.RESOURCES:
			valueToUse = baseURIValue + "#resources";
			break;
		case viewTypes.COLLECTIONS:
			valueToUse = baseURIValue + "#collections";
			break;
		case viewTypes.MODULES:
			valueToUse = baseURIValue + "#modules";
			break;
		case viewTypes.COLLECTION:
		case viewTypes.MODULE:
			baseURIValue = this.baselineURI ? this.baselineURI + "#" : "";
			valueToUse = baseURIValue + this.resourceURI;
			break;
		}

		this._pageOrUriForViewStateManagerCalls = valueToUse;
	},

	_setCurrentStateForLocalStorage: function(/*String?*/viewURI) {
		if (!this._doNotUpdateLocalStorage && !this.isContextArtifactPicker) {
			viewStateManager.setCurrentViewOrFiltersEntry(
				(viewURI ?
				 { viewURI: viewURI } :
				 { filters: this.dashboardTabPage.getCurrentViewAsJson() }
				),
				this._pageOrUriForViewStateManagerCalls
			);
		}
	},

	_changeCalmPropertyValueIfPresent: function(viewAsJson) {
		// summary:
		//		If 'viewAsJson' has a 'calm' property, change this property to NOT
		//		be just an array of straight JSON object, but instead to be an array
		//		of CalmFilter object instances.
		// description:
		//		Unfortunately, unlike all the other properties found in the object
		//		returned by the 'parseFilterFromJson' method in SavedFilterUtils.js,
		//		the 'calm' property is supposed to represent object classes rather
		//		that just straight JSON objects.  We account for this fact by making
		//		sure that this method does the appropriate conversion of the values
		//		found in the 'calm' property array (from JSON objects to CalmFilter
		//		object instances).

		if (viewAsJson.calm) {
			viewAsJson.calm = dojo.map(
				viewAsJson.calm,
				dojo.hitch(this, function(calmJsonObject){
					var linkTypeURI = CalmFilters.getLinkTypeUriForQueryId(calmJsonObject._query);
					return CalmFilters.getCalmFilterForLinkTypeAndFilter(
						linkTypeURI, calmJsonObject._filterId
					);
				})
			);
		}
	},

	_promoteIsoStringsToDates: function(attributeConditionInfos) {
		var i;
		for (i = 0; i < attributeConditionInfos.length; i++) {
			var conditionInfo = attributeConditionInfos[i];
			if (conditionInfo.attributeType === RRMTypes.ATTRIBUTE_TYPES.DateTimeType
				|| conditionInfo.attributeType === RRMTypes.ATTRIBUTE_TYPES.DateType
				|| conditionInfo.attributeType === RRMTypes.ATTRIBUTE_TYPES.TimeType)
			{
				var attributeValues = conditionInfo.attributeValues;
				if (attributeValues.length >= 2 && attributeValues[1]) {
					attributeValues[1] = dojo.date.stamp.fromISOString(attributeValues[1]);
				}
				if (attributeValues.length === 3 && attributeValues[2]) {
					attributeValues[2] = dojo.date.stamp.fromISOString(attributeValues[2]);
				}
			}
		}
	},

	_getCurrentStateFromLocalStorage: function() {
		if (this.isContextArtifactPicker) {
			return {};
		} else {
			var currentStateObject = viewStateManager.getCurrentViewOrFiltersEntry(
				this._pageOrUriForViewStateManagerCalls
			);
			if (currentStateObject && currentStateObject.filters) {
				this._changeCalmPropertyValueIfPresent(currentStateObject.filters);
			}
			if (currentStateObject && currentStateObject.attributeConditionInfos) {
				this._promoteIsoStringsToDates(currentStateObject.attributeConditionInfos);
			}
			return currentStateObject;
		}
	},

	_addDirtyViewEntryToLocalStorage: function(viewURI) {
		if (!this._doNotUpdateLocalStorage && !this.isContextArtifactPicker) {
			viewStateManager.addDirtyViewEntry(
				viewURI,
				this.dashboardTabPage.getCurrentViewAsJson(),
				this._pageOrUriForViewStateManagerCalls
			);
		}
	},

	_removeDirtyViewEntryFromLocalStorage: function(viewURI) {
		if (!this._doNotUpdateLocalStorage && !this.isContextArtifactPicker) {
			viewStateManager.removeDirtyViewEntry(
				viewURI,
				this._pageOrUriForViewStateManagerCalls
			);
		}
	},

	_getDirtyViewJsonIfPresent: function(viewURI) {
		if (this.isContextArtifactPicker) {
			return null;
		} else {
			var viewAsJson = viewStateManager.getDirtyViewJsonIfPresent(
				viewURI, this._pageOrUriForViewStateManagerCalls
			);
			if (viewAsJson) {
				this._changeCalmPropertyValueIfPresent(viewAsJson);
			}
			if (viewAsJson && viewAsJson.attributeConditionInfos) {
				this._promoteIsoStringsToDates(viewAsJson.attributeConditionInfos);
			}
			return viewAsJson;
		}
	},

	_onSaveFilterAfterSelectingItemIfAppropriate: function(compactItemSelectionContainer,
														   otherCompactItemSelectionContainer,
														   itemKey)
	{
		// summary:
		//		The user has just selected the Save button for a dirty view.
		//		Take steps to appropriately save this view.
		// description:
		//		Before attempting to save the dirty view, we first have to make
		//		sure that the filters that are currently in place actually represent
		//		the filters associated with the dirty view represented by 'itemKey'.
		//		If this item is currently selected, we're all set.  But if it's NOT
		//		currently selected (instead the user hovered over the dirty item to
		//		invoke the Save button), we must first select the filter, wait for
		//		the filters to appropriately populate themselves and THEN save the
		//		view.

		if (!compactItemSelectionContainer.isItemSelected(itemKey)) {
			otherCompactItemSelectionContainer.clearSelection();
			compactItemSelectionContainer.clearSelection();
			compactItemSelectionContainer.selectItem(itemKey, false, true);
			this.dashboardTabPage.recordFilterChange();
			this._savedFilterSelectionHasChanged(
				dojo.hitch(this, function(){
					compactItemSelectionContainer.setFocusedSpanNode(
						compactItemSelectionContainer.getItemSpanForKey(itemKey)
					);
					this._onSavedFilterSave(compactItemSelectionContainer, itemKey);
				})
			);
		} else {
			this._onSavedFilterSave(compactItemSelectionContainer, itemKey);
		}
	},

	_onSavedFilterSave: function(compactItemSelectionContainer, itemKey) {
		var view = this._savedFilters.item(itemKey),
			title = domutils.getXMLEncodedText(view.title),
			description = domutils.getXMLEncodedText(view.description),
			scope = view.scope,
			uri = view.uri,
			specificModule = view.specificModule,
			etag = view.etag,
			postData = this.dashboardTabPage.getCurrentViewPostData({
				title: title,
				description: (description.length > 0) ? description : "",
				scope: scope,
				gridColumns: true,
				calmFilters: true,
				singleModuleScope: (
					this.viewType === com.ibm.rdm.web.project.DashboardConstants.VIEW_TYPES.MODULE
					&& specificModule
				)
			});
		var updateViewEtag = function(view, response) {
			var etag = XhrUtils.getResponseEtag(response);
			if (etag && etag.indexOf('"') === 0) {
				etag = etag.substring(1, etag.length-1);
			}
			view.etag = etag;
		};
		StatusDisplay.showStatusMessage(this._messages.savingView);
		com.ibm.rdm.web.server.client.RDMClient.doPut({
			uri: uri,
			projectURI: this.projectURI,
			postData: postData,
			skipErrors: true,
			xhrArgs: {
				headers : {
					"Content-Type": "application/rdf+xml",
					"If-Match": etag
				}
			}
		}).then(
			dojo.hitch(this, function(response){
				updateViewEtag(view, response);
				com.ibm.rdm.web.common.UpdateContext.cachedArtifactHasBeenSaved();
				compactItemSelectionContainer.setItemAsNotDirty(itemKey);
				this._partiallyCollapsedFilterContainer.setItemAsNotDirty(itemKey);
				StatusDisplay.hideStatusMessage();
				ChangeNotifier.notifyListeners({
					eventTopic: com.ibm.rdm.web.notifications.EventTopics.FILTER_EDITED,
					response: response,
					projectURI: this.projectURI,
					resourceUriScope: this.resourceURI
				});
				this._removeDirtyViewEntryFromLocalStorage(itemKey);
				this._partiallyCollapsedFilter.enableRemoveFilter();
				
				//if we are saving a filter which WAS under edit, disable special edit mode and then query.
				if (this.dashboardTabPage.getFilterEditMode()){
					this.dashboardTabPage.setFilterEditMode(false);
					this.dashboardTabPage.refresh();
				}
			}),
			dojo.hitch(this, function(errorResponse){
				StatusDisplay.hideStatusMessage();
				if (errorResponse.xhr.status === 412) {
					// Conflict - some other user has saved this view after we
					// originally obtained it (but before we tried to save it).
					updateViewEtag(view, errorResponse);
					com.ibm.rdm.web.common.UpdateContext.cachedArtifactHasBeenSaved();
					MessageHandler.showWarning(7559);
				} else {
					MessageHandler.showXhrError(
						errorResponse, errorResponse.ioArgs
					);
				}
				this._removeDirtyViewEntryFromLocalStorage(itemKey);
				this._partiallyCollapsedFilter.enableRemoveFilter();
			})
		);
	},

	_setContentToLoadingMessage: function() {
		if (this._rootNodeForExpandedContent.firstChild === this._savedFilterDiv) {
			this._rootNodeForExpandedContent.removeChild(this._savedFilterDiv);
		}
		this._rootNodeForExpandedContent.innerHTML = "";
		var messageToDisplay = this._commonMessages["loading"];
		this._rootNodeForExpandedContent.innerHTML = (
			"<span class=\"loading-message\">"
			+ messageToDisplay
			+ "</span>"
		);
	},

	_setContentToSavedFiltersResults: function() {
		if (this._rootNodeForExpandedContent.firstChild !== this._savedFilterDiv) {
			this._rootNodeForExpandedContent.removeChild(this._rootNodeForExpandedContent.firstChild);
			this._rootNodeForExpandedContent.appendChild(this._savedFilterDiv);
		}
	},

	_startRefresh: function(refreshCompletedCallback) {
		// To handle the case of a refresh being asked for before the last one
		// has completed, we'll ignore subsequent requests if one is currently
		// under way.
		if (!this._refreshInfo) {
			this.currentlySelectedKeys = this.getSelectedSavedFilters();
			// reset refresh flags
			this._refreshInfo = {
				refreshCompletedCallback: refreshCompletedCallback
			};
			this._populateContents();
		}
	},
	
	_createCollection: function (uri){
		var filterDeferred = this._getFilterAsRDF(uri, true);
		// We've retrieved the savedFilter in it's RDF form, here is where we will need to manipulate it...
		filterDeferred.addCallback(this, function(response){
			var artifactFormatURI = RRMTypes.getWellKnownAttributeURI(
						this.projectURI, 
						RRMTypes.WELL_KNOWN_ATTRIBUTE_NAMES.ArtifactFormat);
			
			var viewToPost = this._injectAttributeUris(response, [artifactFormatURI]);
			
			var viewDeferred = com.ibm.rdm.web.query.ViewServiceClient.getViewContents(
			{
				projectURI: this.projectURI,
				postData: viewToPost,
				requestedAttributeUris: [artifactFormatURI],
				displayLinks: false
			});
			viewDeferred.addCallback(this, function(viewResponse){
				
				var collectionFormat = com.ibm.rdm.web.model.types.RRMTypes.getArtifactFormats(this.projectURI).Collection;
				var viewArtifacts = com.ibm.rdm.web.query.ViewServiceClient.parseViewContentsResponse(
						viewResponse, this.projectURI, {}, false);
				
				var resourceURIs = [];
				for (var resource in viewArtifacts.resources) {
						if (viewArtifacts.resources[resource].getEntityType() != collectionFormat){
							resourceURIs.push(viewArtifacts.resources[resource].getResourceURI());
						}
				}
				
				var selectedFolder = this.dashboardTabPage.getSelectedFolder();
		
				var newArtifactDialog = new com.ibm.rdm.web.project.NewArtifactForm({
					artifactFormat: collectionFormat,
					typeURI: null,
					disableFormatsDropdown: true,
					formatsFilter: dojo.hitch(this, function(formatURI){
						return formatURI === collectionFormat;
					}),
					projectURI: this.projectURI,
					folderURI: selectedFolder,
					initialProperties: {
						"http://jazz.net/xmlns/alm/rm/Collection/v1.0/artifacts": resourceURIs
					}
				});
				this._toDestroy.push(newArtifactDialog);
				
				newArtifactDialog.show();
				return viewResponse;
			});
			
			return response;
		});
		return filterDeferred;
	},

	_endRefreshIfNeeded: function() {
		if (this._refreshInfo) {
			if (this.currentlySelectedKeys && this.currentlySelectedKeys.length > 0){
				this._publicSavedFiltersContainer.setSelectedKeys(this.currentlySelectedKeys);
				this._privateSavedFiltersContainer.setSelectedKeys(this.currentlySelectedKeys);
			}
			if (dojo.isFunction(this._refreshInfo.refreshCompletedCallback)){
				this._refreshInfo.refreshCompletedCallback();
			}
			this._refreshInfo = null;
			
		}
	},
	
	_clearLastResultsIfNeeded: function() {
		this._publicSavedFiltersContainer.removeAllItems();
		this._privateSavedFiltersContainer.removeAllItems();
		this._savedFilters.clear();
		delete this._savedFilterNames;
		this._savedFilterNames = {};
	},
	
	_getFilterAsRDF: function(uri) {
		var rdfDeferred = new dojo.Deferred();
		
		var jsonDeferred = com.ibm.rdm.web.server.client.RDMClient.doGet({
			uri: uri,
			xhrArgs:{
				handleAs: "json",
				headers:{
					Accept: "application/json"
				}
			},
			skipCache: true
		});
		
		jsonDeferred.then(dojo.hitch(this, function(jsonResponse){
			var parsedFilter = SavedFilterUtils.parseFilterFromJson(
				jsonResponse.doc, 
				uri,
				this.projectURI, 
				this._customAttributes, 
				this._attributeTypes,
				this._linkTypes
			);
			if(this.viewType != null){
				if (this.viewType === com.ibm.rdm.web.project.DashboardConstants.VIEW_TYPES.MODULE){
					parsedFilter.moduleURI = this.resourceURI;
				}
				else if (this.viewType === com.ibm.rdm.web.project.DashboardConstants.VIEW_TYPES.COLLECTION){
					parsedFilter.collectionURI = this.resourceURI;
					parsedFilter.isCoreView = true;
				} else {
					parsedFilter.isCoreView = true;
				}
			}
			
			parsedFilter.projectURI = this.projectURI;
			
			parsedFilter.requestedAttributeUris = this.dashboardTabPage.determineDesiredCols(parsedFilter.columns);
			
			if (this.viewType === com.ibm.rdm.web.project.DashboardConstants.VIEW_TYPES.MODULE) {
				var requestedAttributeUris = parsedFilter.requestedAttributeUris;
				requestedAttributeUris.push(RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Depth);
				requestedAttributeUris.push(RRMTypes.WELL_KNOWN_IS_HEADING);
				requestedAttributeUris.push(RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Section);
			}
			if (!parsedFilter.orderBy || parsedFilter.orderBy.length == 0) {
				parsedFilter.orderBy = this.dashboardTabPage.getOrderByAttributes();
			}
			
			var viewArgs = {
				gridColumns: parsedFilter
			};
			var rdfObject = com.ibm.rdm.web.query.ViewServiceClient.composeViewsPostData(parsedFilter, viewArgs);
			
			rdfObject = dojox.xml.parser.parse(rdfObject);
			rdfDeferred.callback(rdfObject);
		}));
		
		return rdfDeferred;
	},
	
	_injectAttributeUris: function(view, attributeURIs){
		var selectNodes = domutils.getElementsByTagNameNS(
				view, 
				"select", 
				"http://www.ibm.com/xmlns/rdm/rql/", 
				"rql"
			);
		if (selectNodes.length > 0){
			var buf = [];
			
			for (var i = 0; i < attributeURIs.length; i++) {
				var attribUri = attributeURIs[i];
				var newSelectElement = (dojo.isIE < 9) ? view.createElement('rdf:_' + (i+2)) :
							view.createElementNS("http://www.w3.org/1999/02/22-rdf-syntax-ns#",'rdf:_'+(i+2));
				(dojo.isIE < 9) ? newSelectElement.setAttribute('rdf:parseType', 'Resource') : 
							newSelectElement.setAttributeNS("http://www.w3.org/1999/02/22-rdf-syntax-ns#", 'rdf:parseType', 'Resource');
				var newObject = (dojo.isIE < 9) ? view.createElement("rql:object") :
							view.createElementNS("http://www.ibm.com/xmlns/rdm/rql/", "rql:object");
				(dojo.isIE < 9) ? newObject.text = "R1" : newObject.textContent = "R1";
				var newField = (dojo.isIE < 9) ? view.createElement("rql:field") : 
							view.createElementNS("http://www.ibm.com/xmlns/rdm/rql/", "rql:field");
				(dojo.isIE < 9) ? newField.setAttribute('rdf:resource', attribUri) : 
							newField.setAttributeNS("http://www.w3.org/1999/02/22-rdf-syntax-ns#", 'rdf:resource', attribUri);
				newSelectElement.appendChild(newObject);
				newSelectElement.appendChild(newField);
				selectNodes[0].appendChild(newSelectElement);
			}
		}
		
		return dojox.xml.parser.innerXML(view);
	},

	_savedFilterSelectionHasChanged: function(/*Callback?*/onComplete, editMode) {
		if (!this.isContextArtifactPicker) {
			this.dashboardTabPage.setFilterEditMode(editMode);
		}
		
		this._resetContentInPartiallyCollapsedFilter();
		this._savedFilter = null;
		this._filterSelectionHasChanged();
		var savedFilters = this.getSelectedSavedFilters();
		if (savedFilters.length > 0) {
			this.currentlySelectedKeys = savedFilters;
			var itemKey = this.currentlySelectedKeys[0],
				filter = this._savedFilters.item(itemKey),
				dirtyViewAsJson = this._getDirtyViewJsonIfPresent(itemKey);
			if (dirtyViewAsJson) {
				window.setTimeout(
					dojo.hitch(this, function(){
						this.dashboardTabPage.setSavedFilterValues(dirtyViewAsJson, dojo.hitch(this, function(){
							this.showSelectedFilterAsDirty();
							if (onComplete) {
								onComplete();
							}
						}));
					}),
					1
				);
			} else {
				var xhrArgs = {
					handleAs: "json",
					headers:{
						"Accept": "application/json"
					}
				};
				
				if ( this.configurationURI) {
					xhrArgs.headers[Headers.VVC_CONFIGURATION] = this.configurationURI;
				}
				var deferred = com.ibm.rdm.web.server.client.RDMClient.doGet({
					uri: filter.uri,
					xhrArgs:xhrArgs
				});
				var errorHandler = function() {
					MessageHandler.showError(7236);
				};
				var isOptedIn = ProjectCache.getCmState(this.projectURI);
				var savedFilter = null;
				var parseJSONAndUpdateUI = dojo.hitch(this, function() {
					var parsedFilter = SavedFilterUtils.parseFilterFromJson(
						savedFilter.doc,
						filter.uri,
						this.projectURI,
						this._customAttributes,
						this._attributeTypes,
						this._linkTypes
					);
					
					if (isOptedIn) { 
						var filteredAttributes = [];
						// we need to filter out the CLM link types when opted-in. 
						for (var i = 0; i < parsedFilter.linkedArtifactConditionInfos.length; i++) {
							filteredAttributes = [];
							var currentLinkedInfo = parsedFilter.linkedArtifactConditionInfos[i];
							for (var j = 0; j < currentLinkedInfo.linkConditionInfo.attributeValues.length; j++) { 

								var linkTypeURI = currentLinkedInfo.linkConditionInfo.attributeValues[j];
								linkTypeURI = linkTypeURI.substring(0, linkTypeURI.lastIndexOf("#"));

								if (com.ibm.rdm.web.links.LinkTypeUtils.isCalmLink(linkTypeURI)) {
									// skip the CLM types.
									continue;
								} else { 
									filteredAttributes.push(currentLinkedInfo.linkConditionInfo.attributeValues[j]);
								}
							}
							if (filteredAttributes.length === 1) { 
								// if it's only 1, then it's just the "exists" "does not exist" condition. 
								// We must not leave that in the array alone or will be interpted as "exists all", etc.
								currentLinkedInfo.linkConditionInfo.attributeValues = [];
							} else { 
								currentLinkedInfo.linkConditionInfo.attributeValues = filteredAttributes;
							}
						}
						// make sure that the lifecycle status filter isn't set.  
						parsedFilter.calm = [];
					}


					this.dashboardTabPage.setSavedFilterValues(parsedFilter, onComplete);
				});
				var runPhaseTwo = dojo.hitch(this,function(response) { 
					savedFilter = response;
					parseJSONAndUpdateUI();
				});
				 
				// finish the view request, then request the CM State 
				deferred.then(runPhaseTwo, errorHandler);
			}
		}
		else {
			this.dashboardTabPage.setSavedFilterValues({});
		}
	},
	
	_onEditFilter: function(filterId, title, event){
		var filter = this._savedFilters.item(filterId);
		
		if (event && event.altKey){
			this._publicSavedFiltersContainer.clearSelection();
			this._privateSavedFiltersContainer.clearSelection();
			if (this._publicSavedFiltersContainer.containsKey(filterId)){
				this._publicSavedFiltersContainer.selectItem(filterId, false, true);
			}
			else if (this._privateSavedFiltersContainer.containsKey(filterId)){
				this._privateSavedFiltersContainer.selectItem(filterId, false, true);
			}
			this._onSavedFilterSelectionChanged(filterId, true);
		}
		else{
			var form = new com.ibm.rdm.web.query.NewSavedFilterForm(
				{
					projectURI: this.projectURI,
					resourceURI: this.resourceURI,
					savedFilterNames: this._savedFilterNames,
					callbackScope: this,
					editFilter: filter,
					viewType: this.viewType,
					dashboardTabPage: this.dashboardTabPage
				}
			);   
			this._toDestroy.push(form);
			form.show();
		}
	},
	
	_onDeleteFilter: function(uri) {
		var indexOfGlobalScopeView = dojo.indexOf(this._globalScopeView, uri);
		var deleteFilter = dojo.hitch(this, function() {
			var deleteDeferred = com.ibm.rdm.web.server.client.RDMClient.doDelete({
				projectURI: this.projectURI,
				uri: uri
			});
			deleteDeferred.then(
				dojo.hitch(this, function(response) {
					ChangeNotifier.notifyListeners({
						eventTopic: EventTopics.FILTER_DELETED,
						response: response,
						projectURI : this.projectURI,
						resourceUriScope: this.resourceURI
					});
					var savedFilter = this._savedFilters.item(uri);
					if (savedFilter && savedFilter.specificModule) {
						LinkUtils.getLinksForResource({
							sourceAndTargetURI: SavedFilterUtils.getMaterializedViewURI(uri),
							baselineURI: this.baselineURI,
							skipCache: true
						}).then(
							dojo.hitch(this, function(response) {
								var links = response.resourceInfos;
								dojo.forEach(links, function(link) {
									CalmLinkUtils.deleteLink(this.projectURI, link).then(dojo.hitch(this, function() {
										ChangeNotifier.notifyListeners({
											eventTopic: com.ibm.rdm.web.notifications.EventTopics.LINK_DELETED,
											link: link,
											direction: com.ibm.rdm.web.artifactgrid.ArtifactConstants.SUBJECT_LINK,
											resource: link.getResourceURI(),
											crossWindow: true
										});								
									}));
								}, this);
							})
						);
						this._savedFilters.remove(uri);
					}
				})
			);
			if (indexOfGlobalScopeView != -1) {
				this._globalScopeView.splice(indexOfGlobalScopeView, 1);
			}
		});
		// Prompt a delete all dialog when deleting a view used in all modules
		if (indexOfGlobalScopeView != -1) {
			var msgForm = new com.ibm.rdm.web.common.MessageForm(
				{
					confirmMessage: this._newFilterMessages.confirmAllDelete,
					dialogTitle: this._newFilterMessages.confirmDeleteTitle,
					dialogStyle: "yesNo",
					dialogWidth: 150,
					onOk: deleteFilter
				}
			);   
		}
		else {
			var msgForm = new com.ibm.rdm.web.common.MessageForm(
				{
					confirmMessage: this._newFilterMessages.confirmDelete,
					dialogTitle: this._newFilterMessages.confirmDeleteTitle,
					dialogStyle: "yesNo",
					dialogWidth: 150,
					onOk: deleteFilter
				}
			);   			
		}

		this._toDestroy.push(msgForm);
		
		msgForm.show(); 
	},
	
	_onExportFilter: function(rdf){
		CsvExport.exportXml(rdf, this.projectURI, (this.viewType === com.ibm.rdm.web.project.DashboardConstants.VIEW_TYPES.MODULE), this.baselineURI);
	},
	
	_onPermalink: function(uri, title){
		var filterURI = com.ibm.rdm.web.FRONTING_SERVICE_CONTEXT_ROOT + "web#action=com.ibm.rdm.web.pages";
		if (this.viewType
			&& ((this.viewType === com.ibm.rdm.web.project.DashboardConstants.VIEW_TYPES.MODULE)
				|| (this.viewType === com.ibm.rdm.web.project.DashboardConstants.VIEW_TYPES.COLLECTION))
			) {
			filterURI += (!this.baselineURI) ? ".showArtifact" : ".showArtifactHistory";
			filterURI += "&artifactURI=" + encodeURIComponent(this.resourceURI);
		} else {
			filterURI += (!this.baselineURI) ? ".showProjectDashboard" : ".showProjectHistory";
			filterURI += "&projectURI=" + encodeURIComponent(this.projectURI);
		}
		if(this.baselineURI) {
			filterURI += "&baselineURI="+encodeURIComponent(this.baselineURI);
		}
		filterURI += "&filterId=" + uri.substring(uri.lastIndexOf('/') + 1);
		filterURI = VvcUtils.replaceConfigurationInActionUriWithCurrent(filterURI);
		
		// Please note that we don't need to use:
		//     jazz.util.html.escape(title) in the following
		// as it has already been properly escaped (by using innerHTML)
		// by the calling function.
		var permalinkDialog = new com.ibm.rdm.web.common.PermalinkDialog({
			initLinkURI: filterURI, 
			initDisplayText: title, 
			initDialogWidth: "525px"
		});
		this._toDestroy.push(permalinkDialog);
		permalinkDialog.show();
	},
	
	_onReport: function(uri){
		//Determine the report context to generate a list of appropriate report types.
		var context = null;
		switch(this.viewType){
		case com.ibm.rdm.web.project.DashboardConstants.VIEW_TYPES.MODULE:
		case com.ibm.rdm.web.project.DashboardConstants.VIEW_TYPES.MODULES:
			context = com.ibm.rdm.web.reporting.ReportWizard.viewType.MODULE;
			break;
		case com.ibm.rdm.web.project.DashboardConstants.VIEW_TYPES.COLLECTION:
		case com.ibm.rdm.web.project.DashboardConstants.VIEW_TYPES.COLLECTIONS:
			context = com.ibm.rdm.web.reporting.ReportWizard.viewType.COLLECTION;
			break;
		default:
			context = com.ibm.rdm.web.reporting.ReportWizard.viewType.VIEW;
			break;
		}

		var wizard = new com.ibm.rdm.web.reporting.ReportWizard({
			projectURI: this.projectURI, 
			moduleURI: (this.viewType === com.ibm.rdm.web.project.DashboardConstants.VIEW_TYPES.MODULE) ? this.resourceURI : null,
			collectionURI: (this.viewType === com.ibm.rdm.web.project.DashboardConstants.VIEW_TYPES.COLLECTION) ? this.resourceURI : null,
			savedViewURI: uri, 
			context: context,
			isQuickPrintButton: true
		});
		this._toDestroy.push(wizard);
		
		wizard.show();
	},
	
	_onCreateLink: function(savedViewURI, savedViewTitle) {
		
		var partialMaterliasedViewResource = {};
		var materializedViewURI = com.ibm.rdm.web.query.SavedFilterUtils.getMaterializedViewURI(savedViewURI);
		partialMaterliasedViewResource[RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Identifier] = com.ibm.rdm.web.project.SavedFilter.IDENTIFIER;
		partialMaterliasedViewResource[RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Title] = savedViewTitle;
		partialMaterliasedViewResource.projectURI = this.projectURI;
		partialMaterliasedViewResource.resourceURI = materializedViewURI;
		var resourceInfo = com.ibm.rdm.web.model.resources.ResourceManager.getFromPartial(this.projectURI, partialMaterliasedViewResource);
		
		var linkDialog = new com.ibm.rdm.web.links.LinkDialog({
			projectURI: this.projectURI,
			resourceURI: SavedFilterUtils.getMaterializedViewURI(savedViewURI),
			resourceName: savedViewTitle,
			resourceInfos: [resourceInfo],
			resourceIsObject: false,
			linkTypeURI: this._getInitialCALMLinkType(),
			linkTypeWhitelist: this._getCALMLinkWhitelist(),
			isCollection: true
		});
		this._toDestroy.push(linkDialog);
		linkDialog.show();
	},
	
	_addCreateLinkIfApplicable: function() {
		if (this.viewType === com.ibm.rdm.web.project.DashboardConstants.VIEW_TYPES.MODULE) {
			this.projectOperationsDeferred.then(
				dojo.hitch(this, function(projectOperations) {
					if (projectOperations.item(ProjectCache.MODIFY_CLM_LINK)) {
						com.ibm.rdm.web.calm.CalmRegistry.getProjectAreaLinks(this.projectURI).then(
							dojo.hitch(this, function(projectAreaLinks) {
								var linkTypes = [];
								dojo.forEach(projectAreaLinks, function(projectAreaLink) {
									if (projectAreaLink.getLinkType() === 
											com.ibm.rdm.web.calm.CalmConstants.ProjectAreaLinkType.implementedBy ||
										projectAreaLink.getLinkType() ===
											com.ibm.rdm.web.calm.CalmConstants.ProjectAreaLinkType.validatedBy) {
										linkTypes.push(projectAreaLink);
									}
								});
								if (linkTypes.length > 0) {
									this._projectAreaLinks = linkTypes;
									this._publicSavedFiltersContainer.createLinkMenuItem(this._onCreateLink);
								}
							})
						);
					}
				})
			);
		}
	},
	
	_getCALMLinkWhitelist: function() {
		var whitelist = [];
		dojo.forEach(this._projectAreaLinks, function(projectAreaLink) {
			whitelist.push(com.ibm.rdm.web.calm.CalmConstants.ProjectAreaLinkTypeToCalmLinkTypeURI[projectAreaLink.getLinkType()]);
		});
		return whitelist;
	},
	
	_getInitialCALMLinkType: function() {
		var initial;
		dojo.some(this._projectAreaLinks, function(projectAreaLink) {
			initial = projectAreaLink.getLinkType();
			if (initial === com.ibm.rdm.web.calm.CalmConstants.ProjectAreaLinkType.validatedBy) {
				return false;
			}
		});
		return initial;
	},
	
	_onSaveFilter: function(event){
		
		var wasInEdit = this.dashboardTabPage.getFilterEditMode();
		this.dashboardTabPage.setFilterEditMode(false);
		
		if (wasInEdit){
			this.dashboardTabPage.refresh();
		}
		
		var self = this;		
		
		var showForm = function(){
			var form = new com.ibm.rdm.web.query.NewSavedFilterForm({
				projectURI: self.projectURI,
				resourceURI: self.resourceURI,
				savedFilterNames: self._savedFilterNames,
				callbackScope: self,
				dashboardTabPage: self.dashboardTabPage,
				viewType: self.viewType
			});
			form.show();
			self._toDestroy.push(form);
		};
		showForm();
	},
	
	_getSortedFilters: function() {
		var savedFilters = this._savedFilters.getValueList();
		savedFilters.sort(TagUtil.tagNameComparator);
		return savedFilters;
	},
	
	_onFilterAddedNotify: function(message){
		var newFilterURI = message.notifierParams.response.ioArgs.xhr.getResponseHeader("Location");
		var applyNewFilter = dojo.hitch(this, function(){
			this.clearSelection();
			this.selectFilter(newFilterURI, true);
		});
		setTimeout(dojo.hitch(this, this.refresh, applyNewFilter), 250);
	},
	
	_onFilterEditedNotify: function(){
		setTimeout(dojo.hitch(this, this.refresh), 250);		
	},
	
	_onFilterDeletedNotify: function(){
		// HACK to handle index delay issues in views feed.
		setTimeout(dojo.hitch(this, this.refresh), 250);
	},
	
	_onFilter: function(){
		this._publicSavedFiltersContainer.filterItemsOnName(this._filterText.value);
		this._privateSavedFiltersContainer.filterItemsOnName(this._filterText.value);
	},
	
	// CheckKey gets called when the filter dialog text changes. With a 500ms delay, we will filter
	// The existing saved filters
	_checkKey: function(event){
			setTimeout(dojo.hitch(this, this._onFilter), 500);
	},
	
	doHideListAndSaveButtons: function(){		
		this._rightHandSideDiv.style.visibility= "hidden";
		
	},
	
	_toggleExpansionState: function(){
		// summary:
		//		This is an override of a superclass function.

		var doToggle=true;
		if(this.readOnly){
			  // Saved filter can be read only but still be allowed to toggle. This is
			  // specified by the allowToggleExpansion flag, which is true by default
			  if (!this.allowToggleExpansion){
					doToggle=false;
			  }
		}
	  
		if (doToggle){
			var wasExpanded = this._isSectionExpanded;
			this.inherited("_toggleExpansionState", arguments);
			if (!wasExpanded && !this._loaded) {
				// section is being expanded for the first time, populate
				// the content
				this.startup();
			}
			if (this._populateOnExpansion) {
				this._populateOnExpansion = false;
				this._populateExpandedFilterContents();
			}
		}
	},

	_showUsingExpandedContent: function() {
		// summary:
		//		This is an override of a superclass function.

		this.inherited(arguments);

		// This section has just been expanded.  We need to make sure that the
		// appropriate toolbar treatment is shown (or not shown).
		var showOrHideAppropriateItemHoverToolbar = dojo.hitch(this, function(filtersContainer){
			if (filtersContainer.getSelectedKeys().length > 0) {
				var selectedKey = filtersContainer.getSelectedKeys()[0];
				if (filtersContainer.isItemDirty(selectedKey)) {
					// Note: we're using a setTimeout here to avoid having the toolbar
					// appear about 20 pixels too far to the left ... perhaps this is
					// due to needing the layout to occur before the dojo.position call
					// can really tell where the associated span is.
					window.setTimeout(
						dojo.hitch(this, function(){
							filtersContainer.showSelectedItemAsDirty();
						}),
						1
					);
				} else {
					filtersContainer.hideItemHoverToolbar();
				}
			} else {
				filtersContainer.hideItemHoverToolbar();
			}
		});

		// Only if we this section has truly just been toggled from collapsed
		// to expanded do we want to call 'showOrHideAppropriateItemHoverToolbar'
		// for the public/private view sections (as it would otherwise lead to
		// defect 90439 occurring) ...
		if (!this._lastExpansionStateWasExpanded) {
			showOrHideAppropriateItemHoverToolbar(this._publicSavedFiltersContainer);
			showOrHideAppropriateItemHoverToolbar(this._privateSavedFiltersContainer);
		}

		this._lastExpansionStateWasExpanded = true;
	},

	_showUsingPartiallyCollapsedContent: function() {
		// summary:
		//		This is an override of a superclass function.

		this.inherited(arguments);

		this._setPartiallyCollapsedFilterWidth();

		// This section has just been collapsed.  We need to make sure that the
		// appropriate toolbar treatment is shown (or not shown).
		if (this._partiallyCollapsedFilterContainer.getSelectedKeys().length > 0) {
			var selectedKey = this._partiallyCollapsedFilterContainer.getSelectedKeys()[0];
			if (this._partiallyCollapsedFilterContainer.isItemDirty(selectedKey)) {
				this._partiallyCollapsedFilterContainer.showSelectedItemAsDirty();
			}
			this._partiallyCollapsedFilterContainer.hideItemHoverToolbar();
		}

		this._lastExpansionStateWasExpanded = false;
	},

	_getSelectedViewUri: function() {
		if (this._publicSavedFiltersContainer.getSelectedKeys().length > 0) {
			return this._publicSavedFiltersContainer.getSelectedKeys()[0];
		}
		else if (this._privateSavedFiltersContainer.getSelectedKeys().length > 0) {
			return this._privateSavedFiltersContainer.getSelectedKeys()[0];
		}
	},

	_scrollAsNeededToShowView: function(viewURI, viewportNode) {
		// summary:
		//		Make sure that the view entry represented by 'viewURI' is
		//		currently scrolled into view with regard to the viewport
		//		represented by 'viewportNode'.
		// description:
		//		The view entry is found within a viewport (i.e. a parent node with
		//		overflow set to auto).  Scroll this viewport appropriately so that
		//		the user is able to see the passed 'viewURI' view.

		var span;
		if (this._publicSavedFiltersContainer.containsKey(viewURI)) {
			span = this._publicSavedFiltersContainer.getItemSpanForKey(viewURI);
		}
		else if (this._privateSavedFiltersContainer.containsKey(viewURI)) {
			span = this._privateSavedFiltersContainer.getItemSpanForKey(viewURI);
		}
		if (span) {
			var spanPosition = dojo.position(span),
				viewportPosition = dojo.position(viewportNode),
				additionalOffsetToUse = 40;
			if (spanPosition.y < viewportPosition.y) {
				viewportNode.scrollTop -=
					viewportPosition.y - spanPosition.y + additionalOffsetToUse;
			}
			else if ((spanPosition.y + spanPosition.h) > (viewportPosition.y + viewportPosition.h)) {
				viewportNode.scrollTop +=
					(spanPosition.y + spanPosition.h)
					- (viewportPosition.y + viewportPosition.h)
					+ additionalOffsetToUse;
			}
		}
	}
});

})();
