/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2015. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.defects.web.ui.internal.wizard.LinkCollectionSyncWizard");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.CheckBox");
dojo.require("dojox.xml.parser");

dojo.require("com.ibm.rqm.defects.web.ui.utils.CollectionLinksUtil");
dojo.require("com.ibm.rqm.defects.web.ui.utils.DefectsUIUtil");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.construction.NewItemPopup");
dojo.require("com.ibm.asq.common.web.ui.internal.util.CSHelp");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.rqm.defects.web.ui.internal.LinksTableView");
dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.Utility");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.IncrementalMessageBox");

dojo.require("com.ibm.rqm.defects.web.ui.internal.wizard.QualityTaskCreationWizard");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.requirement.web.client.internal.requirement.RequirementWorkItemClient");

dojo.require("com.ibm.rqm.defects.web.ui.view.common.ProgressDialog");
dojo.require("com.ibm.rqm.defects.web.client.internal.ResourceLinkService");
dojo.require("com.ibm.rqm.defects.web.client.internal.RequirementReconciliationClient");
dojo.require("dojox.uuid.generateRandomUuid");
dojo.require("com.ibm.asq.common.web.client.internal.CommonPreferenceClient");

(function(){
var CollectionLinksUtil = com.ibm.rqm.defects.web.ui.utils.CollectionLinksUtil;    
var LinksTableView = com.ibm.rqm.defects.web.ui.internal.LinksTableView;
var DefectsUIUtil = com.ibm.rqm.defects.web.ui.utils.DefectsUIUtil;
var Utility        = com.ibm.rqm.planning.web.ui.internal.util.Utility;
var CommonUtil = com.ibm.asq.common.web.ui.internal.util.Utility;
var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
var CSHelp         = com.ibm.asq.common.web.ui.internal.util.CSHelp;
var IncrementalMessageBox = com.ibm.asq.common.web.ui.internal.widgets.IncrementalMessageBox;

var Constants = com.ibm.rqm.planning.web.client.internal.Constants;

var RequirementWorkItemClient = com.ibm.rqm.requirement.web.client.internal.requirement.RequirementWorkItemClient;
var ProgressDialog = com.ibm.rqm.defects.web.ui.view.common.ProgressDialog;
var ResourceLinkService = com.ibm.rqm.defects.web.client.internal.ResourceLinkService;
var LinkSyncWizard = com.ibm.rqm.defects.web.ui.internal.wizard.QualityTaskCreationWizard;
var RequirementReconciliationClient = com.ibm.rqm.defects.web.client.internal.RequirementReconciliationClient;
var Preferences = com.ibm.asq.common.web.client.internal.CommonPreferenceClient;

// Summary:
//    An extended link table content provider to show additional data of a updated link.
//    This includes the reason of the update, and the linked corresponding test case(s).
//    
dojo.declare("com.ibm.rqm.defects.web.ui.internal.ExtLinksTableProvider", com.ibm.rqm.defects.web.ui.internal.LinksTableProvider, {
    
	enableCoveredByTPColumn: false,
	enableActionCol:true,
    
    columnWidth: [25, 10, 25, 25, 15],
    columnHeader: [
            DefectsUIUtil.getMessage("labelSummary"),
            DefectsUIUtil.getMessage("labelStatus"),
            DefectsUIUtil.getMessage("labelLinkedTCs"),
            DefectsUIUtil.getMessage("labelRequirementCoveredByChildTPs"),
            DefectsUIUtil.getMessage("labelAction")
    ],
    columnType: [
            TableConstants.RENDER_TYPE_RICH_HOVER,
            TableConstants.RENDER_TYPE_TEXT,
            TableConstants.RENDER_TYPE_RICH_HOVER_LIST,
            TableConstants.RENDER_TYPE_RICH_HOVER_LIST,
            TableConstants.RENDER_TYPE_TEXT
    ],
    labelModified: DefectsUIUtil.getMessage("labelModified"),
    labelDeleted: DefectsUIUtil.getMessage("labelDeleted"),
	labelNew: DefectsUIUtil.getMessage("labelNew"),
    
    getNumColumns: function(content){
        return 5;
    },
    getIsColumnVisible: function(atColumnIndex){
    	
    	if(!this.enableCoveredByTPColumn && atColumnIndex == 3){
    		return false;
    	}
		if(!this.enableActionCol && atColumnIndex == 4)
		{
			return false;
		}
    	
    	return true;
    },
    getIsColumnSortable: function(content, atColumnIndex){
    	//the linked tcs column head should not be sortable
    	if(atColumnIndex == 2 || atColumnIndex == 3){
    		return false;
    	}
        return true;
    },
    getColumnHeading: function(content, index){
        return this.columnHeader[index];
    },
    getColumnWidth: function(content, atColumnIndex){
        return this.columnWidth[atColumnIndex];
    },
    getCellDisplayType: function(rowData, atColumnIndex){
        return this.columnType[atColumnIndex];
    },
    getCellDisplayData: function(rowData, atColumnIndex){
        var val = "";
        if (atColumnIndex == 0) {
            return this.inherited(arguments);
        }
        else if(atColumnIndex == 1) {
			if(!rowData.status || rowData.status=="new")
				return this.labelNew;
            if(rowData.status && rowData.status == "deleted") {
                return  this.labelDeleted;
            }
            return this.labelModified;
        }else if(atColumnIndex == 4){
        	return rowData.action;
        }
        // not used for col 3
        return val;
    },
    getCellDisplayExtendedInfo: function(rowData, atColumnIndex){
        var val = null;
        
        if (atColumnIndex == 0) {
            return this.inherited(arguments);
        }
        if (atColumnIndex == 2) {
        	var nLinks = 0;
            var linkedTcs = rowData.linkedTCs;
            if (linkedTcs != null)
            	nLinks = linkedTcs.length;
            var items = [];
            for (var i = 0; i < nLinks; i++) {
                var item = {};
                
                item.label = linkedTcs[i].testcase.name || linkedTcs[i].testcase.testCaseName;
                item.noUpdateLabel = true;
                item.uri = CommonUtil.getQmResourceLinkUri(CommonUtil.TYPE_CASE, linkedTcs[i].testcase.versionableItemId || linkedTcs[i].testcase.itemID)
                item.openInTab = true;
                item.presentationUri = Utility.getTestCaseURI(linkedTcs[i].testcase.id);
                items.push(item);
            }
            val = {
                items: items,
				useNumericLabel: true
            }
        }if(atColumnIndex == 3){
        	var items = [];
        	var linkedTPs = rowData.linkedTPs;
        	if(!linkedTPs){
        		linkedTPs = [];
        	}
        	for(var step = 0;step < linkedTPs.length;step ++){
        		var item = {};
        		item.label = linkedTPs[step].testPlanName;
        		item.noUpdateLabel = true;
        		item.uri = CommonUtil.getQmResourceLinkUri(CommonUtil.TYPE_PLAN, linkedTPs[step].itemID);
        		item.openInTab = true;
                item.presentationUri = Utility.getTestPlanURI({id: linkedTPs[step].id});
                items.push(item);
        	}
        	val = {
        		items: items,
        		useNumericLabel: true
        	}
        }
        return val;
    }
});
    

// Summary:
//    A page widget to show a link table view to present a list of changed/added/deleted links.
//    Consumer can add actions to perform actions on the links.
//
dojo.declare("com.ibm.rqm.defects.web.ui.internal.wizard.CollectionSyncContentPage", 
             [dijit._Widget, dijit._Templated], 
{
    templateString: '<div class="collection-sync-page">' +
                    '  <div class="page-title" dojoAttachPoint="titleNode" tabIndex="0">${labelTitle}' +
                    '  </div>' +
                    '  <div class="page-description" dojoAttachPoint="descNode" tabIndex="0">${labelDescription}' +
                    '  </div>' +
                    '  <div class="page-list" dojoAttachPoint="listNode">' +
                    '  </div>' +
                    '  <div dojoAttachPoint="actionNodeContainer">' +
                    '  	<div class="page-actions" dojoAttachPoint="actionNode">' +
                    '  	</div>' +
                    '  </div>' +
                    '  <div dojoType="com.ibm.asq.common.web.ui.internal.widgets.IncrementalMessageBox" dojoAttachPoint="messageBox1"/>' + 
                    '</div>',
                    
    widgetsInTemplate: true,
    labelTitle: "",
    labelDescription: "",
    items: [],
    actions: null,
    tableViewActions: [],
    linkTableContentProvider: null,
    tableView: null,
    editable: false,
    displayCreateQTCheckbox: false,
	tableViewColumnActions: [],
     
    
    postCreate: function(){
		if(this.tableView) {
			this._list = this.tableView;
		} else {
	        this._list = new LinksTableView({
	            editable: this.editable,
				printable: true,
	            displayInPopup: false,
	            enableRowSelection: true,
	            linkTableContentProvider: this.linkTableContentProvider,
	            wiLinks: [],
				actions: this.tableViewActions,
				columnActions:this.tableViewColumnActions
	        });
		}
        this.processedItems = [];
        this.listNode.appendChild(this._list.domNode);
        if(this.items) {
            this.setListContent(this.items);
        }
        
       	if(this.displayCreateQTCheckbox){
       		var id = dojox.uuid.generateRandomUuid();
       		var div = document.createElement("div");
			this.createQTCheckbox = new dijit.form.CheckBox({
				id: id,
				name: id,
				checked: false
			});
			this.createQTCheckbox.set("disabled", true);
			div.appendChild(this.createQTCheckbox.domNode);
			this.createQTCheckboxLabel = dojo.create("label", {"class":"SectionMenuAction_Disabled"});
			this.createQTCheckboxLabel.appendChild(document.createTextNode(Constants.getMessage("msgNewQualityTask"))); 
			this.createQTCheckboxLabel["for"] = id;
			div.appendChild(this.createQTCheckboxLabel);
			this.actionNode.appendChild(div);
			
			this.createQTDescription = dojo.create("div", {"tabIndex":0, "class":"page-description", "style":"width:65%;float: left"})
			dojo.place(this.createQTDescription, this.actionNode, "before");
			this.createQTDescription.innerHTML = Constants.getMessage("msgCreateQualityTaskDescription");
        }

        if(this.actions) {
            for(var i = 0; i < this.actions.length; i++) {
                this.addAction(this.actions[i]);
            }
        }   
         
        var self = this;
        if(this.items) {
        	dojo.forEach(this.items, function(item){
        		if (item.status == "others") 
        		{
        			var taskId = dojox.uuid.generateRandomUuid();
        			self.messageBox1.appendInProgressMessage({
        				messageId: taskId,
						summary: item.summary
					});
					var args = {	
					messageId: taskId,		
					summary: item.summary,
					errorObject: item.error
					};
					self.messageBox1.showErrorMessage(args);
				}});      
		}
         
    },
    
    addAction: function(actionDescr)
    {
        var action = document.createElement('button');
		if(actionDescr.compactName){
			action.innerHTML = actionDescr.compactName;
			action.title = actionDescr.name;
		} else {
			action.innerHTML = actionDescr.name;
		}
			
        this.connect(action, 'onclick', dojo.hitch(this, function(id) {
            this.onActionClicked(id, this.getSelection());
        }, actionDescr.id));
		action.id = actionDescr.id;
        action.disabled = true;
        dojo.style(action, "float", "right");
        this.actionNode.appendChild(action);        
    },
    
    isRepairActionEnabled: function(){
        var selection = this.getSelection();
        var enable = selection && selection.length > 0;
		var enableRepair = true;
		if(enable){
			for (var i = 0; i < selection.length; i++) {
				if(selection[i] && selection[i].linkedTCs && selection[i].linkedTCs.length >0){
					continue;
				}else{
					enableRepair = false;
					break;
				}				
			}			
		}
        this.enableRepairAction = enableRepair && enable;          
        return this.enableRepairAction;
    },       
    
    disableAllActions: function() {
        dojo.forEach(this.actionNode.childNodes, function(action){
            action.disabled = true;
       });        
    },
    
    updateProcessedItems: function(items) {
        // make processed items disabled.
        this.processedItems = this.processedItems.concat(items);
        this._list.setDisabledRows(this.processedItems);
    },
    
    setListContent: function(items)
    {
        this.items = items;
        this._list.refresh(items);
    },

    getSelection: function()
    {
        var checkedRows = this._list.getCheckedRows();
        return checkedRows;
    },
    
    getDisabledRows: function(){
    	var content = this.items;
    	var disabledRows = [];
    	var table = this._list.tableViewer;
    	for(var i = 0; i < content.length; i++){
    		if(table.isRowDisabled(content[i])){
    			disabledRows.push(content[i]);
    		}
    	}
    	return disabledRows;
    },
    
    onActionClicked: function(actionId, selectedLinks)
    {    
    },
    
    uninitialize: function(){
    },
    
    isCreateQTEnabled: function(){
    	return this.createQTCheckbox.checked;
    },
    
    getPageShareContent: function(){
    	if(this.displayCreateQTCheckbox){
    		var val = {
    			createQT: this.createQTCheckbox.checked
    		};
    		
			return val;
			    		
    	}
    	return null;
    },
    
    refreshPageShareContent: function(content){
    	if(this.displayCreateQTCheckbox){
    		if(content && content.createQT !== undefined && content.createQT !== null){
    			if(content.createQT !== this.createQTCheckbox.checked)
    			this.createQTCheckbox._onClick()
    		}
    	}
    }

});

dojo.declare("com.ibm.rqm.defects.web.ui.internal.wizard.PendingActionItem4Collection", 
             [dijit._Widget, dijit._Templated], 
{
    templateString: '<div class="collection-sync-action" dojoAttachEvent="onclick:_handleSelected, onkeyup:_handleKeyup" tabIndex="0">' +
                    '  <div class="action-status" dojoAttachPoint="statusNode">' +
                    '    <span class="action-title" dojoAttachPoint="titleNode">${labelTitle}' +
                    '    </span>' +
                    '  </div>' +
                    '</div>',
    
    STATUS_LOADING: "progress-image",
    STATUS_ACTION_PERFORMED: "completed-image",
                        
    labelTitle: "",
    id: "",
    content: null,
    selected: false,
    isDisable: false,
    
    postCreate: function() {
        if(this.selected) {
            this._handleSelected();
        }
        dojo.toggleClass(this.titleNode, "sync-action-disabled", this.isDisable);
    },
    
    setStatus: function(status){
        this.statusNode.className = status;
    },
    
    setSelected: function(selected) {
        if(this.isDisable) {
            return;
        }
        dojo.toggleClass(this.domNode, "sync-action-selected", selected);
		dojo.toggleClass(this.statusNode, "selected-image", selected);
    },
    
    _handleKeyup: function(e){
    	
		if (e && e.keyCode == dojo.keys.ENTER){
			this._handleSelected();
		}
        if(e){
            dojo.stopEvent(e);
        }
    
    },
    
    _handleSelected: function(){
        this.setSelected(true);
        this.onSelected(this);
    },
    
    onSelected: function(id, content) {
        
    }
});

dojo.declare("com.ibm.rqm.defects.web.ui.internal.wizard.LinkCollectionSyncWizard", 
             [dijit._Widget, dijit._Templated], 
{
    templateString: '<div class="collection-sync-wizard">' +
                    '  <table width="100%">' +
                    '  <tr>' + 
                    '    <td class="vertical-align-top" style="width:auto; white-space:nowrap">' +
                    '      <div dojoAttachPoint="selectorNode" class="action-select-area">' +
                    '        <div dojoAttachPoint="selectLabelNode" class="action-select-area-title">${labelSelectArea}' +
                    '        </div>' +
                    '        <div dojoAttachPoint="selectorContentNode" class="action-select-content">' +
                    '        </div>' +
                    '      </div>' +
                    '    </td>' +
                    '    <td class="vertical-align-top" style="width:auto">' +
                    '      <div class="content-area">' +
                    '        <div dojoAttachPoint="contentNode">' +
                    '        </div>' +
                    '      </div>' +
                    '    </td>' +
                    '  </tr>' + 
                    '  <tr>' + 
                    '    <td colspan="2">' +
                    '      <div class="progress-image" style="display:none;" dojoAttachPoint="statusNode">' +
                    '        <span class="status-line-text" dojoAttachPoint="statusTextNode">' +
                    '        </span>' +
                    '      </div>' +
					'      <div dojoType="com.ibm.asq.common.web.ui.internal.widgets.IncrementalMessageBox" dojoAttachPoint="messageBox"/>' + 
                    '    </td>' +
                    '  </tr>' + 
                    '  </table>' +
                    '</div>',
    
	widgetsInTemplate: true,
    initContentId: "initcontent",
    wizardTitle: DefectsUIUtil.getMessage("titleSyncReqColl"),
    labelFinish: DefectsUIUtil.getMessage("labelFinish"),
    labelSelectArea: DefectsUIUtil.getMessage("labelPendingActions"),
    labelPendingAddAction: DefectsUIUtil.getMessage("labelPendingAdds"),
	labelReconcileTestCoverage: DefectsUIUtil.getMessage("labelReconcileTestCoverage"),
    labelPendingUpdateAction: DefectsUIUtil.getMessage("labelPendingUpdates"),
    labelPendingDeleteAction: DefectsUIUtil.getMessage("labelPendingDeleted"),
    labelLoading: DefectsUIUtil.getMessage("labelLoading"),
    loadingMsg: DefectsUIUtil.getMessage("msgLoadingSyncInfo"),
	loadingSubDescMsg:  DefectsUIUtil.getMessage("msgLoadingSyncSubDesc"),
    labelNoActionsRequired: DefectsUIUtil.getMessage("labelNoAction"),
    noActionRequiredMsg: DefectsUIUtil.getMessage("msgNoActionsRequired"),
	noActionRequiredMsgSub: DefectsUIUtil.getMessage("msgNoActionsRequiredSubDesc"),
    addActionTitle: DefectsUIUtil.getMessage("pendingAddActionTitle"),
	pendingTestCoverage: DefectsUIUtil.getMessage("pendingTestCoverage"),
    addActionDescription: DefectsUIUtil.getMessage("pendingAddActionDesc"),
    updateActionTitle: DefectsUIUtil.getMessage("pendingUpdateActionTitle"),
    updateActionDescription: DefectsUIUtil.getMessage("pendingUpdateActionDesc"),
    deleteActionDescription: DefectsUIUtil.getMessage("pendingDeleteActionDesc"),
    removeActionDescription: DefectsUIUtil.getMessage("pendingRemoveActionDesc"),
    unplanActionDescription: DefectsUIUtil.getMessage("pendingUnplannedActionDesc"),
    otherActionDescription: DefectsUIUtil.getMessage("pendingOtherActionDesc"),
    labelCreateTC: DefectsUIUtil.getMessage("labelCreateTC"),
    labelSuspectTC: DefectsUIUtil.getMessage("labelMarkSuspectTC"),
    newTCTitle: DefectsUIUtil.getMessage("titleGenTCTemplates"),
    newTCTitle2: DefectsUIUtil.getMessage("titleDetailGenTCTemplates"),
    reqSummaryAsName: DefectsUIUtil.getMessage("labelUseReqSummary"),
    collectionDeletedMsg: DefectsUIUtil.getMessage("msgCollectionSyncIsGone"),
    
    // source collection's provider service description.
    // filterService: null,
    collections: [],
	taskIdArray: [],
	//this variable is used to store the mapping relation of collections with their contained links, the data structure of this variable is as following
	//	{
	//		"https://rmserver:9443/rm/resources/_jjpYQZRtEeG2zOnINyHNTA" //rm collection uri
	//			:[
	//				"https://rmserver:9443/rm/resources/_jdpYQZRtEeG2zOnINyHNTA", // contained link one
	//				"https://rmserver:9443/rm/resources/_japYQZRtEeG2zOnINyHNTA", // contained link two
	//				...
	//			]
	//      ,...
	//	}
	//to improve the performance, this mapping relation is fetched only once during the reconciliation process, and is reused in any other places where it's needed.
	allCollectionsWithContainedLinks: {},
    testPlan: null,
    isInSection: true,
    // oslcVersion: null,
    
    addedRequirementLinks: [],
    updatedRequirementLinks: [],
    removedRequirementLinks: [],
    deletedRequirementLinks: [],
    otherRequirementLinks: [],
    
    cancelButton: null,
    nextButton: null,
    prevButton: null,
    firstPageContent: null,
    
    hasRepairedLinks: false,
    allContainedLinks: [],
    
    loadingMessageBox: null,
    reconcileAllCollections: false,
	
	//Summary:
	//		A global map for URIs that gonna make call in _getLinksDetail requests
	//		it is for the purpuse of cancel request
	onGoingRequestURIs: [],
    
    postCreate: function(){
    	this.hasRepairedLinks = false;
    	this.updatedRequirementLinks = [];
    	this.removedRequirementLinks = [];
    	this.deletedRequirementLinks = [];
    	this.otherRequirementLinks = [];
    	this.allContainedLinks = [];	
    	this.createLoadingMessageBox();	
    	this._getReconcilationResultByCollections();
    },
    
    onLoadCompleted: function() {        
        this._removeContent(this.initContentId);
		//Remove the placeholder action items.
		this._removeContent("placeholder1");
		this._removeContent("placeholder2");
        if (this.nActionContent == 0) {
			dojo.style(this.selectorNode, "display", "none");
            var infoPanel = document.createElement('div');
            dojo.addClass(infoPanel, "info-panel");
			if(this.collectionWasDeleted){
				infoPanel.innerHTML = this.collectionDeletedMsg;
			}else{
				infoPanel.innerHTML = '<div class="info-panel">' + this.noActionRequiredMsg + '</div>' 
						+'<div class="info-panel-subdesc">' + this.noActionRequiredMsgSub +'</div>';

			}
            this._addContent("no-content", this.labelNoActionsRequired, infoPanel, true, /*disable*/ true);
        }
        else {
            this._displayContent(false, this.defaultContent);
        }
		this.enableWizardButton();
    },
    
	_addContent: function(id, name, content, select, disable) {
		if(this._beingDestroyed) return;
        var action = new com.ibm.rqm.defects.web.ui.internal.wizard.PendingActionItem4Collection({
            id: id,
            labelTitle: name,
            content: content,
            selected: select,
            isDisable: disable,
			//Append onSelect=true parameter when onSelected is triggered
            onSelected: dojo.hitch(this, "_displayContent", true)
        });
        this.contentAttr[id] = action;
        this.selectorContentNode.appendChild(action.domNode);
		// Append the content domNode to this.contentNode in order to render
		// table cell correctly.
        if(content.domNode) {
            content = content.domNode;
        }
        this.contentNode.appendChild(content);
        return action;
    },

    _removeContent: function(id) {
        var contentAttr = this.contentAttr[id];
		if (contentAttr) {
			var content = contentAttr.content;
			if (content.domNode) {
				content = content.domNode;
			}
			if (content) {
				var self = this;
				dojo.forEach(this.contentNode.childNodes, function(child){
					if (child === content) {
						self.contentNode.removeChild(child);
					}
				});
			}
			this.selectorContentNode.removeChild(contentAttr.domNode);
			contentAttr.destroyRecursive();
			delete this.contentAttr[id];
		}
    },
    
    enableWizardButton: function(){
    	if(this.nextButton){
    		this.nextButton.disabled = false;
    	}
    	if(this.prevButton){
    		this.prevButton.disabled = false;
    	}
    	if(this.cancelButton){
    		this.cancelButton.disabled = false;
    	}if(this.finishButton){
    		this.finishButton.disabled = false;
    	}
    },
    
    disableWizardButton: function(){
    	if(this.nextButton){
    		this.nextButton.disabled = true;
    	}
    	if(this.prevButton){
    		this.prevButton.disabled = true;
    	}
    	if(this.cancelButton){
    		this.cancelButton.disabled = true;
    	}
    	if(this.finishButton){
    		this.finishButton.disabled = true;
    	}
    	
    },
    
    _collectReqURIs: function(rmItems){
    	
    	var reqURIs = [];
    	
    	if(!rmItems){
    		return reqURIs;
    	}
    	
    	for(var step = 0;step < rmItems.length;step ++){
    		reqURIs.push(rmItems[step].uri);
    	}

    	return reqURIs;
    },
    
    showThirdPage: function(){
    	this.loadComplete = false;
        this.contentAttr = {};
        this.nActionContent = 0;
        this.currSelected = null;
        this.defaultContent = null;
        
        dojo.empty(this._dialog.actionsContainer);
        dojo.empty(this.messageBox.domNode);
        this._dialog.contentContainer.appendChild(this.domNode);
        
		dojo.style(this.selectorNode, "display", "block");
        var loadPanel = document.createElement('div');
        dojo.addClass(loadPanel, "info-panel");
        loadPanel.innerHTML = this.loadingMsg;
        
       	var loadingItem = this._addContent(this.initContentId, this.labelLoading, loadPanel, true, /*disable*/true);
		var loadingItem1 = this._addContent("placeholder1", this.labelReconcileTestCoverage, document.createElement('div'), false);
		var loadingItem2 = this._addContent("placeholder2", this.labelPendingUpdateAction, document.createElement('div'), false);
		dojo.style(loadingItem1.domNode, "visibility", "hidden");
		dojo.style(loadingItem2.domNode, "visibility", "hidden");
        loadingItem.setStatus(loadingItem.STATUS_LOADING);
        
        var finishButtonDiv = document.createElement("span");
        this.finishButton = document.createElement("button");
        this.finishButton.innerHTML = Constants.getMessage("labelFinish");
		this.finishButton.tabIndex = "0";
		dojo.connect(this.finishButton, "onclick", this, dojo.hitch(this, function(){
			this._dialog.close();
			this.takeRequirementsAction();
//			this.onOk(this.collections, this);
		}));
		finishButtonDiv.appendChild(this.finishButton);
		finishButtonDiv.className = "button-div";
		
		this._dialog.actionsContainer.appendChild(finishButtonDiv);
        
        this.disableWizardButton();
        
        this._dialog.updatePosition();
           
        this._updatePendingItems(this.reconciliationResult);
        this.onLoadCompleted();
    },
    
    showSecondPage: function(selectedItems){

		if(!selectedItems || selectedItems.length == 0){
			this.showThirdPage();
			return;
		}

        this.loadComplete = false;
        this.contentAttr = {};
        this.nActionContent = 0;
        
        dojo.empty(this._dialog.contentContainer);
        dojo.empty(this._dialog.actionsContainer);
        this._dialog.contentContainer.appendChild(this.domNode);
        
		//dojo.style(this.selectorNode, "display", "none");    
        
        var nextButtonDiv = document.createElement("span");
        this.nextButton = document.createElement("button");
        this.nextButton.innerHTML = Constants.getMessage("labelNext");
		this.nextButton.tabIndex = "0";
		dojo.connect(this.nextButton, "onclick", this, dojo.hitch(this, function(){
			var contentPage = this.currSelected.content;
            console.debug(contentPage.getDisabledRows());
			this.addedRequirementLinks = this.addedRequirementLinks.concat(contentPage.getDisabledRows());
			this._removeContent("add");
			this.showThirdPage();
		}));
		nextButtonDiv.appendChild(this.nextButton);
		nextButtonDiv.className = "button-div";
		
		this._dialog.actionsContainer.appendChild(nextButtonDiv);

        var cancelButtonDiv = document.createElement("span");
        this.cancelButton = document.createElement("button");
        this.cancelButton.innerHTML = Constants.getMessage("labelCancel");
		this.cancelButton.tabIndex = "0";
		dojo.connect(this.cancelButton, "onclick", this, dojo.hitch(this, function(){
			this._dialog.close();
		}));
		cancelButtonDiv.appendChild(this.cancelButton);
		cancelButtonDiv.className = "button-div";
		
		this._dialog.actionsContainer.appendChild(cancelButtonDiv);
    
    	this.disableWizardButton();
    
        this._updatePendingAddItems(selectedItems);
		this.onLoadCompleted();
        this.enableWizardButton();
        this._dialog.updatePosition();
    },
    
	_getReconcilationResultByCollections: function(){
    	var reqCollectionURIs = [];
    	dojo.forEach(this.collections, function(coll){
    		reqCollectionURIs.push(coll.uri);
    	});
    	var args = {
    		vtpItemId: this.testPlan.versionableItemId,
    		reqCollectionURIs: reqCollectionURIs
    		
    	};
    	this.loadingMessageBox.updatePendingMessageToInProgress({
        		messageId: this.loadingMessageBox.getReqLinksMsgId	
        	});
    	RequirementReconciliationClient.getReconcilationResultByCollections(args, dojo.hitch(this, function(status, results){
    		console.debug(results);
    		this.reconciliationResult = results;
    		this.loadingMessageBox.showSuccessMessage({
        		messageId: this.loadingMessageBox.getReqLinksMsgId	
        	});
        	this.stopLoadingMessage();
        	
        	//display uncovered requirement links
    		this.showSecondPage(results["new"]);
    	}), dojo.hitch(this, function(status, error){
    		this.loadingMessageBox.showErrorMessage({
        		messageId: this.loadingMessageBox.getReqLinksMsgId,
        		errorObject: error
        	});
    		console.debug(error);
    	}));
    },
    
    _updatePendingAddItems: function(newItems) {        
        if (newItems && newItems.length > 0) {
        	
        	var hasChildTPs = false;
        	for(var i = 0; i < newItems.length; i++){
        		if(newItems[i].linkedTPs && newItems[i].linkedTPs.length > 0){
        			hasChildTPs = true;
        			break;
        		}
        	}
        	
        	var provider = new com.ibm.rqm.defects.web.ui.internal.ExtLinksTableProvider();
        	provider.columnType = [
             		             "ExtendedFullOSLCHover",
            		             TableConstants.RENDER_TYPE_TEXT,
            		             TableConstants.RENDER_TYPE_RICH_HOVER_LIST,
            		             TableConstants.RENDER_TYPE_RICH_HOVER_LIST,
            		             TableConstants.RENDER_TYPE_TEXT
            		             ];  
        	provider.enableCoveredByTPColumn = hasChildTPs;
			provider.enableActionCol = false;
        	
            var content = new com.ibm.rqm.defects.web.ui.internal.wizard.CollectionSyncContentPage({
				labelTitle: this.pendingTestCoverage,
                labelDescription: this.addActionDescription,
                items: newItems,
                linkTableContentProvider: provider,
                tableViewColumnActions:[
				{
					labelSingle: Constants.getMessage("labelGenerateNewTestCase"),
					labelMultiple: Constants.getMessage("labelGenerateNewTestCases"),
					iconClass: "add-new-testcases-image",
					isVisibleCallback: function(selectionCount){
                        return selectionCount > 1;
                    },
					isEnabledCallback: null,
					actionId: Constants.getMessage("labelGenerateTestCasesID"),
					actionHandler: dojo.hitch(this, "_handleTableViewColumnActionGenerateTCs")
				},
				{
					labelSingle: Constants.getMessage("labelGenerateNewTestCase"),
					labelMultiple: Constants.getMessage("labelGenerateNewTestCase"),
					iconClass: "add-new-testcase-image",
					isVisibleCallback: null,
					isEnabledCallback: null,
					actionId: Constants.getMessage("labelGenerateOneTestCaseID"),
					actionHandler: dojo.hitch(this, "_handleTableViewColumnActionGenerateOneTC")
				},
				{
					labelSingle: Constants.getMessage("labelAddTestCaseToTestPlan"),
					labelMultiple: Constants.getMessage("labelAddTestCaseToTestPlan"),
					iconClass: "add-icon-image",
					isVisibleCallback: dojo.hitch(this,function(selectionCount){
						return this.currSelected.content.isRepairActionEnabled();						
					}),
					isEnabledCallback: dojo.hitch(this, function(){
						return this.currSelected.content.isRepairActionEnabled();
					}),
					actionId: Constants.getMessage("labelAddTestCaseToTestPlanID"),
					actionHandler: dojo.hitch(this, "_handleTableViewColumnActionAddTCToTP")
				}
				],
				actions: [],
                onActionClicked: dojo.hitch(this, "_handlePageAction")
            });
            this.addContent = this._addContent("add", this.labelReconcileTestCoverage, content, false);
            this.defaultContent = this.addContent;
            this.nActionContent++;
        }
    },
	
	_updatePendingItems: function(items) {
		this._updatePendingModifiedItems(items["modified"]);    
		this._updatePendingRemovedItems(items["removed"]);
		this._updatePendingDeletedItems(items["deleted"]);
		this._updatePendingUnplannedItems(items["unplanned"]);
		this._updatePendingOtherItems(items["other"]);
	},
	
	_updatePendingModifiedItems: function(items){
		var suspectedLinks = items;
		this.updatedRequirementLinks = suspectedLinks;
	
    	if (suspectedLinks.length > 0) {
        	var content = new com.ibm.rqm.defects.web.ui.internal.wizard.CollectionSyncContentPage({
				labelTitle: this.updateActionTitle,                    	
            	labelDescription: this.updateActionDescription,
            	linkTableContentProvider: new com.ibm.rqm.defects.web.ui.internal.ExtLinksTableProvider(),
            	items: suspectedLinks,
            	actions: [],
        		tableViewActions: [
        		{
					label: Constants.getMessage("labelIgnoreRequirementChanges"),
					actionHandler: dojo.hitch(this, "_handleIgnoreRequirementChanges"),
					iconClass: "ignore-requirement-change-image",
					disabledOnReadOnly: true
				},
				{
					label: Constants.getMessage("labelDismissRequirementChanges"),
					actionHandler: dojo.hitch(this, "_handleDismissRequirementChanges", this.updatedRequirements),
					iconClass: "dismiss-requirement-change-image",
					disabledOnReadOnly: true
				},
				{
					label: Constants.getMessage("labelAcceptRequirementChanges"),
					actionHandler: dojo.hitch(this, "_handleAcceptRequirementChanges"),
					iconClass: "req-suspect-icon-image",
					disabledOnReadOnly: true
				}            		
        		],
    			editable: true,
            	onActionClicked: dojo.hitch(this, "_handlePageAction"),
            	displayCreateQTCheckbox: true
        	});
        	var updateContent = this._addContent("update", this.labelPendingUpdateAction, content, false);
            this.defaultContent = updateContent;
        	this.nActionContent++;
   	 	}
	},
	
	_updatePendingUnplannedItems: function(items) {    
		//unplanned
		var suspectedLinks = items;
		var preference = Preferences.getOSLCPreference();
        if ((this.reconcileAllCollections || !preference.hideUnplannedRequirements) && suspectedLinks && suspectedLinks.length > 0) {
        	var linkTableContentProvider = new com.ibm.rqm.defects.web.ui.internal.ExtLinksTableProvider();
        	linkTableContentProvider.enableActionCol = false;
        	
            var content = new com.ibm.rqm.defects.web.ui.internal.wizard.CollectionSyncContentPage({
				labelTitle: DefectsUIUtil.getMessage("pendingUnplannedActionTitle"),
                labelDescription: this.unplanActionDescription,
                linkTableContentProvider: linkTableContentProvider,
                items: suspectedLinks,
                actions: [],
                tableViewActions: [],
            	editable: true,
                onActionClicked: dojo.hitch(this, "_handlePageAction"),
                displayCreateQTCheckbox: false
            });
			var unplannedContent = this._addContent("unplan", DefectsUIUtil.getMessage("labelPendingUnplanned"), content, false);
            if(!this.defaultContent) {
                this.defaultContent = unplannedContent;
            }
            this.nActionContent++;
        }
    },
	
	_updatePendingRemovedItems: function(items) {    
		//Removed
		var suspectedLinks = items;
		this.removedRequirementLinks = suspectedLinks;
        if (suspectedLinks && suspectedLinks.length > 0) {
            var content = new com.ibm.rqm.defects.web.ui.internal.wizard.CollectionSyncContentPage({
				labelTitle: DefectsUIUtil.getMessage("pendingRemovedActionTitle"),
                labelDescription: this.removeActionDescription,
                linkTableContentProvider: new com.ibm.rqm.defects.web.ui.internal.ExtLinksTableProvider(),
                items: suspectedLinks,
                actions: [],
                tableViewActions: [                    	
                {
					label: Constants.getMessage("labelIgnoreRequirementChanges"),
					actionHandler: dojo.hitch(this, "_handleIgnoreRequirementChanges"),
					iconClass: "ignore-requirement-change-image",
					disabledOnReadOnly: true
				},
				{
					label: Constants.getMessage("labelAcceptRequirementChanges"),
					actionHandler: dojo.hitch(this, "_handleAcceptRequirementChanges"),
					iconClass: "req-suspect-icon-image",
					disabledOnReadOnly: true
				}
				],
            	editable: true,
                onActionClicked: dojo.hitch(this, "_handlePageAction"),
                displayCreateQTCheckbox: true
            });
			var removeContent = this._addContent("remove", DefectsUIUtil.getMessage("labelPendingRemoved"), content, false);
            if(!this.defaultContent) {
                this.defaultContent = removeContent;
            }
            this.nActionContent++;
        }
    },
    
    _updatePendingOtherItems: function(otherItems) {    
		//Others
		var suspectedLinks = otherItems;
		this.otherRequirementLinks = suspectedLinks;
        if (suspectedLinks && suspectedLinks.length > 0) {
      
            var content = new com.ibm.rqm.defects.web.ui.internal.wizard.CollectionSyncContentPage({
                labelTitle: DefectsUIUtil.getMessage("pendingOtherActionTitle"),
                labelDescription: this.otherActionDescription,
                linkTableContentProvider: new com.ibm.rqm.defects.web.ui.internal.ExtLinksTableProvider(),
                items: suspectedLinks,
                actions: [],
                tableViewActions: [                    	
                {
					label: Constants.getMessage("labelIgnoreRequirementChanges"),
					actionHandler: dojo.hitch(this, "_handleIgnoreRequirementChanges"),
					iconClass: "ignore-requirement-change-image",
					disabledOnReadOnly: true
				}
				],
            	editable: true,
                onActionClicked: dojo.hitch(this, "_handlePageAction")
            });
			var otherContent = this._addContent("others", DefectsUIUtil.getMessage("labelPendingOther"), content, false);
            if(!this.defaultContent) {
                this.defaultContent = otherContent;
            }
            this.nActionContent++;
        }
    },
      
    _updatePendingDeletedItems: function(deletedItems) {    
		//deleted
		var suspectedLinks = deletedItems;
		this.deletedRequirementLinks = suspectedLinks;
        if (suspectedLinks && suspectedLinks.length > 0) {
      
            var content = new com.ibm.rqm.defects.web.ui.internal.wizard.CollectionSyncContentPage({ 
			    labelTitle: DefectsUIUtil.getMessage("pendingDeleteActionTitle"),               
                labelDescription: this.deleteActionDescription,
                linkTableContentProvider: new com.ibm.rqm.defects.web.ui.internal.ExtLinksTableProvider(),
                items: suspectedLinks,
                actions: [],
                tableViewActions: [                    	
                {
					label: Constants.getMessage("labelIgnoreRequirementChanges"),
					actionHandler: dojo.hitch(this, "_handleIgnoreRequirementChanges"),
					iconClass: "ignore-requirement-change-image",
					disabledOnReadOnly: true
				},
				{
					label: Constants.getMessage("labelAcceptRequirementChanges"),
					actionHandler: dojo.hitch(this, "_handleAcceptRequirementChanges"),
					iconClass: "req-suspect-icon-image",
					disabledOnReadOnly: true
				}
				],
            	editable: true,
                onActionClicked: dojo.hitch(this, "_handlePageAction"),
                displayCreateQTCheckbox: true
            });
			var deletedContent = this._addContent("deleted", DefectsUIUtil.getMessage("labelPendingDeleted"), content, false);
            if(!this.defaultContent) {
               this.defaultContent = deletedContent;
            }
            this.nActionContent++;
        }
    },
    
    _handlePageAction: function(actionId, selectedLinks)
    {
        if (actionId == "createTC") {
            this._displayNewTestCasesTemplate(selectedLinks);
        }
        else if (actionId == "suspectTC") {
            var tcItemIds = [];
			var tcItemIdMap = {};
            var nLinks = selectedLinks.length;
            
            for (var i = 0; i < nLinks; i++) {
                var tcs = selectedLinks[i].linkedTCs;
                var nTcs = tcs.length;
                for (var j = 0; j < nTcs; j++) {
					var versionableItemId = tcs[j].testcase.versionableItemId;
					tcItemIdMap[versionableItemId] = versionableItemId;
                }
            }
			for(var itemId in tcItemIdMap){
				tcItemIds.push(itemId);
			}
            this.changeTestCaseSuspectState(this, selectedLinks, tcItemIds);
        }else if(actionId == "genTC"){
        	for(var step = 0;step < selectedLinks.length;step ++){
        		selectedLinks[step].action = Constants.getMessage("labelActionGenNewTestCase");
        	}
        	this.addContent.content._list.tableViewer.refresh();
        	this._displayNewTestCasesTemplate(selectedLinks);
        }else if(actionId == "addTC"){
        	this.hasRepairedLinks = true;
        	for(var step = 0;step < selectedLinks.length;step ++){
        		selectedLinks[step].action = Constants.getMessage("labelActionExistingTestCase");
        	}
        	this.addContent.content._list.tableViewer.refresh();
        	this.repairTestCases(this, selectedLinks);
        }
    },
    
	/**
	 * Summary: 
	 * The old method display the specific content by clearing this.contentNode
	 * then appending the new content's domNode. The flaw is that the content's
	 * table cell could not render correctly if the content domNode has not been
	 * attached to the dom tree. Working together with '_addContent', which 
	 * attaches the content to the domNode at the beginning, this new method controls
	 * the displaying by change visibility and display style. 
	 * @param {Object} onSelect
	 * @param {Object} contentAttr
	 */
    _displayContent: function(onSelect, contentAttr) {
        if(this.currSelected === contentAttr) {
            return;
        }
        var oldContent = this.currSelected;
        
        if(this.currSelected) {
            this.currSelected.setSelected(false);
        }
        this.currSelected = contentAttr;
        this.currSelected.setSelected(true);
		
		for (var i in this.contentAttr){
			var id = this.contentAttr[i].content.id;
			//If the ceontent's domNode is already attached to this.contentNode,
			//then attached should be the domNode reference, or it's null 
			var attached = dojo.byId(id);
			if(attached && attached.id == contentAttr.content.id){
					dojo.style(attached, "display", "block");
					dojo.style(attached, "visibility", "visible");
					dojo.style(attached, "overflow", "");
					dojo.style(attached, "float", "");
					dojo.style(attached, "height", "");
			} else if (attached && attached.id != contentAttr.content.id){
				if(onSelect){
					dojo.style(attached, "display", "none");
					dojo.style(attached, "visibility", "");
					dojo.style(attached, "overflow", "");
					dojo.style(attached, "float", "");
					dojo.style(attached, "height", "");
				} else {
					dojo.style(attached, "visibility", "hidden");
					//Fix that the hidden content occupies the bottom area, leaving big blank area
					dojo.style(attached, "overflow", "visible");
					dojo.style(attached, "float", "left");
					dojo.style(attached, "height", "0");
				}
			}
		}
		
		if(oldContent && oldContent.content && dojo.isFunction(oldContent.content.getPageShareContent)){
			var content = oldContent.content.getPageShareContent();
			if(this.currSelected && this.currSelected.content && dojo.isFunction(this.currSelected.content.refreshPageShareContent)){
				this.currSelected.content.refreshPageShareContent(content);
			}
		}
		
    },
    
    open: function() {
        if(!this.dialog) {
            this._showAsDialog();
            this.showLoadingMessage(this.labelLoading);
        }
    },
    
    onOk: function(collections) {
        // notification when the wizard is closed on ok.
    },
    
    createLoadingMessageBox: function(){
    	this.loadingMessageBox = new IncrementalMessageBox();
    	
    	var message = {
    		messageId: "msg" + dojox.uuid.generateRandomUuid(),
    		summary: Constants.getMessage("msgReconcileAllContainedRequirementLinks")
    	};
    	this.loadingMessageBox.getReqLinksMsgId = message.messageId;
    	this.loadingMessageBox.appendPendingMessage(message);
    },

    showLoadingMessage: function(message){
    	dojo.empty(this._dialog.contentContainer);
    	var div = document.createElement("div");
    	div.appendChild(this.loadingMessageBox.domNode);
    	this._dialog.contentContainer.appendChild(div);
    	div = dojo.create("div", {"style": "width:100%;text-align:right"}, this._dialog.contentContainer);
    	var button = document.createElement("button");
		button.innerHTML = Constants.getMessage("labelCancel");
		button.tabIndex = "0";
		div.appendChild(button);
		dojo.connect(button, "onclick", dojo.hitch(this, function(){
			ResourceLinkService.cancelGetReqLinksDetailRequests(this.onGoingRequestURIs);
			this.stopLoadingMessage();
			this._dialog.close();
		}));
    },
    
    stopLoadingMessage: function(){
    	dojo.empty(this._dialog.contentContainer);
    },
    
    _showAsDialog: function() {
        var dialogArgs = {
            primaryTitle: this.wizardTitle,
//            okLabel: this.labelFinish,
//            okCallback: dojo.hitch(this, "onOk", this.collections, this),
            zIndex: 998
        };   
        this._dialog = new com.ibm.asq.common.web.ui.internal.widgets._CommonDialog(dialogArgs);       
//        this._dialog = new com.ibm.asq.common.web.ui.internal.widgets.SelectionDialog(dialogArgs);
        var cshelp = new CSHelp({contextID: "rqm_sync_req_coll"});

        this._dialog.setSecondaryTitleNode(cshelp.domNode);
//        this._dialog._noButton.style.display = "none"; 
        
        this._dialog.setWidth(this.width);
        //this._dialog.contentContainer.appendChild(this.domNode);
        this._dialog.open(false);
        this._dialog.updatePosition();
    },
    
    _displayNewTestCasesTemplate: function(links) {
    	var _self = this;
		var nameEditable = false;
		var defaultName = this.reqSummaryAsName;
		if(links.length && links.length == 1){
			nameEditable = true;
			defaultName = links[0].summary || links[0].name;
		}
		
		var args = {
			itemType:Constants.MODEL_TYPE_CASE,
    		nameEditable: nameEditable,
			//82287: The test case didn't add to the specified test plan when use requirement to generate test case.
			//hide the test plan filed in the dialog.
    		displayRelatedTestPlans:false,
    		//preselectedPlanItemId:this.testPlan?this.testPlan.itemId:null,
    		displayOwner:true,
    		defaultName: defaultName,
    		onOkCallback: function(){
    			_self.generateNewTestCases(_self,links,this.getAttributes());
    		}    
		};
		
		//teamArea need to be set in case of the user need the permission to create the test case, refer to WI 83274 for more
		var teamArea = this.parentEditor.getTeamAreaItemId();
        var teamAreaName = this.parentEditor.getTeamAreaName();
            
        if(teamArea && teamArea!= "" && teamArea!= Constants.UNASSIGNED_TEAMAREA_CONSTANT_VALUE){
			args.teamAreaItemId = teamArea;      
            args.teamAreaName = teamAreaName;     	
         }

    	var newTCDialog = new com.ibm.rqm.planning.web.ui.internal.view.construction.NewItemPopup(args);    	    	
    },
    
	_displayNewOneTestCaseTemplate: function(links) {
    	var _self = this;
		var nameEditable = true;
		var defaultName = null;
		if(links.length && links.length == 1){
			defaultName = links[0].summary || links[0].name;
		}
		
		var args = {
    		itemType:Constants.MODEL_TYPE_CASE,
    		nameEditable: nameEditable,
			//82287: The test case didn't add to the specified test plan when use requirement to generate test case.
			//hide the test plan filed in the dialog.
    		displayRelatedTestPlans:false,
    		//preselectedPlanItemId:this.testPlan?this.testPlan.itemId:null,
    		displayOwner:true,
    		defaultName: defaultName,
    		onOkCallback: function(){
    			_self.generateNewOneTestCase(_self,links,this.getAttributes());
    		}    
		};
		
		//teamArea need to be set in case of the user need the permission to create the test case, refer to WI 83274 for more
		var teamArea = this.parentEditor.getTeamAreaItemId();
        var teamAreaName = this.parentEditor.getTeamAreaName();
            
        if(teamArea && teamArea!= "" && teamArea!= Constants.UNASSIGNED_TEAMAREA_CONSTANT_VALUE){
			args.teamAreaItemId = teamArea;      
            args.teamAreaName = teamAreaName;     	
         }

    	var newTCDialog = new com.ibm.rqm.planning.web.ui.internal.view.construction.NewItemPopup(args);    	    	
    	
    },
	
    // Summary:
    //      Notification of create test cases is invoked. Client needs to override to handle.
    //      create new test cases. 
    //      monitor - the monitor to call to report back progress
    //      links - the set of links to create test cases
    //      template - a template of initial values for the new test cases. 
    //
    generateNewTestCases: function(monitor, links, template) {
    },
    
	generateNewOneTestCase:function(monitor, links, template)
	{
		
	},
	
    repairTestCases: function(monitor, links){
    	
    },
    
    // Summary:
    //      Notification of changing test cases suspect states. Client needs to override to handle.
    //      create new test cases. 
    //      monitor - the monitor to call to report back progress
    //      links - the set of links that has corresponding test cases
    //      tcItemIds - test cases' itemIds to be changed.
    //
    changeTestCaseSuspectState: function(monitor, links, tcItemIds) {
        
    },
    
    // Implement monitor interface.
    beginTask: function(title, taskId) {
//        dojo.style(this.statusNode, "display", "");
//        this.statusTextNode.innerHTML = title;
//        this.taskTitle = title;
        var currTask = this.currSelected;
        
        if(currTask.content && currTask.content.disableAllActions)
            currTask.content.disableAllActions();
		
		this.disableWizardButton();
		
		this.messageBox.appendInProgressMessage({
			summary: title,
			messageId: taskId
		});
		
		this.taskIdArray.push(taskId);
    },
    
//    // Implement monitor interface.
//    subTask: function(subTask) {
//        // just in case the client didn't call beginTask
//        dojo.style(this.statusNode, "display", "");
//        var task = this.taskTitle ? this.taskTitle + " - " : "";
//        this.statusTextNode.innerHTML = task + subTask;
//    },
    
    // Implement monitor interface.
    done: function(processedLinks, taskId, title, errorObject)
    {
//        dojo.style(this.statusNode, "display", "none");
//        this.statusTextNode.innerHTML = "";
        
        if (processedLinks) {
            var currTask = this.currSelected;
            var contentPage = currTask.content;
            currTask.setStatus(currTask.STATUS_ACTION_PERFORMED);
            contentPage.updateProcessedItems(processedLinks);
        }
		
		var args = {
				messageId: taskId,
				summary: title
			};
		if(dojo.isArray(processedLinks)){
			var details = [];
			dojo.forEach(processedLinks, function(link){
				details.push({
					message: link.name || link.summary || link.description || link.uri
				});
			});
			args.details = details;
		}
		if(dojo.isObject(errorObject)){
			args.errorObject = errorObject;
			this.messageBox.showErrorMessage(args);
		} else {
			this.messageBox.showSuccessMessage(args);
		}
		
		//remove the finished task
		//IE does not support indexOf method for array
		//var index = this.taskIdArray.indexOf(taskId);
		for(var index = this.taskIdArray.length; index-- && this.taskIdArray[index] != taskId;); 
		if(index > -1){
			this.taskIdArray.splice(index, 1);
		}
		
		if(!this.taskIdArray.length){
			this.enableWizardButton();
		}
    },

    uninitialize: function(){
    },
    
    handleCreateQC: function(content){
    	    	
    	var rmLinks = this.contentAttr[content].content._list.tableViewer.content;

   		var wizard = new com.ibm.rqm.defects.web.ui.internal.wizard.QualityTaskCreationWizard({
			testcaseLinks: rmLinks,
			creationFinishedHandler: dojo.hitch(this, "_handleQualitTaskAlreadyCreated", this.contentAttr[content].content)
		});
        wizard.open();
    },
    
    _handleQualitTaskAlreadyCreated: function(content){
    	var actions = content._list.tableViewer.tableActionList;
    	for(var step = 0;step < actions.length;step ++){
    		var action = actions[step];
    		if(action.label == Constants.getMessage("toolTipCreateNewQC")){
    			action.disabled = true;
    		}
    	}
    },
    
    prepareCreatingQualityTask: function(rmLinks, deletedLinks){

		var needCreateQT = false;
		if(this.currSelected && this.currSelected.content && dojo.isFunction(this.currSelected.content.isCreateQTEnabled)){
			needCreateQT = this.currSelected.content.isCreateQTEnabled();
		}
		if(!this.parentEditor){
			this.parentEditor = null;
		}
		var wizard = new LinkSyncWizard({
			testcaseLinks: rmLinks,
			deletedLinks: deletedLinks,
			section: this,
			needCreateQT: needCreateQT,
			parentEditor: this.parentEditor
		});
		
        wizard.open();
          
	},
    
	//set suspect status for all the test cases who had been marked as suspected givin their requirements' updates	
    suspectRelatedTestCases: function(suspectRequirements, testplanId){
    	var context = {
    		allTestCases: [],
    		invoker: this
    	};
    	
		var args = {};
		args.testPlanItemId = testplanId; //refer to 62710
		var addedReqUrls = [];
		for(var step = 0;step < suspectRequirements.length;step ++){
			addedReqUrls.push(suspectRequirements[step].uri);
    	}
		args.reqURIs = addedReqUrls;

    	var handler = {
        	success: dojo.hitch(context, function(results){
				if(!results || results.length <1) return;
				
        		for(var step = 0;step < results.length;step ++){
        			var testcase = results[step];    
					this.allTestCases.push(testcase);    			        			
        		}
        			
        		var tcItemIds = [];
        		for(var step = 0;step < this.allTestCases.length;step ++){
        			tcItemIds.push(this.allTestCases[step].itemID)
        		}        		
        		
				com.ibm.rqm.defects.web.ui.utils.CollectionLinksUtil.changeTestCaseSuspectState([], tcItemIds, dojo.hitch(this, function(testCases){                
                	dojo.publish(GlobalEvents.ArtifactsUpdated ,[Constants.SUSPECT_TYPE, testCases]);
					}), dojo.hitch(this, function(e){}));        		        				
            }),
            error:function(e){
            }
    	};
    	
		var responseHandler = new ServiceResponseHandler(handler, "success", "error");    	
    	RequirementWorkItemClient.getTestCasesByRequirementURIs(responseHandler, args);
    },
    
    batchUpdateRequirementsModifiedTimeSuccess: function(){
    	console.log("batch update suspect requirements success.");
		this.onOk(this.collections, this);
    },
    
    batchUpdateRequirementsModifiedTimeError: function(e){
    	console.error("batch update suspect requirements error.");
    },
    
    batchUpdateRequirementsModifiedTime: function(requirements){
    	
    	var params = {
    		reqURIs: [],
    		lastModifiedTimes: []
    	};
    	
    	for(var step = 0;step < requirements.length;step ++){
    		params.reqURIs.push(requirements[step].uri);
    		params.lastModifiedTimes.push(requirements[step].lastModifiedTime);
    	}
    	
    	var responseHandler = new ServiceResponseHandler(this, "batchUpdateRequirementsModifiedTimeSuccess", "batchUpdateRequirementsModifiedTimeError");
    	
    	RequirementWorkItemClient.updateCalmRequirementsModifiedTimes(responseHandler, params);

    },
    
    retreiveAndUpdateLastModifiedTime: function(requirements){
    	 
    	 if(!requirements || requirements.length == 0){
    	 	return;
    	 }
    	 
    	 var context = {
    	 	
    	 	requirements: requirements,
    	 	nProcessedCount: 0,
    	 	invoker: this,
    	 	
    	 	success: function(requirementItem){   	 		
    	 		
    	 		var lastModifiedDate = dojo.date.stamp.fromISOString(requirementItem.lastModified);
        		var lastModifiedDate = new Date(lastModifiedDate.getTime() + 1000);
        		
        		for(var step = 0;step < this.requirements.length;step ++){      			
        			var requirement = this.requirements[step];
        			if(requirement.uri == requirementItem.uri){
        				requirement.lastModifiedTime = dojo.date.stamp.toISOString(lastModifiedDate, {zulu: true});
        				break;
        			}  			
        		}
        		
        		this.nProcessedCount ++;
        		
        		if(this.nProcessedCount == this.requirements.length){
        			
        			if(this.invoker.needUpdateCollectionLastSyncTime && this.requirements.length > 0){
        				
        				var latestModifiedTime = dojo.date.stamp.fromISOString(this.requirements[0].lastModifiedTime);
        				
        				for(var step = 1;step < this.requirements.length;step ++){
        					var tempTime = dojo.date.stamp.fromISOString(this.requirements[step]);
        					if(tempTime > latestModifiedTime){
        						latestModifiedTime = tempTime;
        					}
        				}
        				
        				for(var step = 0;step < this.invoker.collections.length;step ++){
        					this.invoker.collections[step].lastSync = dojo.date.stamp.toISOString(latestModifiedTime, {zulu: true});
        				}
        				
        			}
        			
        			this.invoker.batchUpdateRequirementsModifiedTime(this.requirements);
        		}
        		
    	 	}
    	 };
    	 
    	 
    	 for(var step = 0;step < requirements.length;step ++){
        	var requirement = requirements[step];
        	var serviceProviderManager = com.ibm.rqm.defects.web.client.RequirementsManager2.getInstance();
        	serviceProviderManager.initServiceProviderForArtifact(
            
            dojo.hitch(this, function(requirementItem, currentServiceProvider){
            	if(currentServiceProvider){
            		ResourceLinkService.getRemoteRRCRequirement(requirementItem.uri, dojo.hitch(context, "success"), currentServiceProvider.getOSLCCoreVersion());
            	}
            }, requirement), requirement.uri);
        				
        }
        
    },

    takeRequirementsAction: function(){
    	
    	var needUpdateRequirements = [];
    	var needSuspectRequirements = [];
    	var needDeleteRequirements = [];
    	var needRemovedRequirements = [];
    	var needSuspectInTestPlanRequirements = [];
    	
    	var removedReqs = [];
    	
    	this.needUpdateCollectionLastSyncTime = true;
    	
    	//updated ones
    	for(var step = 0;step < this.updatedRequirementLinks.length;step ++){
    		
    		if(this.updatedRequirementLinks[step].action === Constants.getMessage("labelDismissRequirementChanges")){
    			needUpdateRequirements.push(this.updatedRequirementLinks[step]);
    		}else if(this.updatedRequirementLinks[step].action === Constants.getMessage("labelAcceptRequirementChanges")){
    			needUpdateRequirements.push(this.updatedRequirementLinks[step]);
    			needSuspectRequirements.push(this.updatedRequirementLinks[step]);
    		}else{
    			this.needUpdateCollectionLastSyncTime = false;
    		}

    	}
    	
    	//Deleted ones.
    	for(var step = 0;step < this.deletedRequirementLinks.length;step ++){
    		if(this.deletedRequirementLinks[step].action === Constants.getMessage("labelAcceptRequirementChanges")){
    			needDeleteRequirements.push(this.deletedRequirementLinks[step]);
    			needSuspectRequirements.push(this.deletedRequirementLinks[step]);
    			removedReqs.push(this.deletedRequirementLinks[step]);
    		}
    	}
    	
    	//Removed ones
    	for(var step = 0;step < this.removedRequirementLinks.length;step ++){
    		if(this.removedRequirementLinks[step].action === Constants.getMessage("labelAcceptRequirementChanges")){
    			needRemovedRequirements.push(this.removedRequirementLinks[step]);
    			needSuspectInTestPlanRequirements.push(this.removedRequirementLinks[step]);
    			removedReqs.push(this.removedRequirementLinks[step]);
    		}
    	}
    	
    	
    	var addedReqMapByCollItemId = {};
    	var removedReqMapByCollItemId = {};
    	var collItemIdMap = {};
    	for(var i = 0; i < this.addedRequirementLinks.length; i++){
    		var link = this.addedRequirementLinks[i];
    		var parentCollections = link.parentCollections;
    		for(var j = 0; j < parentCollections.length; j++){
    			var collItemID = parentCollections[j].itemID;
    			if(addedReqMapByCollItemId[collItemID]){
    				addedReqMapByCollItemId[collItemID].push(link.uri || link. url);
    			} else {
    				addedReqMapByCollItemId[collItemID] = [link.uri || link. url];
    				collItemIdMap[collItemID] = collItemID;
    			}
    		}
    	}
    	for(var i = 0; i < removedReqs.length; i++){
    		var link = removedReqs[i];
    		var parentCollections = link.parentCollections;
    		for(var j = 0; j < parentCollections.length; j++){
    			var collItemID = parentCollections[j].itemID;
    			if(removedReqMapByCollItemId[collItemID]){
    				removedReqMapByCollItemId[collItemID].push(link.uri || link. url);
    			} else {
    				removedReqMapByCollItemId[collItemID] = [link.uri || link. url];
    				collItemIdMap[collItemID] = collItemID;
    			}
    		}
    	}
    	
    	ProgressDialog.doRunWithProgress(DefectsUIUtil.getMessage("msgTakingRequirementsAction"), dojo.hitch(this, function(monitor){
			monitor.beginTask(DefectsUIUtil.getMessage("msgTakingRequirementsAction"));
            if(needUpdateRequirements.length > 0){
				this.retreiveAndUpdateLastModifiedTime(needUpdateRequirements);
    		}
			if(needSuspectRequirements.length > 0){
				this.suspectRelatedTestCases(needSuspectRequirements, "");
				this.prepareCreatingQualityTask(needSuspectRequirements, needDeleteRequirements);
			}
			
			// For the removed ones, we only display the testcases in the test plan
			if(needSuspectInTestPlanRequirements.length > 0){
				// set the testplanItemId to only get the test cases in the test plan
				this.suspectRelatedTestCases(needSuspectInTestPlanRequirements, this.testPlan.versionableItemId);
				this.prepareCreatingQualityTask(needSuspectInTestPlanRequirements, needRemovedRequirements);
			}
			
			
			for(var collItemId in collItemIdMap){
				var args = {
					reqColItemId: collItemId,
					addedReqUris: addedReqMapByCollItemId[collItemId] || [],
					removedReqUris: removedReqMapByCollItemId[collItemId] || [],
					testPlanItemId: this.testPlan.versionableItemId
				};
				RequirementReconciliationClient.postUpdateCalmRequirementCollection(args, dojo.hitch(this, function(result){
					console.debug(result);
				}),dojo.hitch(this, function(error){
					console.debug(error);
				}));
			}
			
			monitor.done();	
		}));

    },
    
    
    setRequirementAction: function(action){    	
    	var content = this.currSelected.content;
    	
    	var selectedRows = content.getSelection();
    	
    	if(!selectedRows || selectedRows.length == 0){
    		return;
    	}
    	
    	for(var step = 0;step < selectedRows.length;step ++){
    		var row = selectedRows[step];
    		row.action = action;
    	}
    	
    	content._list.tableViewer.refresh();
		if(content.isCreateQTEnabled){
    		if(action == Constants.getMessage("labelAcceptRequirementChanges")){
				//only enable the check box when user is trying to accept the changes
				content.createQTCheckbox.set("disabled", false);
				dojo.removeClass(content.createQTCheckboxLabel, "SectionMenuAction_Disabled");
				content.createQTDescription.style.display = "none";
			} else{
				content.createQTCheckbox.set("disabled", true);
				dojo.addClass(content.createQTCheckboxLabel, "SectionMenuAction_Disabled");
				content.createQTDescription.style.display = "";
			}
		}
    },
    
	//UX design, if the action is ignore, do not show any message in the column
    _handleIgnoreRequirementChanges: function(){
    	this.setRequirementAction("");
    },
    
    _handleDismissRequirementChanges: function(){
    	this.setRequirementAction(Constants.getMessage("labelDismissRequirementChanges"));
    },
    
    _handleAcceptRequirementChanges: function(){
    	this.setRequirementAction(Constants.getMessage("labelAcceptRequirementChanges"));
    },
	
	takeTableViewColumnAction:function(action){
		
		var content = this.currSelected.content;
    	var selectedRows = content.getSelection();
    	if(!selectedRows || selectedRows.length == 0){
    		return;
    	}
		if (action == Constants.getMessage("labelGenerateTestCasesID") || action == Constants.getMessage("labelGenerateOneTestCaseID")) {
			var selectedRowsLength = selectedRows.length;
			for (var step = 0; step < selectedRowsLength; step++) {
				var row = selectedRows[step];
				row.action = Constants.getMessage("labelActionGenNewTestCase");
			}
			content._list.tableViewer.refresh();
			if (action == Constants.getMessage("labelGenerateTestCasesID")) {
				this._displayNewTestCasesTemplate(selectedRows);
			}
			else{
				this._displayNewOneTestCaseTemplate(selectedRows);
			}
		}
		else if(action == Constants.getMessage("labelAddTestCaseToTestPlanID"))
		{
			this.hasRepairedLinks = true;
			for (var step = 0; step < selectedRows.length; step++) {
				var row = selectedRows[step];
				row.action = Constants.getMessage("labelActionExistingTestCase");
			}
			content._list.tableViewer.refresh();
        	this.repairTestCases(this, selectedRows);
		}
		
	},
	
	_handleTableViewColumnActionGenerateTCs:function()
	{
		this.takeTableViewColumnAction(Constants.getMessage("labelGenerateTestCasesID"));
	},
	
	_handleTableViewColumnActionGenerateOneTC:function()
	{
		this.takeTableViewColumnAction(Constants.getMessage("labelGenerateOneTestCaseID"));
	},
	_handleTableViewColumnActionAddTCToTP:function()
	{
		this.takeTableViewColumnAction(Constants.getMessage("labelAddTestCaseToTestPlanID"));
	}
	
        
});
    
})();
