<?xml version="1.0" encoding="utf-8"?>
<!-- 
/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012, 2014. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 -->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:fo="http://www.w3.org/1999/XSL/Format" 
    xmlns:rqm="http://jazz.net/xmlns/alm/qm/v0.1/"
    xmlns:dc="http://purl.org/dc/elements/1.1/"
    xmlns:prep="http://com.ibm.rqm.print/prep"
    xmlns:pref="http://com.ibm.rqm.export.pdf/preferences"
    xmlns:log="xalan://com.ibm.rqm.print.render.fop.xml.xslt.extension.LoggingExtension"
    xmlns:time="xalan://com.ibm.rqm.print.render.fop.xml.xslt.extension.TimeUtilsExtension"
    xmlns:xhtml="http://www.w3.org/1999/xhtml"
    xmlns:utils="xalan://com.ibm.rqm.print.render.fop.xml.xslt.extension.UtilsExtension"
    exclude-result-prefixes="xsl pref log time utils">

    <!-- Rich Text Section -->
    <xsl:template match="rqm:section[@id='com.ibm.rqm.planning.editor.section.planBusinessObjectives']">
        <fo:block><xsl:apply-templates/></fo:block>
    </xsl:template>
    
    <xsl:template name="buildRichTextSection">
        <xsl:param name="header"/>
        <xsl:param name="rich-text"/>
        
        <xsl:if test="$preferences/pref:renderEmptySections='true' or $rich-text[xhtml:div[*]]"> 
	        <!-- section header -->
	        <xsl:if test="$header">
	            <fo:block xsl:use-attribute-sets="section-heading"><xsl:value-of select="$header"/></fo:block>    
	        </xsl:if>
	        
	        <!-- section content -->
	        <fo:block><xsl:apply-templates select="$rich-text"/></fo:block>
        </xsl:if>
    </xsl:template>
    
    <!-- Custom Grid Section -->
    <xsl:template name="buildCustomGridSection">
        <xsl:param name="header"/>
        <xsl:param name="grid-json"/>
        
        <xsl:if test="$grid-json or $preferences/pref:renderEmptySections='true'"> 
            <!-- section header -->
            <xsl:if test="$header">
                <fo:block xsl:use-attribute-sets="section-heading"><xsl:value-of select="$header"/></fo:block>    
            </xsl:if>
            
            <!-- section content -->
            <fo:block><xsl:apply-templates select="utils:createXhtmlFromGrid($grid-json)"/></fo:block>
        </xsl:if>
    </xsl:template>
    
    <xsl:template name="buildFormalReviewSection">
        <xsl:param name="strings"/>
        <xsl:param name="approvals"/>
        
        <!-- section header -->
        <fo:block xsl:use-attribute-sets="section-heading"><xsl:value-of select="$strings/Formal_Review"/></fo:block>
        
        <!-- section content -->
        <xsl:if test="$approvals or $preferences/pref:renderEmptyTables='true'">
            <fo:table xsl:use-attribute-sets="artifact-table">
                <fo:table-header xsl:use-attribute-sets="artifact-table-header">
                    <!-- review type -->
                    <fo:table-cell xsl:use-attribute-sets="artifact-table-header-cell">
                        <fo:block><xsl:value-of select="$strings/Review_Type"/></fo:block>
                    </fo:table-cell>
                    <!-- name -->
                    <fo:table-cell xsl:use-attribute-sets="artifact-table-header-cell">
                        <fo:block><xsl:value-of select="$strings/Name"/></fo:block>
                    </fo:table-cell>
                    <!-- status -->
                    <fo:table-cell xsl:use-attribute-sets="artifact-table-header-cell">
                        <fo:block><xsl:value-of select="$strings/Status"/></fo:block>
                    </fo:table-cell>
                    <!-- comments -->
                    <fo:table-cell xsl:use-attribute-sets="artifact-table-header-cell">
                        <fo:block><xsl:value-of select="$strings/Comments"/></fo:block>
                    </fo:table-cell>
                    <!-- due date -->
                    <fo:table-cell xsl:use-attribute-sets="artifact-table-header-cell date-header">
                        <fo:block><xsl:value-of select="$strings/Due_Date"/></fo:block>
                    </fo:table-cell>
                </fo:table-header>
                <fo:table-body>
                    <xsl:choose>
                        <xsl:when test="$approvals">
                            <xsl:for-each select="$approvals">
                                <fo:table-row>
                                    <!-- review type -->
                                    <fo:table-cell xsl:use-attribute-sets="artifact-table-body-cell" width="55">
                                        <xsl:variable name="type" select="rqm:approvalType"/>
                                        <fo:block-container overflow="hidden">
                                        	<fo:block><xsl:value-of select="$strings/*[name()=$type]"/></fo:block>
                                    	</fo:block-container>
                                    </fo:table-cell>
                                    <!-- name -->
                                    <fo:table-cell xsl:use-attribute-sets="artifact-table-body-cell user-header">
                                        <fo:block-container overflow="hidden">
                                        	<fo:block><xsl:apply-templates select="rqm:approval/rqm:approvalOwner"/></fo:block>
                                    	</fo:block-container>
                                    </fo:table-cell>
                                    <!-- status -->
                                    <fo:table-cell xsl:use-attribute-sets="artifact-table-body-cell" width="55">
                                        <xsl:variable name="status" select="rqm:approval/rqm:approvalStatus"/>
                                        <fo:block-container overflow="hidden">
                                        	<fo:block><xsl:value-of select="$strings/*[name()=$status]"/></fo:block>
                                    	</fo:block-container>
                                    </fo:table-cell>
                                    <!-- comments -->
                                    <fo:table-cell xsl:use-attribute-sets="artifact-table-body-cell">
                                        <fo:block-container overflow="hidden">
                                        	<fo:block><xsl:apply-templates select="rqm:approval/rqm:comment"/></fo:block>
                                    	</fo:block-container>
                                    </fo:table-cell>
                                    <!-- due date -->
                                    <fo:table-cell xsl:use-attribute-sets="artifact-table-body-cell date-cell">
                                        <fo:block-container overflow="hidden">
                                        	<fo:block><xsl:value-of select="time:formatMediumDate(rqm:approvalDate, $locale)"/></fo:block>
                                    	</fo:block-container>
                                    </fo:table-cell>
                                </fo:table-row>
                            </xsl:for-each>    
                        </xsl:when>
                        <xsl:otherwise>
                            <fo:table-row>
                                <xsl:call-template name="empty-columns">
                                    <xsl:with-param name="column-count" select="5"/>
                                </xsl:call-template>
                            </fo:table-row>
                        </xsl:otherwise>
                    </xsl:choose>
                </fo:table-body>
            </fo:table>
        </xsl:if>
    </xsl:template>

    <!-- Risk Assessment -->
    <xsl:template match="rqm:section[@id='com.ibm.rqm.risk.editor.section.planAssessment' or @id='com.ibm.rqm.risk.editor.section.suiteAssessment' or @id='com.ibm.rqm.risk.editor.section.caseAssessment']">
        <!-- Risk Assessment Summary (rich text) -->
        <xsl:call-template name="buildRichTextSection">
            <xsl:with-param name="rich-text" select="*[@extensionDisplayName='RQM-KEY-TP-RISK-TITLE']"/>
        </xsl:call-template>
        
        <!-- Risk Assessment: OOOOO (value) -->
        <fo:block space-before="10pt" font-weight="bold"><xsl:value-of select="$strings/Risk_Assessment"/><xsl:value-of select="concat(':',' ')"/>
            <xsl:call-template name="drawRiskIconSet">
                <xsl:with-param name="iconCount"><xsl:value-of select="rqm:risk/rqm:riskLevel"/></xsl:with-param>
                <xsl:with-param name="positiveIcon">risk-selected_obj.gif</xsl:with-param>
                <xsl:with-param name="negativeIcon">risk-unselect_obj.gif</xsl:with-param>
            </xsl:call-template>
        </fo:block>
        
        <xsl:if test="$preferences/pref:renderEmptyTables='true' or rqm:risk/rqm:factors/rqm:factor">
            <fo:table xsl:use-attribute-sets="artifact-table">
                <fo:table-header xsl:use-attribute-sets="artifact-table-header">
                    <!-- Risk -->                                           
                    <fo:table-cell xsl:use-attribute-sets="artifact-table-header-cell">
                        <fo:block><xsl:value-of select="$strings/Risk"/></fo:block>
                    </fo:table-cell>
                    <!-- Importance -->
                    <fo:table-cell xsl:use-attribute-sets="artifact-table-header-cell">
                        <fo:block><xsl:value-of select="$strings/Importance"/></fo:block>
                    </fo:table-cell>
                    <!-- Mitigation Action -->
                    <fo:table-cell xsl:use-attribute-sets="artifact-table-header-cell">
                        <fo:block><xsl:value-of select="$strings/Mitigation_Action"/></fo:block>
                    </fo:table-cell>
                </fo:table-header>
                <fo:table-body>
                    <xsl:choose>
                        <xsl:when test="rqm:risk/rqm:factors/rqm:factor">
                            <xsl:for-each select="rqm:risk/rqm:factors/rqm:factor">
                                <xsl:variable name="riskName" select="rqm:name"/>
                                
                                <fo:table-row>
                                    <!-- Risk -->
                                    <fo:table-cell xsl:use-attribute-sets="artifact-table-body-cell">
                                        <fo:block-container overflow="hidden">
	                                        <fo:block>
	                                            <xsl:choose>
	                                                <!-- built in risk factors -->
	                                                <xsl:when test="$strings/*[name()=$riskName]">
	                                                    <xsl:value-of select="$strings/*[name()=$riskName]"/></xsl:when>
	                                                <!-- custom risk factors -->
	                                                <xsl:otherwise>
	                                                    <xsl:value-of select="rqm:name"/></xsl:otherwise>
	                                            </xsl:choose>
	                                        </fo:block>
                                        </fo:block-container>
                                    </fo:table-cell>
                                    <!-- Importance -->
                                    <fo:table-cell xsl:use-attribute-sets="artifact-table-body-cell">
                                        <fo:block-container overflow="hidden">
	                                        <fo:block>
	                                            <xsl:call-template name="drawRiskIconSet">
	                                                <xsl:with-param name="iconCount"><xsl:value-of select="rqm:weight"/></xsl:with-param>
	                                                <xsl:with-param name="positiveIcon">importance_selected_obj.gif</xsl:with-param>
	                                                <xsl:with-param name="negativeIcon">importance_unselected_obj.gif</xsl:with-param>
	                                            </xsl:call-template>    
	                                        </fo:block>
                                        </fo:block-container>
                                    </fo:table-cell>
                                    <!-- Mitigation Action -->
                                    <fo:table-cell xsl:use-attribute-sets="artifact-table-body-cell">
                                        <fo:block-container overflow="hidden">
                                        	<fo:block><xsl:value-of select="rqm:mitigationAction"/></fo:block>
                                    	</fo:block-container>
                                    </fo:table-cell>
                                </fo:table-row>
                            </xsl:for-each>
                        </xsl:when>
                        <xsl:otherwise>
                            <fo:table-row>
                                <xsl:call-template name="empty-columns">
                                    <xsl:with-param name="column-count" select="3"/>
                                </xsl:call-template>
                            </fo:table-row>
                        </xsl:otherwise>
                    </xsl:choose>
                </fo:table-body>
            </fo:table>
        </xsl:if>
        
        <!-- Community Risk | Risk Discussion -->        
        <xsl:variable name="community-assessment" select="rqm:risk/rqm:commnunityAssessment"/>
        
        <xsl:if test="$community-assessment">
            <fo:block font-weight="bold" space-before="10pt">
                <xsl:value-of select="$strings/Community_Risk"/>:
                <xsl:text> </xsl:text>
                <xsl:call-template name="drawRiskIconSet">
                    <xsl:with-param name="iconCount"><xsl:value-of select="$community-assessment/rqm:riskLevel"/></xsl:with-param>
                    <xsl:with-param name="positiveIcon">risk-selected_obj.gif</xsl:with-param>
                    <xsl:with-param name="negativeIcon">risk-unselect_obj.gif</xsl:with-param>
                </xsl:call-template>
            </fo:block>
            
            <fo:table xsl:use-attribute-sets="table">
                <fo:table-column column-width="30%"/>
                <fo:table-column column-width="70%"/>
                <fo:table-body>
                    <fo:table-row>
                        <!-- Community Risk: -->
                        <fo:table-cell>
                            <fo:block>
                                <fo:table xsl:use-attribute-sets="border" border-width=".5" table-layout="fixed" inline-progression-dimension="80%">
                                    <fo:table-body>
                                        <fo:table-row>
                                            <fo:table-cell padding="5pt">
                                                <fo:block>
                                                    <fo:table table-layout="fixed">
                                                        <fo:table-body>
                                                            <!-- Very High -->
                                                            <fo:table-row>
                                                                <fo:table-cell>
                                                                    <fo:block><xsl:value-of select="$strings/Very_High"/>:</fo:block>
                                                                </fo:table-cell>
                                                                <fo:table-cell>
                                                                    <fo:block text-align="right"><xsl:value-of select="$community-assessment/rqm:veryHighAmount"/></fo:block>
                                                                </fo:table-cell>
                                                            </fo:table-row>
                                                            <!-- High -->
                                                            <fo:table-row>
                                                                <fo:table-cell>
                                                                    <fo:block><xsl:value-of select="$strings/High"/>:</fo:block>
                                                                </fo:table-cell>
                                                                <fo:table-cell>
                                                                    <fo:block text-align="right"><xsl:value-of select="$community-assessment/rqm:highAmount"/></fo:block>
                                                                </fo:table-cell>
                                                            </fo:table-row>
                                                            <!-- Neutral -->
                                                            <fo:table-row>
                                                                <fo:table-cell>
                                                                    <fo:block><xsl:value-of select="$strings/Neutral"/>:</fo:block>
                                                                </fo:table-cell>
                                                                <fo:table-cell>
                                                                    <fo:block text-align="right"><xsl:value-of select="$community-assessment/rqm:neutralAmount"/></fo:block>
                                                                </fo:table-cell>
                                                            </fo:table-row>
                                                            <!-- Low -->
                                                            <fo:table-row>
                                                                <fo:table-cell>
                                                                    <fo:block><xsl:value-of select="$strings/Low"/>:</fo:block>
                                                                </fo:table-cell>
                                                                <fo:table-cell>
                                                                    <fo:block text-align="right"><xsl:value-of select="$community-assessment/rqm:lowAmount"/></fo:block>
                                                                </fo:table-cell>
                                                            </fo:table-row>
                                                            <!-- Very Low -->
                                                            <fo:table-row>
                                                                <fo:table-cell>
                                                                    <fo:block><xsl:value-of select="$strings/Very_Low"/></fo:block>
                                                                </fo:table-cell>
                                                                <fo:table-cell>
                                                                    <fo:block text-align="right"><xsl:value-of select="$community-assessment/rqm:veryLowAmount"/></fo:block>
                                                                </fo:table-cell>
                                                            </fo:table-row>
                                                        </fo:table-body>
                                                    </fo:table>
                                                </fo:block>
                                            </fo:table-cell>
                                        </fo:table-row>
                                    </fo:table-body>
                                </fo:table>
                            </fo:block>
                        </fo:table-cell>
                        <!-- Risk Discussion -->
                        <fo:table-cell>
                            <fo:block>
                                <xsl:if test="$community-assessment/*[local-name()='riskComment']">
                                    <fo:table xsl:use-attribute-sets="table" display-align="center" space-before="10pt">
                                        <fo:table-header xsl:use-attribute-sets="risk-discussion-header">
                                            <fo:table-row height="15">
                                                <fo:table-cell>
                                                    <fo:block text-indent="3pt"><xsl:value-of select="$strings/Discussion"/></fo:block>
                                                </fo:table-cell>
                                            </fo:table-row>
                                        </fo:table-header>
                                        <fo:table-body>
                                            <xsl:for-each select="$community-assessment/*[local-name()='riskComment']">
                                                <fo:table-row height="12">
                                                    <fo:table-cell>
                                                        <fo:block text-indent="6pt">
                                                            <fo:inline><xsl:value-of select="position()"/>. </fo:inline>
                                                            <fo:inline space-start="7pt"><xsl:value-of select="prep:creator/@username"/></fo:inline>
                                                            <fo:inline space-start="10pt"><xsl:value-of select="time:formatMediumDateShortTime(*[local-name() = 'commentDate'], $locale)"/></fo:inline>
                                                        </fo:block>
                                                    </fo:table-cell>
                                                </fo:table-row>
                                                <fo:table-row height="14">
                                                    <fo:table-cell>
                                                        <fo:block text-indent="22pt"><xsl:value-of select="*[local-name() = 'comment']"/></fo:block>
                                                    </fo:table-cell>
                                                </fo:table-row>
                                            </xsl:for-each>
                                        </fo:table-body>
                                    </fo:table>
                                </xsl:if>
                            </fo:block>
                        </fo:table-cell>
                    </fo:table-row>
                </fo:table-body>
            </fo:table>
        </xsl:if>
    </xsl:template>
    
    <xsl:template name="buildRiskAssessmentSection">
        <!-- deprecated -->
        <xsl:param name="strings"/>
        
        <xsl:param name="risk"/>
        <xsl:param name="risk-assessment-summary"/>
        
        <!-- section header -->
        <fo:block xsl:use-attribute-sets="section-heading"><xsl:value-of select="$strings/Risk_Assessment"/></fo:block>
        
        <!-- section content -->
        <!-- Risk Assessment Summary (rich text) -->
        <xsl:call-template name="buildRichTextSection">
            <xsl:with-param name="rich-text" select="$risk-assessment-summary"/>
        </xsl:call-template>
        
        <!-- Risk Assessment: OOOOO (value) -->
        <fo:block space-before="10pt" font-weight="bold"><xsl:value-of select="$strings/Risk_Assessment"/><xsl:value-of select="concat(':',' ')"/>
            <xsl:call-template name="drawRiskIconSet">
                <xsl:with-param name="iconCount"><xsl:value-of select="$risk/rqm:riskLevel"/></xsl:with-param>
                <xsl:with-param name="positiveIcon">risk-selected_obj.gif</xsl:with-param>
                <xsl:with-param name="negativeIcon">risk-unselect_obj.gif</xsl:with-param>
            </xsl:call-template>
        </fo:block>
        
        <xsl:if test="$preferences/pref:renderEmptyTables='true' or $risk/rqm:factors/rqm:factor">
            <fo:table xsl:use-attribute-sets="artifact-table">
                <fo:table-header xsl:use-attribute-sets="artifact-table-header">
                    <!-- Risk -->                                           
                    <fo:table-cell xsl:use-attribute-sets="artifact-table-header-cell">
                        <fo:block><xsl:value-of select="$strings/Risk"/></fo:block>
                    </fo:table-cell>
                    <!-- Importance -->
                    <fo:table-cell xsl:use-attribute-sets="artifact-table-header-cell">
                        <fo:block><xsl:value-of select="$strings/Importance"/></fo:block>
                    </fo:table-cell>
                    <!-- Mitigation Action -->
                    <fo:table-cell xsl:use-attribute-sets="artifact-table-header-cell">
                        <fo:block><xsl:value-of select="$strings/Mitigation_Action"/></fo:block>
                    </fo:table-cell>
                </fo:table-header>
                <fo:table-body>
                    <xsl:choose>
                        <xsl:when test="$risk/rqm:factors/rqm:factor">
                            <xsl:for-each select="$risk/rqm:factors/rqm:factor">
                                <xsl:variable name="riskName" select="rqm:name"/>
                                
                                <fo:table-row>
                                    <!-- Risk -->
                                    <fo:table-cell xsl:use-attribute-sets="artifact-table-body-cell">
                                        <fo:block-container overflow="hidden">
	                                        <fo:block>
	                                            <xsl:choose>
	                                                <!-- built in risk factors -->
	                                                <xsl:when test="$strings/*[name()=$riskName]">
	                                                    <xsl:value-of select="$strings/*[name()=$riskName]"/></xsl:when>
	                                                <!-- custom risk factors -->
	                                                <xsl:otherwise>
	                                                    <xsl:value-of select="rqm:name"/></xsl:otherwise>
	                                            </xsl:choose>
	                                        </fo:block>
                                        </fo:block-container>
                                    </fo:table-cell>
                                    <!-- Importance -->
                                    <fo:table-cell xsl:use-attribute-sets="artifact-table-body-cell">
                                        <fo:block-container overflow="hidden">
	                                        <fo:block>
	                                            <xsl:call-template name="drawRiskIconSet">
	                                                <xsl:with-param name="iconCount"><xsl:value-of select="rqm:weight"/></xsl:with-param>
	                                                <xsl:with-param name="positiveIcon">importance_selected_obj.gif</xsl:with-param>
	                                                <xsl:with-param name="negativeIcon">importance_unselected_obj.gif</xsl:with-param>
	                                            </xsl:call-template>    
	                                        </fo:block>
                                        </fo:block-container>
                                    </fo:table-cell>
                                    <!-- Mitigation Action -->
                                    <fo:table-cell xsl:use-attribute-sets="artifact-table-body-cell">
                                        <fo:block-container overflow="hidden">
                                        	<fo:block><xsl:value-of select="rqm:mitigationAction"/></fo:block>
                                    	</fo:block-container>
                                    </fo:table-cell>
                                </fo:table-row>
                            </xsl:for-each>
                        </xsl:when>
                        <xsl:otherwise>
                            <fo:table-row>
                                <xsl:call-template name="empty-columns">
                                    <xsl:with-param name="column-count" select="3"/>
                                </xsl:call-template>
                            </fo:table-row>
                        </xsl:otherwise>
                    </xsl:choose>
                </fo:table-body>
            </fo:table>
        </xsl:if>
        
        <!-- Community Risk: -->
        <xsl:if test="$risk/rqm:commnunityAssessment">
            <fo:block space-before="10pt" font-weight="bold"><xsl:value-of select="$strings/Community_Risk"/></fo:block>
            <fo:block><xsl:value-of select="$strings/Risk_Level"/><xsl:value-of select="concat(':',' ')"/>
                <xsl:call-template name="drawRiskIconSet">
                    <xsl:with-param name="iconCount"><xsl:value-of select="$risk/rqm:commnunityAssessment/rqm:riskLevel"/></xsl:with-param>
                    <xsl:with-param name="positiveIcon">risk-selected_obj.gif</xsl:with-param>
                    <xsl:with-param name="negativeIcon">risk-unselect_obj.gif</xsl:with-param>
                </xsl:call-template></fo:block>
            <fo:table xsl:use-attribute-sets="border" table-layout="fixed" inline-progression-dimension="20%">
                <fo:table-body>
                    <fo:table-row>
                        <fo:table-cell padding="5pt">
                            <fo:block>
                                <fo:table table-layout="fixed">
                                    <fo:table-body>
                                        <!-- Very High -->
                                        <fo:table-row>
                                            <fo:table-cell>
                                                <fo:block><xsl:value-of select="$strings/Very_High"/>:</fo:block>
                                            </fo:table-cell>
                                            <fo:table-cell>
                                                <fo:block text-align="right"><xsl:value-of select="$risk/rqm:commnunityAssessment/rqm:veryHighAmount"/></fo:block>
                                            </fo:table-cell>
                                        </fo:table-row>
                                        <!-- High -->
                                        <fo:table-row>
                                            <fo:table-cell>
                                                <fo:block><xsl:value-of select="$strings/High"/>:</fo:block>
                                            </fo:table-cell>
                                            <fo:table-cell>
                                                <fo:block text-align="right"><xsl:value-of select="$risk/rqm:commnunityAssessment/rqm:highAmount"/></fo:block>
                                            </fo:table-cell>
                                        </fo:table-row>
                                        <!-- Neutral -->
                                        <fo:table-row>
                                            <fo:table-cell>
                                                <fo:block><xsl:value-of select="$strings/Neutral"/>:</fo:block>
                                            </fo:table-cell>
                                            <fo:table-cell>
                                                <fo:block text-align="right"><xsl:value-of select="$risk/rqm:commnunityAssessment/rqm:neutralAmount"/></fo:block>
                                            </fo:table-cell>
                                        </fo:table-row>
                                        <!-- Low -->
                                        <fo:table-row>
                                            <fo:table-cell>
                                                <fo:block><xsl:value-of select="$strings/Low"/>:</fo:block>
                                            </fo:table-cell>
                                            <fo:table-cell>
                                                <fo:block text-align="right"><xsl:value-of select="$risk/rqm:commnunityAssessment/rqm:lowAmount"/></fo:block>
                                            </fo:table-cell>
                                        </fo:table-row>
                                        <!-- Very Low -->
                                        <fo:table-row>
                                            <fo:table-cell>
                                                <fo:block><xsl:value-of select="$strings/Very_Low"/></fo:block>
                                            </fo:table-cell>
                                            <fo:table-cell>
                                                <fo:block text-align="right"><xsl:value-of select="$risk/rqm:commnunityAssessment/rqm:veryLowAmount"/></fo:block>
                                            </fo:table-cell>
                                        </fo:table-row>
                                    </fo:table-body>
                                </fo:table>
                            </fo:block>
                        </fo:table-cell>
                    </fo:table-row>
                </fo:table-body>
            </fo:table>
        </xsl:if>
    </xsl:template>
    
    <xsl:template name="drawRiskIconSet">
    <!-- Calls templates to draw the actual icons after doing math. -->
        <xsl:param name="iconCount"/>
        <xsl:param name="positiveIcon"/>
        <xsl:param name="negativeIcon"/>
        <xsl:choose>
            <xsl:when test="$iconCount">
                <xsl:call-template name="drawSpecificRiskIcons"><xsl:with-param name="iconCount" select="$iconCount"/><xsl:with-param name="filename" select="$positiveIcon"/></xsl:call-template>
                <xsl:call-template name="drawSpecificRiskIcons"><xsl:with-param name="iconCount" select="5-$iconCount"/><xsl:with-param name="filename" select="$negativeIcon"/></xsl:call-template>
            </xsl:when>
            <xsl:otherwise>
                <xsl:call-template name="drawSpecificRiskIcons"><xsl:with-param name="iconCount">5</xsl:with-param><xsl:with-param name="filename" select="$negativeIcon"/></xsl:call-template>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <xsl:template name="drawSpecificRiskIcons">
    <!-- Recursively draws the specified icon. -->
        <xsl:param name="iconCount"/>
        <xsl:param name="filename"/>
        <xsl:if test="$iconCount > 0">
            <fo:external-graphic src="{concat('basecache://com.ibm.asq.common.web/resources/ui/internal/images/icons/elcl16/',$filename)}" scaling="uniform" scaling-method="integer-pixels" content-width="65%"/>
            <xsl:call-template name="drawSpecificRiskIcons"><xsl:with-param name="iconCount" select="$iconCount - 1"/><xsl:with-param name="filename" select="$filename" /></xsl:call-template>
        </xsl:if>
    </xsl:template>
    
    <xsl:template name="empty-columns">
        <xsl:param name="column-count"/>
        
        <xsl:if test="$column-count > 0">
            <fo:table-cell><fo:block/></fo:table-cell>
            
            <xsl:call-template name="empty-columns">
                <xsl:with-param name="column-count" select="$column-count - 1"/>
            </xsl:call-template>
        </xsl:if>
    </xsl:template>
    
</xsl:stylesheet>