<?xml version="1.0" encoding="utf-8"?>
<!-- 
/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2014. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 -->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:fo="http://www.w3.org/1999/XSL/Format"
	xmlns:prep="http://com.ibm.rqm.print/prep"
	xmlns:rqm="http://jazz.net/xmlns/alm/qm/v0.1/"
	xmlns:dc="http://purl.org/dc/elements/1.1/"
	xmlns:alm="http://jazz.net/xmlns/alm/v0.1/"
	xmlns:ewi="http://jazz.net/xmlns/alm/qm/v0.1/executionworkitem/v0.1"
	xmlns:er="http://jazz.net/xmlns/alm/qm/v0.1/executionresult/v0.1"
	xmlns:time="xalan://com.ibm.rqm.print.render.fop.xml.xslt.extension.TimeUtilsExtension"
	xmlns:utils="xalan://com.ibm.rqm.print.render.fop.xml.xslt.extension.UtilsExtension"
	xmlns:terms="http://purl.org/dc/terms/"
    xmlns:log="xalan://com.ibm.rqm.print.render.fop.xml.xslt.extension.LoggingExtension"
	exclude-result-prefixes="xsl rqm dc alm terms">
	
	<xsl:import  href="cache://stylesheets/render/abstract-table-summary.xsl"/>
	
	<xsl:template name="buildExecutionResultTable">
		<xsl:param name="columns"/>
		<xsl:param name="results"/>
		
		<fo:table xsl:use-attribute-sets="artifact-table">
		
			<xsl:for-each select="$columns">
				<fo:table-column column-width="{concat(prep:width, '%')}"/>
			</xsl:for-each>
			
			<fo:table-header xsl:use-attribute-sets="artifact-table-header">
				<xsl:for-each select="$columns">
					<fo:table-cell xsl:use-attribute-sets="artifact-table-header-cell">
						<xsl:variable name="header" select="prep:name"/>
						<fo:block>
                        	<xsl:choose>
                        		<xsl:when test="starts-with($header, 'isCategoryColumn')">
                        			<xsl:value-of select="substring-after($header, 'isCategoryColumn')"/>
                        		</xsl:when>
                        		<xsl:when test="starts-with($header, 'isCustomColumn')">
                        			<xsl:value-of select="substring-after($header, 'isCustomColumn')"/> 			
                        		</xsl:when>
                        		<xsl:otherwise>
                        			<xsl:choose>
                        				<xsl:when test="$header='stateIds'">
                        					<xsl:value-of select="$strings/Status"/>
                        				</xsl:when>
                        				<xsl:when test="$header='id'">
                        					<fo:block xsl:use-attribute-sets="id-header"><xsl:value-of select="$strings/id"/></fo:block>
                        				</xsl:when>
                        				<xsl:otherwise>
                        					<xsl:value-of select="$strings/*[name()=translate($header, ' ', '_')]"/>	
                        				</xsl:otherwise>
                        			</xsl:choose>
                        		</xsl:otherwise>
                        	</xsl:choose>
                        </fo:block>
					</fo:table-cell>
				</xsl:for-each>
			</fo:table-header>
			
			<fo:table-body>
				<xsl:choose>
					<xsl:when test="$results">
						<xsl:for-each select="$results">
							<xsl:variable name="result" select="current()"/>
							<fo:table-row>
								<xsl:for-each select="$columns">
								
									<xsl:apply-templates mode="er-value" select="current()">
										<xsl:with-param name="artifact" select="$result"/>
									</xsl:apply-templates>
								</xsl:for-each>
							</fo:table-row>
						</xsl:for-each>
					</xsl:when>
					<xsl:otherwise>
						<!-- inserts emptly columns so that the table header can still appear in the document -->
						<fo:table-row>
							<xsl:for-each select="$columns">
								<fo:table-cell><fo:block/></fo:table-cell>
							</xsl:for-each>
						</fo:table-row>
					</xsl:otherwise>
				</xsl:choose>
			</fo:table-body>
		</fo:table>
	</xsl:template>
	
    <xsl:template mode="er-value" match="prep:column">
        <xsl:param name="artifact"/>
        
        <fo:table-cell xsl:use-attribute-sets="artifact-table-body-cell">
        	<fo:block-container overflow="hidden">
	            <!--  match type-specific value templates locally defined in this file  -->
	            <xsl:apply-templates mode="er-value" select="prep:name">
	                <xsl:with-param name="artifact" select="$artifact"/>
	            </xsl:apply-templates>
	            <!--  and now match generic value templates from abstract-table-summary.xsl  -->
	            <xsl:apply-templates select="prep:name">
	                <xsl:with-param name="artifact" select="$artifact"/>
	            </xsl:apply-templates>
            </fo:block-container>
        </fo:table-cell>
    </xsl:template>
	
	<!-- ID -->
	<xsl:template mode="er-value" match="prep:column/prep:name[(.='id' or .='ID')]">
		<xsl:param name="artifact"/>
		<fo:block xsl:use-attribute-sets="id-cell">
			<fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{$artifact/dc:identifier}">
				<xsl:value-of select="$artifact/rqm:webId"/>
			</fo:basic-link>
		</fo:block>
	</xsl:template>
	
	<!-- Name -->
	<xsl:template mode="er-value" match="prep:column/prep:name[(.='name' or .='Name')]">
		<xsl:param name="artifact"/>
		
		<fo:block>
			<fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{$artifact/dc:identifier}">
				<xsl:apply-templates select="$artifact/dc:title"/>
			</fo:basic-link>
		</fo:block>
	</xsl:template>

	<!-- Locked -->
	<xsl:template mode="er-value" match="prep:column/prep:name[.='locked']">
		<xsl:param name="artifact"/>
		
		<fo:block><xsl:apply-templates select="$artifact/rqm:locked"/></fo:block>
	</xsl:template>
	
	<!-- Owner -->
	<xsl:template mode="er-value" match="prep:column/prep:name[(.='ownerUUIDs' or .='Owner')]">
		<xsl:param name="artifact"/>
		
		<fo:block><xsl:apply-templates select="$artifact/prep:owner/@username"/></fo:block>
	</xsl:template>
	
	<!-- Created By -->
	<xsl:template mode="er-value" match="prep:column/prep:name[.='originator']">
		<xsl:param name="artifact"/>
		
		<fo:block><xsl:apply-templates select="$artifact/prep:creator/@username"/></fo:block>
	</xsl:template>
	
	<!-- Weight -->
	<xsl:template mode="er-value" match="prep:column/prep:name[.='weight']">
		<xsl:param name="artifact"/>
		
		<fo:block><xsl:value-of select="$artifact/er:weight"/></fo:block>
	</xsl:template>
	
	<!-- State -->
    <xsl:template mode="er-value" match="prep:column/prep:name[.='documentStateIds']">
        <xsl:param name="artifact"/>
        <xsl:if test="$artifact">
 	        <xsl:variable name="approvalstateid" select="$artifact/er:approvalstate"/>                  
			<xsl:variable name="approvalstatename" select="$strings/*[name()=$approvalstateid]"/>    
	        <fo:block>
	            <xsl:choose>
	            	<xsl:when test="$approvalstatename">
	                	<xsl:value-of select="$approvalstatename"/>
					</xsl:when>
	                <xsl:otherwise>
	                	<xsl:variable name="workflowType" select="'TESTCASERESULT'"/>    

						<xsl:value-of select="utils:getStateName($projectArea, $workflowType, $approvalstateid)"/>
	                </xsl:otherwise>
	            </xsl:choose>
	        </fo:block>
	    </xsl:if>            
    </xsl:template>
	
	 <!-- Status (or Actual Result) -->
    <xsl:template mode="er-value" match="prep:column/prep:name[(.='stateIds' or .='Actual Result')]">
        <xsl:param name="artifact"/>
        <xsl:if test="$artifact">
	        <xsl:variable name="state" select="$artifact/alm:state"/>
	    	<xsl:variable name="customStateName" select="$artifact/alm:state/@value"/>
	    	
	    	<fo:block>
	    		<fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{$artifact/dc:identifier}">
		    		<xsl:call-template name="printResultStateIcon">
		    			<xsl:with-param name="size">45%</xsl:with-param>
		    			<xsl:with-param name="priority_id"><xsl:value-of select="$state"/></xsl:with-param>
		    		</xsl:call-template>
		    		<xsl:text> </xsl:text>
		    		<xsl:choose>
		    			<xsl:when test="$customStateName">
		    				<xsl:value-of select="$customStateName"/></xsl:when>
		    			<xsl:otherwise>
		    				<xsl:value-of select="$strings/*[name()=$state]"/></xsl:otherwise>
		    		</xsl:choose>
	    		</fo:basic-link>
	    	</fo:block>
	    </xsl:if>
    </xsl:template>
    
    <!-- Test Case -->
	<xsl:template mode="er-value" match="prep:column/prep:name[.='testCase']">
		<xsl:param name="artifact"/>

  		<xsl:if test="$artifact">
			<xsl:variable name="testcase" select="$artifact/rqm:testcase"/>
			
			<xsl:choose>
				<xsl:when test="$testcase">
					<fo:block>
						<fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{$testcase/dc:identifier}">
							<xsl:apply-templates select="$testcase/dc:title"/>
						</fo:basic-link>
					</fo:block>
				</xsl:when>
				<xsl:otherwise><fo:block/></xsl:otherwise>
			</xsl:choose>
		</xsl:if>
	</xsl:template>
	
	<!-- Test Script -->
	<xsl:template mode="er-value" match="prep:column/prep:name[(.='testScript' or .='Script Name')]">
		<xsl:param name="artifact"/>
		<xsl:if test="$artifact">
			<xsl:variable name="testscript" select="$artifact/rqm:executionscript"/>
			
			<xsl:choose>
				<xsl:when test="$testscript">
					<fo:block>
						<fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{$testscript/dc:identifier}">
							<xsl:apply-templates select="$testscript/dc:title"/>
						</fo:basic-link>
					</fo:block>		
				</xsl:when>
				<xsl:otherwise><fo:block><xsl:value-of select="$strings/None"/></fo:block></xsl:otherwise>
			</xsl:choose>
		</xsl:if>
	</xsl:template>
	
	<!-- Script Type -->
	<xsl:template mode="er-value" match="prep:column/prep:name[.='Script Type']">
		<xsl:param name="artifact"/>
		<xsl:variable name="scriptType" select="$artifact/rqm:executionscript/terms:type"/>
		
		<fo:block><xsl:value-of select="$strings/*[name()=$scriptType]"/></fo:block>
	</xsl:template>
	
	<!-- Test Plan -->
	<xsl:template mode="er-value" match="prep:column/prep:name[.='testPlan']">
		<xsl:param name="artifact"/>
		<xsl:variable name="testplan" select="$artifact/rqm:testplan"/>
		
		<xsl:choose>
			<xsl:when test="$testplan">
				<fo:block>
					<fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{$testplan/dc:identifier}">
						<xsl:apply-templates select="$testplan/dc:title"/>
					</fo:basic-link>
				</fo:block>
			</xsl:when>
			<xsl:otherwise><fo:block/></xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<!-- Iteration -->
	<xsl:template mode="er-value" match="prep:column/prep:name[.='testPhases']">
		<xsl:param name="artifact"/>
		<xsl:variable name="iteration" select="$artifact/rqm:testphase"/>
		
		<xsl:choose>
			<xsl:when test="$iteration">
				<fo:block>
					<fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{$iteration/dc:identifier}">
						<xsl:apply-templates select="$iteration/dc:title"/>
					</fo:basic-link>
				</fo:block>
			</xsl:when>
			<xsl:otherwise><fo:block><xsl:value-of select="$strings/Unassigned"/></fo:block></xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<!-- Start Date -->
	<xsl:template mode="er-value" match="prep:column/prep:name[.='Start Date']">
		<xsl:param name="artifact"/>
		
		<fo:block><xsl:value-of select="time:formatMediumDate($artifact/er:starttime, $locale)"/></fo:block>
	</xsl:template>
	
	<!-- Start Time -->
	<xsl:template mode="er-value" match="prep:column/prep:name[.='Start Time']">
		<xsl:param name="artifact"/>
		
		<fo:block>
			<xsl:choose>
				<xsl:when test="../../@key='RQM-TER-HISTORY-TABLEVIEWER'">
					<xsl:value-of select="time:formatMediumDate($artifact/er:starttime, $locale)"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="time:formatShortTime($artifact/er:starttime, $locale)"/>
				</xsl:otherwise>
			</xsl:choose>	
		</fo:block>
	</xsl:template>
	
	<!-- Completed -->
	<xsl:template mode="er-value" match="prep:column/prep:name[.='completed']">
		<xsl:param name="artifact"/>
		
		<fo:block><xsl:value-of select="time:formatMediumDate($artifact/er:endtime, $locale)"/></fo:block>
	</xsl:template>
	
	<!-- Total Run Time -->
	<xsl:template mode="er-value" match="prep:column/prep:name[.='Total Run Time']">
		<xsl:param name="artifact"/>
		<fo:block><xsl:value-of select="time:formatTimeEstimateNoMillis($artifact/er:starttime, $artifact/er:endtime, $locale)"/></fo:block>
	</xsl:template>
	
	<!-- Test Execution Record Name -->
	<xsl:template mode="er-value" match="prep:column/prep:name[.='TER']">
		<xsl:param name="artifact"/>
		<xsl:variable name="ter" select="$artifact/rqm:executionworkitem"/>
		
		<fo:block>
			<xsl:if test="$ter">
				<fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="$ter/dc:identifier">
					<xsl:apply-templates select="$ter/dc:title"/>
				</fo:basic-link>
			</xsl:if>
		</fo:block>
	</xsl:template>
	
	<!-- Test Environment -->
	<xsl:template mode="er-value" match="prep:column/prep:name[.='configuration']">
		<xsl:param name="artifact"/>
		<xsl:variable name="configuration" select="$artifact/rqm:executionworkitem/rqm:configuration"/>
		
		<xsl:choose>
			<xsl:when test="$configuration">
				<fo:block>
					<fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{$configuration/dc:identifier}">
						<xsl:apply-templates select="$configuration/dc:title"/>
					</fo:basic-link>
				</fo:block>
			</xsl:when>
			<xsl:otherwise><fo:block><xsl:value-of select="$strings/Unassigned"></xsl:value-of></fo:block></xsl:otherwise>
		</xsl:choose>
	</xsl:template>
    
	
	<!-- Build -->
	<xsl:template mode="er-value" match="prep:column/prep:name[.='buildRecordName']">
		<xsl:param name="artifact"/>
		<xsl:variable name="build" select="$artifact/rqm:buildrecord"/>
		
		<xsl:choose>
			<xsl:when test="$build">
				<fo:block>
					<fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{$build/dc:identifier}">
						<xsl:apply-templates select="$build/dc:title"/>
					</fo:basic-link>
				</fo:block>
			</xsl:when>
			<xsl:otherwise><fo:block><xsl:value-of select="$strings/Unassigned"/></fo:block></xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<!-- Defects -->
	<xsl:template mode="er-value" match="prep:column/prep:name[.='defects']">
		<xsl:param name="artifact"/>
		<xsl:variable name="defects" select="$artifact/rqm:defect"/>			
		
		<fo:block>
			<xsl:choose>
				<xsl:when test="$defects">
					<xsl:for-each select="$defects">
						<xsl:if test="position() > 1">, </xsl:if>
						
						<fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="$defects/@href">
							<xsl:apply-templates select="$defects/@summary"/>
						</fo:basic-link>
					</xsl:for-each>
				</xsl:when>
				<xsl:otherwise>--</xsl:otherwise>
			</xsl:choose>	
		</fo:block>
	</xsl:template>
	
	<!-- Passed Points -->
	<xsl:template mode="er-value" match="prep:column/prep:name[.='pointsPass']">
		<xsl:param name="artifact"/>
		<fo:block><xsl:value-of select="$artifact/er:pointspassed"/></fo:block>
	</xsl:template>
	
	<!-- Failed Points -->
	<xsl:template mode="er-value" match="prep:column/prep:name[.='pointsFail']">
		<xsl:param name="artifact"/>
		<fo:block><xsl:value-of select="$artifact/er:pointsfailed"/></fo:block>
	</xsl:template>
	
	<!-- Blocked Points -->
	<xsl:template mode="er-value" match="prep:column/prep:name[.='pointsBlocked']">
		<xsl:param name="artifact"/>
		<fo:block><xsl:value-of select="$artifact/er:pointsblocked"/></fo:block>
	</xsl:template>
	
	<!-- Inconclusive Points -->
	<xsl:template mode="er-value" match="prep:column/prep:name[.='pointsInconclusive']">
		<xsl:param name="artifact"/>
		<fo:block><xsl:value-of select="$artifact/er:pointsinconclusive"/></fo:block>
	</xsl:template>
	
	<!-- Attempted Points -->
	<xsl:template mode="er-value" match="prep:column/prep:name[.='pointsAttempted']">
		<xsl:param name="artifact"/>
		<fo:block><xsl:value-of select="$artifact/er:pointsattempted"/></fo:block>
	</xsl:template>
	
	<!-- TestedBy  -->
	<xsl:template mode="er-value" match="prep:column/prep:name[.='testedBy']">
		<xsl:param name="artifact"/>
		<fo:block xsl:use-attribute-sets="value">
			<xsl:for-each select="$artifact/er:testedby/prep:tester">
				<fo:block><xsl:value-of select="@username"/></fo:block>
				<!-- fo:block><xsl:value-of select="$artifact/er:testedby"/></fo:block -->
				<xsl:if test="not(position()=last())">
					<xsl:text>, </xsl:text>
				</xsl:if>
			</xsl:for-each>
		</fo:block>
	</xsl:template>
	
	<!-- Audit -->
	<xsl:template mode="er-value" match="prep:column/prep:name[.='testCaseAtExecutionDocumentStateIds']">
		<xsl:param name="artifact"/>
        <xsl:if test="$artifact">
			<xsl:variable name="testcasestateatexecutionstateid" select="$artifact/er:testcasestateatexecution"/>
			<xsl:if test="$testcasestateatexecutionstateid">
				<xsl:variable name="testcasestateatexecutionstatename" select="$strings/*[name()=$testcasestateatexecutionstateid]"/>    
		        <fo:block>
		            <xsl:choose>
		            	<xsl:when test="$testcasestateatexecutionstatename">
		                	<xsl:value-of select="$testcasestateatexecutionstatename"/>
						</xsl:when>
		                <xsl:otherwise>
		                	<xsl:variable name="workflowType" select="'TESTCASE'"/>    
							<xsl:value-of select="utils:getStateName($projectArea, $workflowType, $testcasestateatexecutionstateid)"/>
						</xsl:otherwise>
		            </xsl:choose>
		        </fo:block>
	        </xsl:if>
	    </xsl:if>            
	</xsl:template>
	<xsl:template mode="er-value" match="prep:column/prep:name[.='testPlanAtExecutionDocumentStateIds']">
		<xsl:param name="artifact"/>
        <xsl:if test="$artifact">
			<xsl:variable name="testplanstateatexecutionstateid" select="$artifact/er:testplanstateatexecution"/>
			<xsl:if test="$testplanstateatexecutionstateid">
				<xsl:variable name="testplanstateatexecutionstatename" select="$strings/*[name()=$testplanstateatexecutionstateid]"/>    
		        <fo:block>
		            <xsl:choose>
		            	<xsl:when test="$testplanstateatexecutionstatename">
		                	<xsl:value-of select="$testplanstateatexecutionstatename"/>
						</xsl:when>
		                <xsl:otherwise>
		                	<xsl:variable name="workflowType" select="'TESTPLAN'"/>    
		                	<xsl:value-of select="utils:getStateName($projectArea, $workflowType, $testplanstateatexecutionstateid)"/>
		                </xsl:otherwise>
		            </xsl:choose>
		        </fo:block>
	        </xsl:if>
	    </xsl:if>            
	</xsl:template>
	<xsl:template mode="er-value" match="prep:column/prep:name[.='testScriptAtExecutionDocumentStateIds']">
		<xsl:param name="artifact"/>
        <xsl:if test="$artifact">
			<xsl:variable name="testscriptstateatexecutionstateid" select="$artifact/er:testscriptstateatexecution"/>
			<xsl:if test="$testscriptstateatexecutionstateid">
				<xsl:variable name="testscriptstateatexecutionstatename" select="$strings/*[name()=$testscriptstateatexecutionstateid]"/>    
		        <fo:block>
		            <xsl:choose>
		            	<xsl:when test="$testscriptstateatexecutionstatename">
		                	<xsl:value-of select="$testscriptstateatexecutionstatename"/>
						</xsl:when>
		                <xsl:otherwise>
		                	<xsl:variable name="workflowType" select="'TESTSCRIPT'"/>    
		                	<xsl:value-of select="utils:getStateName($projectArea, $workflowType, $testscriptstateatexecutionstateid)"/>
		                </xsl:otherwise>
		            </xsl:choose>
		        </fo:block>
			</xsl:if>
	    </xsl:if>            
	</xsl:template>
	
	<!-- Execution Result -->
	<xsl:template mode="er-value" match="prep:column/prep:name[.='Execution Result']">
		<xsl:param name="artifact"/>
		
		<fo:block>
			<fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{$artifact/dc:identifier}">
				<xsl:value-of select="$artifact/rqm:webId"/>
			</fo:basic-link>
		</fo:block>
	</xsl:template>

    <!-- isCurrent -->
    <xsl:template mode="er-value" match="prep:column/prep:name[.='isCurrent' ]">
        <xsl:param name="artifact"/>
		<fo:block><xsl:value-of select="$artifact/er:isCurrent"/></fo:block>
    </xsl:template>
		   
</xsl:stylesheet>