<?xml version="1.0" encoding="utf-8"?>
<!-- 
/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2015. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 -->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:fo="http://www.w3.org/1999/XSL/Format"
	xmlns:prep="http://com.ibm.rqm.print/prep"
	xmlns:pref="http://com.ibm.rqm.export.pdf/preferences"
	xmlns:pref_table="http://com.ibm.rqm.export.pdf/preferences/table"
	xmlns:rqm="http://jazz.net/xmlns/alm/qm/v0.1/"
	xmlns:dc="http://purl.org/dc/elements/1.1/"
	xmlns:alm="http://jazz.net/xmlns/alm/v0.1/"
	xmlns:ewi="http://jazz.net/xmlns/alm/qm/v0.1/executionworkitem/v0.1"
	xmlns:er="http://jazz.net/xmlns/alm/qm/v0.1/executionresult/v0.1"
	xmlns:log="xalan://com.ibm.rqm.print.render.fop.xml.xslt.extension.LoggingExtension"
	xmlns:time="xalan://com.ibm.rqm.print.render.fop.xml.xslt.extension.TimeUtilsExtension"
	xmlns:utils="xalan://com.ibm.rqm.print.render.fop.xml.xslt.extension.UtilsExtension"
	exclude-result-prefixes="xsl rqm dc alm">
	
	<xsl:import  href="cache://stylesheets/render/abstract-table-summary.xsl"/>
	
	<!-- for debugging only -->
	<!--<xsl:include href="cache://stylesheets/render/_config/attribute-sets.xsl"/>-->
	
	<xsl:template name="buildEWITable">
		<xsl:param name="columns"/>
		<xsl:param name="records"/>
		
		<fo:table xsl:use-attribute-sets="artifact-table">
			<xsl:for-each select="$columns">
				<fo:table-column column-width="{concat(pref_table:width, '%')}"/>
			</xsl:for-each>
			
			<fo:table-header xsl:use-attribute-sets="artifact-table-header">
				<xsl:for-each select="$columns">
					<fo:table-cell xsl:use-attribute-sets="artifact-table-header-cell">
						<xsl:variable name="header" select="pref_table:name"/>
						<fo:block>
                        	<xsl:choose>
                        		<xsl:when test="starts-with($header, 'isCategoryColumn') and contains($header, 'parentCategory') and substring-after($header, 'parentCategory') = ''">
                        			<fo:block>
                        				<fo:inline space-end="2pt">
											<fo:external-graphic scaling="uniform" scaling-method="integer-pixels" 
											content-width="45%" src="'basecache://com.ibm.asq.common.web/resources/ui/internal/images/icons/obj16/qm_testcase16.gif'"/>
										</fo:inline>
                        				<xsl:value-of select="substring-before(substring-after($header, 'isCategoryColumn'), 'parentCategory')"/>
                        			</fo:block>
                        		</xsl:when>
                        		<xsl:when test="starts-with($header, 'isCategoryColumn') and not (contains($header, 'parentCategory'))">
									<xsl:value-of select="substring-after($header, 'isCategoryColumn')"/>
                        		</xsl:when>
                        		<xsl:when test="starts-with($header, 'isCustomColumn')">
                        			<xsl:value-of select="substring-after($header, 'isCustomColumn')"/> 			
                        		</xsl:when>
                        		<xsl:when test="$header='id'">
                        			<fo:block xsl:use-attribute-sets="id-header"><xsl:value-of select="$strings/id"/></fo:block>
                        		</xsl:when>
                        		<xsl:when test="$header='priority'">
                        			<fo:block xsl:use-attribute-sets="priority-header"><xsl:value-of select="$strings/Priority"/></fo:block>
                        		</xsl:when>
                        		<xsl:when test="$header='virtualService'">
                        			<xsl:value-of select="$strings/Deployment_Plan"/>
                        		</xsl:when>
                        		<xsl:when test="$header='stateIds'">
                        			<xsl:value-of select="$strings/Last_Result"/>
                        		</xsl:when>
                        		<xsl:otherwise>
                        			<xsl:value-of select="$strings/*[name()=$header]"/>
                        		</xsl:otherwise>
                        	</xsl:choose>
                        </fo:block>
					</fo:table-cell>
				</xsl:for-each>
			</fo:table-header>
			
			<fo:table-body>
				<xsl:choose>
					<xsl:when test="$records">
						<xsl:for-each select="$records">
							<xsl:variable name="record" select="current()"/>
							
							<fo:table-row>
								<xsl:for-each select="$columns">
									<xsl:apply-templates mode="ewi-value" select="current()">
										<xsl:with-param name="artifact" select="$record"/>
									</xsl:apply-templates>
								</xsl:for-each>
							</fo:table-row>
						</xsl:for-each>
					</xsl:when>
					<xsl:otherwise>
						<!-- inserts emptly columns so that the table header can still appear in the document -->
						<fo:table-row>
							<xsl:for-each select="$columns">
								<fo:table-cell><fo:block/></fo:table-cell>
							</xsl:for-each>
						</fo:table-row>
					</xsl:otherwise>
				</xsl:choose>
			</fo:table-body>
		</fo:table>
	</xsl:template>

    <xsl:template mode="ewi-value" match="pref_table:column">
        <xsl:param name="artifact"/>
        
        <fo:table-cell xsl:use-attribute-sets="artifact-table-body-cell">
        	<fo:block-container overflow="hidden">
	            <!--  match type-specific value templates locally defined in this file  -->
	            <xsl:apply-templates mode="ewi-value" select="pref_table:name">
	                <xsl:with-param name="artifact" select="$artifact"/>
	            </xsl:apply-templates>
	            <!--  and now match generic value templates from abstract-table-summary.xsl  -->
	            <xsl:apply-templates select="pref_table:name">
	                <xsl:with-param name="artifact" select="$artifact"/>
	            </xsl:apply-templates>
            </fo:block-container>
        </fo:table-cell>
    </xsl:template>
	
	<!-- ID -->
	<xsl:template mode="ewi-value" match="pref_table:column/pref_table:name[.='id']">
		<xsl:param name="artifact"/>
		
		<fo:block xsl:use-attribute-sets="id-cell">
			<fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{$artifact/dc:identifier}">
				<xsl:value-of select="$artifact/rqm:webId"/>
			</fo:basic-link>
		</fo:block>
	</xsl:template>
	
	<!-- Priority -->
	<xsl:template mode="ewi-value" match="pref_table:column/pref_table:name[.='priority']">
		<xsl:param name="artifact"/>
		<fo:block xsl:use-attribute-sets="priority-cell"><xsl:apply-templates select="$artifact/ewi:priority"/></fo:block>
	</xsl:template>
	
	<!-- Blocking Status -->
	<xsl:template mode="ewi-value" match="pref_table:column/pref_table:name[.='isBlocking']">
		<xsl:param name="artifact"/>
		
		<xsl:variable name="isblocking" select="$artifact/rqm:blockingdefects"/>
		
		<xsl:choose>
			<xsl:when test="$isblocking">
				<fo:block><xsl:value-of select="$strings/True"/></fo:block>
			</xsl:when>
			<xsl:otherwise><fo:block/></xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<!-- Name -->
	<xsl:template mode="ewi-value" match="pref_table:column/pref_table:name[.='name']">
		<xsl:param name="artifact"/>
		
		<fo:block>
			<fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{$artifact/dc:identifier}">
				<xsl:apply-templates select="$artifact/dc:title"/>
			</fo:basic-link>
		</fo:block>
	</xsl:template>
	
	<!-- Test Plan -->
	<xsl:template mode="ewi-value" match="pref_table:column/pref_table:name[.='testPlanName']">
		<xsl:param name="artifact"/>
		<xsl:variable name="testplan" select="$artifact/rqm:testplan"/>
		
		<xsl:choose>
			<xsl:when test="$testplan">
				<fo:block>
					<fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{$testplan/dc:identifier}">
						<xsl:apply-templates select="$testplan/dc:title"/>
					</fo:basic-link>
				</fo:block>
			</xsl:when>
			<xsl:otherwise><fo:block><xsl:value-of select="$strings/Unassigned"/></fo:block></xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<!-- Test Case -->
	<xsl:template mode="ewi-value" match="pref_table:column/pref_table:name[.='testCaseName']">
		<xsl:param name="artifact"/>
		<xsl:variable name="testcase" select="$artifact/rqm:testcase"/>
		
		<xsl:choose>
			<xsl:when test="$testcase">
				<fo:block>
					<fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{$testcase/dc:identifier}">
						<xsl:apply-templates select="$testcase/dc:title"/>
					</fo:basic-link>
				</fo:block>
			</xsl:when>
			<xsl:otherwise><fo:block/></xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<!-- Categories -->
    <xsl:template match="pref_table:column/pref_table:name[starts-with(., 'isCategoryColumn') and contains(., 'parentCategory') and substring-after(., 'parentCategory') = '']">
    	<xsl:param name="artifact"/>
        <xsl:variable name="testcase" select="$artifact/rqm:testcase"/>
        
        <xsl:variable name="term" select="substring-before(substring-after(., 'isCategoryColumn'), 'parentCategory')"/>
        <xsl:variable name="termValues" select="$testcase/rqm:category[@term=$term]"/>
        <xsl:choose>
            <xsl:when test="$termValues">
                <fo:block><xsl:for-each select="$termValues"><xsl:value-of select="@value"/><xsl:if test="position() != last()">, </xsl:if></xsl:for-each></fo:block>
            </xsl:when>
            <xsl:otherwise>
                <fo:block><xsl:value-of select="$strings/Unassigned"/></fo:block>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <xsl:template match="pref_table:column/pref_table:name[starts-with(., 'isCategoryColumn') and not (contains (., 'parentCategory'))]">
        <xsl:param name="artifact"/>
        
        <xsl:variable name="term" select="substring-after(., 'isCategoryColumn')"/>
        <xsl:variable name="termValues" select="$artifact/rqm:category[@term=$term]"/>
        <xsl:choose>
            <xsl:when test="$termValues">
                <fo:block><xsl:for-each select="$termValues"><xsl:value-of select="@value"/><xsl:if test="position() != last()">, </xsl:if></xsl:for-each></fo:block>
            </xsl:when>
            <xsl:otherwise>
                <fo:block><xsl:value-of select="$strings/Unassigned"/></fo:block>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
	
	<!-- Iteration -->
	<xsl:template mode="ewi-value" match="pref_table:column/pref_table:name[.='testPhases']">
		<xsl:param name="artifact"/>
		<xsl:variable name="iteration" select="$artifact/rqm:testphase"/>
		
		<xsl:choose>
			<xsl:when test="$iteration">
				<fo:block>
					<fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{$iteration/dc:identifier}">
						<xsl:apply-templates select="$iteration/dc:title"/>
					</fo:basic-link>
				</fo:block>
			</xsl:when>
			<xsl:otherwise><fo:block><xsl:value-of select="$strings/Unassigned"/></fo:block></xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	
	<!-- Test Script -->
	<xsl:template mode="ewi-value" match="pref_table:column/pref_table:name[.='defaultScriptName']">
		<xsl:param name="artifact"/>
		<xsl:variable name="testscript" select="$artifact/rqm:testscript"/>
		
		<xsl:choose>
			<xsl:when test="$testscript">
				<fo:block>
					<fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{$testscript/dc:identifier}">
						<xsl:apply-templates select="$testscript/dc:title"/>
					</fo:basic-link>
				</fo:block>		
			</xsl:when>
			<xsl:otherwise><fo:block><xsl:value-of select="$strings/None"/></fo:block></xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<!-- Test Environment -->
	<xsl:template mode="ewi-value" match="pref_table:column/pref_table:name[.='configuration']">
		<xsl:param name="artifact"/>
		<xsl:variable name="configuration" select="$artifact/rqm:configuration"/>
		
		<xsl:choose>
			<xsl:when test="$configuration">
				<fo:block>
					<fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{$configuration/dc:identifier}">
						<xsl:apply-templates select="$configuration/dc:title"/>
					</fo:basic-link>
				</fo:block>
			</xsl:when>
			<xsl:otherwise><fo:block><xsl:value-of select="$strings/Unassigned"></xsl:value-of></fo:block></xsl:otherwise>
		</xsl:choose>
		
	</xsl:template>
	
	<!-- Defects -->
	<xsl:template mode="ewi-value" match="pref_table:column/pref_table:name[.='defects']">
		<xsl:param name="artifact"/>
		<xsl:variable name="defects" select="$artifact/rqm:executionresult/rqm:defect"/>			
		
		<fo:block>
			<xsl:choose>
				<xsl:when test="$defects">
					<xsl:for-each select="$defects">
						<xsl:if test="position() > 1">, </xsl:if>
						
						<fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="@href">
							<xsl:apply-templates select="@summary"/>
						</fo:basic-link>
					</xsl:for-each>
				</xsl:when>
				<xsl:otherwise>--</xsl:otherwise>
			</xsl:choose>	
		</fo:block>
	</xsl:template>
	
	<!-- Build -->
	<xsl:template mode="ewi-value" match="pref_table:column/pref_table:name[.='buildRecordName']">
		<xsl:param name="artifact"/>
		<xsl:variable name="build" select="$artifact/rqm:executionresult/rqm:buildrecord"/>
		
		<xsl:choose>
			<xsl:when test="$build">
				<fo:block>
					<fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{$build/dc:identifier}">
						<xsl:apply-templates select="$build/dc:title"/>
					</fo:basic-link>
				</fo:block>
			</xsl:when>
			<xsl:otherwise><fo:block><xsl:value-of select="$strings/Unassigned"/></fo:block></xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<!-- Risk -->
	<xsl:template mode="ewi-value" match="pref_table:column/pref_table:name[.='calculatedRisk']">
		<xsl:param name="artifact"/>
		
		<fo:block>
			<xsl:call-template name="drawRiskIconSet">
				<xsl:with-param name="iconCount"><xsl:value-of select="$artifact/rqm:risk/rqm:riskLevel"/></xsl:with-param>
				<xsl:with-param name="positiveIcon">risk-selected_obj.gif</xsl:with-param>
				<xsl:with-param name="negativeIcon">risk-unselect_obj.gif</xsl:with-param>
			</xsl:call-template>
		</fo:block>
	</xsl:template>
	
	<!-- Community Risk -->
	<xsl:template mode="ewi-value" match="pref_table:column/pref_table:name[.='communityRisk']">
		<xsl:param name="artifact"/>
		
		<fo:block>
			<xsl:call-template name="drawRiskIconSet">
				<xsl:with-param name="iconCount"><xsl:value-of select="$artifact/rqm:risk/rqm:commnunityAssessment/rqm:riskLevel"/></xsl:with-param>
				<xsl:with-param name="positiveIcon">risk-selected_obj.gif</xsl:with-param>
				<xsl:with-param name="negativeIcon">risk-unselect_obj.gif</xsl:with-param>
			</xsl:call-template>
		</fo:block>
	</xsl:template>
	
	<!-- Owner -->
	<xsl:template mode="ewi-value" match="pref_table:column/pref_table:name[.='ownerUUIDs']">
		<xsl:param name="artifact"/>
		
		<fo:block><xsl:apply-templates select="$artifact/prep:owner"/></fo:block>
	</xsl:template>
	
	<!-- Created By -->
	<xsl:template mode="ewi-value" match="pref_table:column/pref_table:name[.='originator']">
		<xsl:param name="artifact"/>
		
		<fo:block><xsl:apply-templates select="$artifact/dc:creator"/></fo:block>
	</xsl:template>
	
	<!-- Estimate -->
	<xsl:template mode="ewi-value" match="pref_table:column/pref_table:name[.='estimate']">
		<xsl:param name="artifact"/>
		
		<fo:block>
			<xsl:if test="$artifact/rqm:estimate">
				<xsl:value-of select="time:formatTimeEstimate($artifact/rqm:estimate, $locale)"/>
			</xsl:if>
		</fo:block>
	</xsl:template>
	
	<!-- Time Spent -->
	<xsl:template mode="ewi-value" match="pref_table:column/pref_table:name[.='timeSpent']">
		<xsl:param name="artifact"/>
		
		<fo:block>
			<xsl:if test="$artifact/rqm:timeSpent">
				<xsl:value-of select="time:formatTimeEstimate($artifact/rqm:timeSpent, $locale)"/>
			</xsl:if>
		</fo:block>
	</xsl:template>
	
	<!-- Weight -->
	<xsl:template mode="ewi-value" match="pref_table:column/pref_table:name[.='weight']">
		<xsl:param name="artifact"/>
		
		<fo:block><xsl:value-of select="$artifact/ewi:weight"/></fo:block>
	</xsl:template>
	
	<!-- Total Points -->
	<xsl:template mode="ewi-value" match="pref_table:column/pref_table:name[.='totalPoints']">
		<xsl:param name="artifact"/>
		
		<fo:block><xsl:value-of select="$artifact/ewi:weight"/></fo:block>
	</xsl:template>
	
	<!-- Modified -->
	<xsl:template mode="ewi-value" match="pref_table:column/pref_table:name[.='modified']">
		<xsl:param name="artifact"/>
		
		<fo:block><xsl:value-of select="time:formatMediumDate($artifact/alm:updated, $locale)"/></fo:block>
	</xsl:template>
	
 	<!-- Custom Categories (handled by abstract-table-summary.xsl -->
	
	<!-- Last Result -->
	<xsl:template mode="ewi-value" match="pref_table:column/pref_table:name[.='stateIds']">
		<xsl:param name="artifact"/>
		<xsl:variable name="executionresult" select="$artifact/rqm:executionresult"/>		
		
		<fo:block>
			<xsl:choose>
				<xsl:when test="$executionresult">
					<xsl:variable name="state" select="$executionresult/alm:state"/>
					<xsl:variable name="customStateName" select="$executionresult/alm:state/@value"/>
					
					<fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{$executionresult/dc:identifier}">
						<xsl:call-template name="printResultStateIcon">
							<xsl:with-param name="size">45%</xsl:with-param>
							<xsl:with-param name="priority_id"><xsl:value-of select="$state"/></xsl:with-param>
						</xsl:call-template>
						<xsl:text> </xsl:text>
						<xsl:choose>
							<xsl:when test="$customStateName">
								<xsl:value-of select="$customStateName"/></xsl:when>
							<xsl:otherwise>
								<xsl:value-of select="$strings/*[name()=$state]"/></xsl:otherwise>
						</xsl:choose>
					</fo:basic-link>
				</xsl:when>
				<xsl:otherwise>
					<xsl:call-template name="printResultStateIcon">
						<xsl:with-param name="size">45%</xsl:with-param>
					</xsl:call-template>
					<xsl:text> </xsl:text>
					<xsl:value-of select="$strings/Not_Run"/>
				</xsl:otherwise>
			</xsl:choose>
		</fo:block>
	</xsl:template>
	
	<!-- Last Result Completed -->
	<xsl:template mode="ewi-value" match="pref_table:column/pref_table:name[.='resultCompleted']">
		<xsl:param name="artifact"/>
		
		<fo:block><xsl:value-of select="time:formatMediumDate($artifact/rqm:executionresult/er:endtime, $locale)"/></fo:block>
	</xsl:template>
	
	<!-- Last Result Modified -->
	<xsl:template mode="ewi-value" match="pref_table:column/pref_table:name[.='resultModified']">
		<xsl:param name="artifact"/>
		
		<fo:block><xsl:value-of select="time:formatMediumDate($artifact/rqm:executionresult/alm:updated, $locale)"/></fo:block>
	</xsl:template>
	
	<!-- Passed Points -->
	<xsl:template mode="ewi-value" match="pref_table:column/pref_table:name[.='passedPoints']">
		<xsl:param name="artifact"/>
		<xsl:variable name="result" select="$artifact/rqm:executionresult"/>
		
		<fo:block><xsl:value-of select="$result/er:pointspassed"/></fo:block>
	</xsl:template>
	
	<!-- Failed Points -->
	<xsl:template mode="ewi-value" match="pref_table:column/pref_table:name[.='failedPoints']">
		<xsl:param name="artifact"/>
		<xsl:variable name="result" select="$artifact/rqm:executionresult"/>
		
		<fo:block><xsl:value-of select="$result/er:pointsfailed"/></fo:block>
	</xsl:template>
	
	<!-- Attempted Points -->
	<xsl:template mode="ewi-value" match="pref_table:column/pref_table:name[.='attemptedPoints']">
		<xsl:param name="artifact"/>
		<xsl:variable name="result" select="$artifact/rqm:executionresult"/>
		
		<fo:block><xsl:value-of select="$result/er:pointsattempted"/></fo:block>
	</xsl:template>
	
	<!-- Blocked Points -->
	<xsl:template mode="ewi-value" match="pref_table:column/pref_table:name[.='blockedPoints']">
		<xsl:param name="artifact"/>
		<xsl:variable name="result" select="$artifact/rqm:executionresult"/>
		
		<fo:block><xsl:value-of select="$result/er:pointsblocked"/></fo:block>
	</xsl:template>
	
	<!-- Inconclusive Points -->
	<xsl:template mode="ewi-value" match="pref_table:column/pref_table:name[.='inconclusivePoints']">
		<xsl:param name="artifact"/>
		<xsl:variable name="result" select="$artifact/rqm:executionresult"/>
		
		<fo:block><xsl:value-of select="$result/er:pointsinconclusive"/></fo:block>
	</xsl:template>
	
	<!-- Perm Failed Points -->
	<xsl:template mode="ewi-value" match="pref_table:column/pref_table:name[.='permFailedPoints']">
		<xsl:param name="artifact"/>
		<xsl:variable name="result" select="$artifact/rqm:executionresult"/>
		
		<fo:block><xsl:value-of select="$result/rqm:pointspermfailed"/></fo:block>
	</xsl:template>
	
	<!-- Deferred Points -->
	<xsl:template mode="ewi-value" match="pref_table:column/pref_table:name[.='deferredPoints']">
		<xsl:param name="artifact"/>
		<xsl:variable name="result" select="$artifact/rqm:executionresult"/>
		
		<fo:block><xsl:value-of select="$result/rqm:pointsdeferred"/></fo:block>
	</xsl:template>
	
	<!-- Deployment Plan -->
	<xsl:template mode="ewi-value" match="pref_table:column/pref_table:name[.='virtualService']">
		<xsl:param name="artifact"/>
		
		<xsl:choose>
            <xsl:when test="$artifact/rqm:runsOnDeployment">
                <fo:block>
                    <fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{$artifact/rqm:runsOnDeployment/@href}">
                        <xsl:apply-templates select="$artifact/rqm:runsOnDeployment/@summary"/>
                    </fo:basic-link>
                </fo:block>
            </xsl:when>
            <xsl:otherwise><fo:block>--</fo:block></xsl:otherwise>
        </xsl:choose>
	</xsl:template>	
</xsl:stylesheet>