<?xml version="1.0" encoding="utf-8"?>
<!-- 
/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2013. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 -->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:fo="http://www.w3.org/1999/XSL/Format"
    xmlns:rqm="http://jazz.net/xmlns/alm/qm/v0.1/"
    xmlns:dc="http://purl.org/dc/elements/1.1/"
    xmlns:alm="http://jazz.net/xmlns/alm/v0.1/"
    xmlns:prep="http://com.ibm.rqm.print/prep"
    xmlns:time="xalan://com.ibm.rqm.print.render.fop.xml.xslt.extension.TimeUtilsExtension"
    exclude-result-prefixes="xsl rqm dc alm time prep">
    
    <xsl:import  href="cache://stylesheets/render/abstract-table-summary.xsl"/>
    
    <!-- for debugging only -->
    <!--<xsl:include href="cache://stylesheets/render/_config/attribute-sets.xsl"/>-->
    
    <xsl:template name="buildKeywordTable">
        <xsl:param name="columns"/>
        <xsl:param name="keyword-list"/>
        
        <fo:table xsl:use-attribute-sets="artifact-table">
            <xsl:for-each select="$columns">
                <fo:table-column column-width="{concat(prep:width, '%')}"/>
            </xsl:for-each>
            
            <fo:table-header xsl:use-attribute-sets="artifact-table-header">
                <xsl:for-each select="$columns">
                    <fo:table-cell xsl:use-attribute-sets="artifact-table-header-cell">
                        <xsl:variable name="header" select="prep:name"/>
                        <fo:block>
                        	<xsl:choose>
                        		<xsl:when test="starts-with($header, 'isCategoryColumn')">
                        			<xsl:value-of select="substring-after($header, 'isCategoryColumn')"/>
                        		</xsl:when>
                        		<xsl:otherwise>
                        			<xsl:choose>
                        				<xsl:when test="$header='name'">
                        					<xsl:value-of select="$strings/Keyword"/>
                        				</xsl:when>
                        				<xsl:otherwise>
                        					<xsl:value-of select="$strings/*[name()=$header]"/>	
                        				</xsl:otherwise>
                        			</xsl:choose>
                        		</xsl:otherwise>
                        	</xsl:choose>
                        </fo:block>
                    </fo:table-cell>
                </xsl:for-each>
            </fo:table-header>

            <fo:table-body>
                <xsl:choose>
                    <xsl:when test="$keyword-list">
                        <xsl:for-each select="$keyword-list">
                            <xsl:variable name="keyword" select="current()"/>

                            <fo:table-row>
                                <xsl:for-each select="$columns">
                                    <xsl:apply-templates select="current()">
                                        <xsl:with-param name="artifact" select="$keyword"/>
                                    </xsl:apply-templates>
                                </xsl:for-each>
                            </fo:table-row>
                        </xsl:for-each>
                    </xsl:when>
                    <xsl:otherwise>
                        <!-- inserts emptly columns so that the table header can still appear in the document -->
                        <fo:table-row>
                            <xsl:for-each select="$columns">
                                <fo:table-cell><fo:block/></fo:table-cell>
                            </xsl:for-each>
                        </fo:table-row>
                    </xsl:otherwise>
                </xsl:choose>
            </fo:table-body>
        </fo:table>
    </xsl:template>
    
    <!-- Id -->
    <xsl:template match="prep:column/prep:name[.='id']">
        <xsl:param name="artifact"/>
        
        <fo:block>
            <xsl:value-of select="$artifact/rqm:webId"/>
        </fo:block>
    </xsl:template>
    
    <!-- Name -->
    <xsl:template match="prep:column/prep:name[.='name']">
        <xsl:param name="artifact"/>
        
        <fo:block><fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{$artifact/dc:identifier}"><xsl:apply-templates select="$artifact/dc:title"/></fo:basic-link></fo:block>
    </xsl:template>
    
    <!-- Owner -->
    <xsl:template match="prep:column/prep:name[.='ownerUUIDs']">
        <xsl:param name="artifact"/>
        
        <fo:block>
            <xsl:choose>
                <xsl:when test="$artifact/prep:owner = 'Unassigned'">
                    <xsl:value-of select="$strings/Unassigned"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="$artifact/prep:owner/@username"/>        
                </xsl:otherwise>
            </xsl:choose>
        </fo:block>
    </xsl:template>

    <!-- Created By -->
    <xsl:template match="prep:column/prep:name[.='originator']">
        <xsl:param name="artifact"/>
        
        <fo:block>
            <xsl:choose>
                <xsl:when test="$artifact/prep:creator = 'Unassigned'">
                    <xsl:value-of select="$strings/Unassigned"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="$artifact/prep:creator/@username"/>        
                </xsl:otherwise>
            </xsl:choose>
        </fo:block>
    </xsl:template>
    
    <!-- Modified -->
    <xsl:template match="prep:column/prep:name[.='modifiedDateTime']">
        <xsl:param name="artifact"/>
        
        <fo:block><xsl:value-of select="time:formatMediumDate($artifact/alm:updated, $locale)"/></fo:block>
    </xsl:template>
    
    <!-- Tags -->
    <xsl:template match="prep:column/prep:name[.='tags']">
        <xsl:param name="artifact"/>
        
        <fo:block><xsl:value-of select="$artifact/rqm:tags"/></fo:block>
    </xsl:template>
   
   <!-- Associated Script -->
    <xsl:template match="prep:column/prep:name[.='scriptName']">
		<xsl:param name="artifact"/>
        <xsl:variable name="executionscripts" select="$artifact/rqm:channelscript/rqm:executionscript"/>
		
		<fo:block>
		    <xsl:variable name="script-count" select="count($executionscripts)"/>
		    
		    <xsl:choose>
		        <xsl:when test="$script-count > 1">
		            <xsl:for-each select="$executionscripts">
		                <fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{dc:identifier}">
		                    (<xsl:number/>)
		                </fo:basic-link>
		                <xsl:if test="position() != last()">
		                    <xsl:text>, </xsl:text>
		                </xsl:if>
		            </xsl:for-each>
		        </xsl:when>
		        <xsl:when test="$script-count = 1">
		            <xsl:for-each select="$executionscripts">
		                <fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{dc:identifier}">
		                    <xsl:value-of select="dc:title"/>
		                </fo:basic-link>
		            </xsl:for-each>
		        </xsl:when>
		        <xsl:otherwise>--</xsl:otherwise>
		    </xsl:choose>
		</fo:block>
    </xsl:template>
    
</xsl:stylesheet>