<?xml version="1.0" encoding="utf-8"?>
<!-- 
/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 -->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:fo="http://www.w3.org/1999/XSL/Format"
    xmlns:rqm="http://jazz.net/xmlns/alm/qm/v0.1/"
    xmlns:dc="http://purl.org/dc/elements/1.1/"
    xmlns:alm="http://jazz.net/xmlns/alm/v0.1/"
    xmlns:time="xalan://com.ibm.rqm.print.render.fop.xml.xslt.extension.TimeUtilsExtension"
    exclude-result-prefixes="xsl rqm dc alm time">
    
    <!-- for debugging only -->
    <!-- <xsl:include href="cache://stylesheets/render/_config/attribute-sets.xsl"/>
    <xsl:param name="strings"/>-->
    
    <xsl:import href="cache://stylesheets/render/abstract-table-summary.xsl"/>
    
    <xsl:template name="buildRequestTable">
        <xsl:param name="requests"/>
        
        <xsl:param name="render-id"/>                <xsl:param name="render-summary"/>
        <xsl:param name="render-created"/>           <xsl:param name="render-respondBy"/>
        <xsl:param name="render-assignedTo"/>        <xsl:param name="render-priority"/>
        <xsl:param name="render-status"/>
        
        <fo:table xsl:use-attribute-sets="artifact-table">
            <fo:table-header xsl:use-attribute-sets="artifact-table-header">
                <!-- ID -->
                <xsl:if test="$render-id">
                    <fo:table-cell xsl:use-attribute-sets="artifact-table-header-cell id-header">
                        <fo:block><xsl:value-of select="$strings/ID"/></fo:block>
                    </fo:table-cell>
                </xsl:if>
                <!-- Summary -->
                <xsl:if test="$render-summary">
                    <fo:table-cell xsl:use-attribute-sets="artifact-table-header-cell">
                        <fo:block><xsl:value-of select="$strings/Summary"/></fo:block>
                    </fo:table-cell>
                </xsl:if>
                <!-- Created -->
                <xsl:if test="$render-created">
                    <fo:table-cell xsl:use-attribute-sets="artifact-table-header-cell timestamp-header">
                        <fo:block><xsl:value-of select="$strings/Created"/></fo:block>
                    </fo:table-cell>
                </xsl:if>
                <!-- Respond By -->
                <xsl:if test="$render-respondBy">
                    <fo:table-cell xsl:use-attribute-sets="artifact-table-header-cell date-header">
                        <fo:block><xsl:value-of select="$strings/Respond_by"/></fo:block>
                    </fo:table-cell>
                </xsl:if>
                <!-- Assigned To -->
                <xsl:if test="$render-assignedTo">
                    <fo:table-cell xsl:use-attribute-sets="artifact-table-header-cell" width="1in">
                        <fo:block><xsl:value-of select="$strings/Assigned_to"/></fo:block>
                    </fo:table-cell>
                </xsl:if>
                <!-- Priority -->
                <xsl:if test="$render-priority">
                    <fo:table-cell xsl:use-attribute-sets="artifact-table-header-cell" width=".5in">
                        <fo:block xsl:use-attribute-sets="priority-header"><xsl:value-of select="$strings/Priority"/></fo:block>
                    </fo:table-cell>
                </xsl:if>
                <!-- Status -->
                <xsl:if test="$render-status">
                    <fo:table-cell xsl:use-attribute-sets="artifact-table-header-cell" width="1in">
                        <fo:block><xsl:value-of select="$strings/Status"/></fo:block>
                    </fo:table-cell>
                </xsl:if>
            </fo:table-header>
            <fo:table-body>
                <xsl:choose>
                    <xsl:when test="$requests">
                        <xsl:for-each select="$requests">
                            <fo:table-row>
                                <!-- ID -->
                                <xsl:if test="$render-id">
                                    <fo:table-cell xsl:use-attribute-sets="artifact-table-body-cell id-cell">
                                        <fo:block-container overflow="hidden">
                                        	<fo:block><xsl:value-of select="rqm:webId"/></fo:block>
                                        </fo:block-container>
                                    </fo:table-cell>
                                </xsl:if>
                                <!-- Summary -->
                                <xsl:if test="$render-summary">
                                    <fo:table-cell xsl:use-attribute-sets="artifact-table-body-cell">
                                        <fo:block-container overflow="hidden">
                                        	<fo:block><xsl:apply-templates select="rqm:summary"/></fo:block>
                                    	</fo:block-container>
                                    </fo:table-cell>
                                </xsl:if>
                                <!-- Created -->
                                <xsl:if test="$render-created">
                                    <fo:table-cell xsl:use-attribute-sets="artifact-table-body-cell timestamp-cell">
                                        <fo:block-container overflow="hidden">
                                        	<fo:block><xsl:value-of select="time:formatMediumDateShortTime(rqm:createDate, $locale)"/></fo:block>
                                    	</fo:block-container>
                                    </fo:table-cell>
                                </xsl:if>
                                <!-- Respond By -->
                                <xsl:if test="$render-respondBy">
                                    <fo:table-cell xsl:use-attribute-sets="artifact-table-body-cell date-cell">
                                        <fo:block-container overflow="hidden">
                                        	<fo:block><xsl:value-of select="time:formatMediumDate(rqm:confirmationDate, $locale)"/></fo:block>
                                    	</fo:block-container>
                                    </fo:table-cell>
                                </xsl:if>
                                <!-- Assigned To -->
                                <xsl:if test="$render-assignedTo">
                                    <fo:table-cell xsl:use-attribute-sets="artifact-table-body-cell">
                                        <fo:block-container overflow="hidden">
                                        	<fo:block><xsl:value-of select="rqm:fulfiller"/></fo:block>
                                    	</fo:block-container>
                                    </fo:table-cell>
                                </xsl:if>
                                <!-- Priority -->
                                <xsl:if test="$render-priority">
                                    <fo:table-cell xsl:use-attribute-sets="artifact-table-body-cell">
                                        <fo:block-container overflow="hidden">
                                            <fo:block xsl:use-attribute-sets="priority-cell">
                                                <xsl:apply-templates select="rqm:priority" />
                                            </fo:block>  
                                    	</fo:block-container>
                                    </fo:table-cell>
                                </xsl:if>
                                <!-- Status -->
                                <xsl:if test="$render-status">
                                    <fo:table-cell xsl:use-attribute-sets="artifact-table-body-cell">
                                        <fo:block-container overflow="hidden">
                                        	<fo:block><xsl:value-of select="alm:state"/></fo:block>
                                        </fo:block-container>
                                    </fo:table-cell>
                                </xsl:if>
                            </fo:table-row>
                        </xsl:for-each>
                    </xsl:when>
                    <xsl:otherwise>
                        <fo:table-row>
                            <!-- ID -->
                            <xsl:if test="$render-id">
                                <fo:table-cell><fo:block/></fo:table-cell>
                            </xsl:if>
                            <!-- Summary -->
                            <xsl:if test="$render-summary">
                                <fo:table-cell><fo:block/></fo:table-cell>
                            </xsl:if>
                            <!-- Created -->
                            <xsl:if test="$render-created">
                                <fo:table-cell><fo:block/></fo:table-cell>
                            </xsl:if>
                            <!-- Date Header -->
                            <xsl:if test="$render-respondBy">
                                <fo:table-cell><fo:block/></fo:table-cell>
                            </xsl:if>
                            <!-- Assigned To -->
                            <xsl:if test="$render-assignedTo">
                                <fo:table-cell><fo:block/></fo:table-cell>
                            </xsl:if>
                            <!-- Priority -->
                            <xsl:if test="$render-priority">
                                <fo:table-cell><fo:block/></fo:table-cell>
                            </xsl:if>
                            <!-- Status -->
                            <xsl:if test="$render-status">
                                <fo:table-cell><fo:block/></fo:table-cell>
                            </xsl:if>
                        </fo:table-row>
                    </xsl:otherwise>
                </xsl:choose>
            </fo:table-body>
        </fo:table>
    </xsl:template>
    
</xsl:stylesheet>