<?xml version="1.0" encoding="utf-8"?>
<!-- 
/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2014. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 -->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:fo="http://www.w3.org/1999/XSL/Format"
    xmlns:rqm="http://jazz.net/xmlns/alm/qm/v0.1/"
    xmlns:dc="http://purl.org/dc/elements/1.1/"
    xmlns:alm="http://jazz.net/xmlns/alm/v0.1/"
    xmlns:prep="http://com.ibm.rqm.print/prep"
    xmlns:pref="http://com.ibm.rqm.export.pdf/preferences"
    xmlns:pref_table="http://com.ibm.rqm.export.pdf/preferences/table"
    xmlns:log="xalan://com.ibm.rqm.print.render.fop.xml.xslt.extension.LoggingExtension"
    xmlns:utils="xalan://com.ibm.rqm.print.render.fop.xml.xslt.extension.UtilsExtension"
    xmlns:time="xalan://com.ibm.rqm.print.render.fop.xml.xslt.extension.TimeUtilsExtension"
    exclude-result-prefixes="xsl rqm dc alm log utils time prep">
    
    <xsl:import  href="cache://stylesheets/render/abstract-table-summary.xsl"/>

    <!-- for debugging only -->
    <!--<xsl:include href="cache://stylesheets/render/_config/attribute-sets.xsl"/>
    <xsl:include href="cache://stylesheets/render/_util/utils.xsl"/>-->
    
    <xsl:template name="buildTestcaseTable">
        <xsl:param name="columns"/>
        <xsl:param name="testcases"/>
        
        <fo:table xsl:use-attribute-sets="artifact-table">
            <xsl:for-each select="$columns">
                <fo:table-column column-width="{concat(pref_table:width, '%')}"/>
            </xsl:for-each>
                       
            <fo:table-header xsl:use-attribute-sets="artifact-table-header">
                <xsl:for-each select="$columns">
                    <fo:table-cell xsl:use-attribute-sets="artifact-table-header-cell">
                        <xsl:variable name="header" select="pref_table:name"/>
                        <fo:block>
                        	<xsl:choose>
                        		<xsl:when test="starts-with($header, 'isCategoryColumn')">
                        			<xsl:value-of select="substring-after($header, 'isCategoryColumn')"/>
                        		</xsl:when>
                        	    <xsl:when test="starts-with($header, 'isCustomColumn')">
                        	        <xsl:value-of select="substring-after($header, 'isCustomColumn')"/> 			
                        	    </xsl:when>
                        	    <xsl:when test="$header='id'">
                        	        <fo:block xsl:use-attribute-sets="id-header"><xsl:value-of select="$strings/id"/></fo:block>
                        	    </xsl:when>
                        	    <xsl:when test="$header='priority'">
                        	        <fo:block xsl:use-attribute-sets="priority-header"><xsl:value-of select="$strings/Priority"/></fo:block>
                        	    </xsl:when>
                        	    <xsl:when test="$header='systemRisk'">
                        	        <fo:block xsl:use-attribute-sets="priority-header"><xsl:value-of select="$strings/Risk"/></fo:block>
                        	    </xsl:when>
                        	    <xsl:when test="$header='testScripts'">
                        	        <fo:block xsl:use-attribute-sets="priority-header"><xsl:value-of select="$strings/testScript"/></fo:block>
                        	    </xsl:when>
                        		<xsl:otherwise>
                        			<xsl:value-of select="$strings/*[name()=$header]"/>
                        		</xsl:otherwise>
                        	</xsl:choose>
                        </fo:block>
                    </fo:table-cell>
                </xsl:for-each>
            </fo:table-header>

            <fo:table-body>
                <xsl:choose>
                    <xsl:when test="$testcases">
                        <xsl:for-each select="$testcases">
                            <xsl:variable name="testcase" select="current()"/>
                            <fo:table-row>
                                <xsl:for-each select="$columns">
                                    <xsl:apply-templates mode="tc-value" select="current()">
                                        <xsl:with-param name="artifact" select="$testcase"/>
                                    </xsl:apply-templates>
                                </xsl:for-each>
                            </fo:table-row>
                        </xsl:for-each>
                    </xsl:when>
                    <xsl:otherwise>
                        <!-- inserts empty columns so that the table header can still appear in the document -->
                        <fo:table-row>
                            <xsl:for-each select="$columns">
                                <fo:table-cell><fo:block/></fo:table-cell>
                            </xsl:for-each>
                        </fo:table-row>
                    </xsl:otherwise>
                </xsl:choose>
            </fo:table-body>
        </fo:table>
    </xsl:template>
    
    <xsl:template mode="tc-value" match= "pref_table:column">
        <xsl:param name="artifact"/>
        
        <fo:table-cell xsl:use-attribute-sets="artifact-table-body-cell">
        	<fo:block-container overflow="hidden">
	            <!--  match type-specific value templates locally defined in this file  -->
	            <xsl:apply-templates mode="tc-value" select="pref_table:name">
	                <xsl:with-param name="artifact" select="$artifact"/>
	            </xsl:apply-templates>
	            <!--  and now match generic value templates from abstract-table-summary.xsl  -->
	            <xsl:apply-templates select="pref_table:name">
	                <xsl:with-param name="artifact" select="$artifact"/>
	            </xsl:apply-templates>
            </fo:block-container>
        </fo:table-cell>
    </xsl:template>

    <!-- ID -->
    <xsl:template mode="tc-value" match="pref_table:column/pref_table:name[.='id']">
        <xsl:param name="artifact"/>
        
        <fo:block xsl:use-attribute-sets="id-cell"><fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{$artifact/dc:identifier}"><xsl:value-of select="$artifact/rqm:webId"/></fo:basic-link></fo:block>
    </xsl:template>
    
    <!-- Priority -->
    <xsl:template mode="tc-value" match="pref_table:column/pref_table:name[.='priority']">
        <xsl:param name="artifact"/>
        
        <fo:block xsl:use-attribute-sets="priority-cell">
            <xsl:apply-templates select="$artifact/rqm:priority" />
        </fo:block>
    </xsl:template>
    
    <!-- Name -->
    <xsl:template mode="tc-value" match="pref_table:column/pref_table:name[.='name' ]">
        <xsl:param name="artifact"/>
        
        <fo:block><fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{$artifact/dc:identifier}"><xsl:apply-templates select="$artifact/dc:title"/></fo:basic-link></fo:block>
    </xsl:template>
    
    <!-- Modified -->
    <xsl:template mode="tc-value" match="pref_table:column/pref_table:name[.='modified' ]">
        <xsl:param name="artifact"/>
        
        <fo:block><xsl:value-of select="time:formatMediumDate($artifact/alm:updated, $locale)"/></fo:block>
    </xsl:template>
    
    <!-- State -->
    <xsl:template mode="tc-value" match="pref_table:column/pref_table:name[.='state' ]">
        <xsl:param name="artifact"/>
        
        <xsl:variable name="stateId" select="$artifact/alm:state"/>                  
		<xsl:variable name="stateName" select="$strings/*[name()=$stateId]"/>    
        <fo:block>
            <xsl:choose>
            	<xsl:when test="$stateName">
                	<xsl:value-of select="$stateName"/>
				</xsl:when>
                <xsl:otherwise>
                	<xsl:variable name="workflowType" select="'TESTCASE'"/>    
					<xsl:value-of select="utils:getStateName($projectArea, $workflowType, $stateId)"/>
                </xsl:otherwise>
            </xsl:choose>
        </fo:block>                    
    </xsl:template>
    
    <!-- Owner -->
    <xsl:template mode="tc-value" match="pref_table:column/pref_table:name[.='owner' ]">
        <xsl:param name="artifact"/>
        <xsl:variable name="owner" select="$artifact/prep:owner"/>
        <xsl:choose>
        	<xsl:when test="$owner='unassigned'">
        		<fo:block><xsl:value-of select="$strings/Unassigned"/></fo:block>
        	</xsl:when>
        	<xsl:otherwise>
        		<fo:block><xsl:apply-templates select="$owner"/></fo:block>
        	</xsl:otherwise>
        </xsl:choose>       
    </xsl:template>
    
    <!-- Created By -->
    <xsl:template mode="tc-value" match="pref_table:column/pref_table:name[.='originator' ]">
		<xsl:param name="artifact"/>
		 <xsl:variable name="creator" select="$artifact/prep:creator"/>
        <xsl:choose>
        	<xsl:when test="$creator='unassigned'">
        		<fo:block><xsl:value-of select="$strings/Unassigned"/></fo:block>
        	</xsl:when>
        	<xsl:otherwise>
        		<fo:block><xsl:apply-templates select="$creator"/></fo:block>
        	</xsl:otherwise>
        </xsl:choose>    
	</xsl:template>
	
    <!-- Weight -->
    <xsl:template mode="tc-value" match="pref_table:column/pref_table:name[.='weight' ]">
        <xsl:param name="artifact"/>
        
        <fo:block><xsl:value-of select="$artifact/rqm:weight"/></fo:block>
    </xsl:template>
    
    <!-- Suspect -->
    <xsl:template mode="tc-value" match="pref_table:column/pref_table:name[.='suspect' ]">
        <xsl:param name="artifact"/>
        <xsl:variable name="isSuspect" select="$artifact/rqm:suspect"/>
        
        <xsl:choose>
        	<xsl:when test="translate($isSuspect, $uppercase, $lowercase)='true'">
        		<fo:block><xsl:value-of select="$strings/True"/></fo:block>
        	</xsl:when>
        	<xsl:otherwise>
        		<fo:block><xsl:value-of select="$strings/False"/></fo:block>
        	</xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <!-- Estimate -->
    <xsl:template mode="tc-value" match="pref_table:column/pref_table:name[.='estimate' ]">
        <xsl:param name="artifact"/>
        <xsl:choose>
        	<xsl:when test="$artifact/rqm:estimate">
        		<fo:block><xsl:value-of select="time:formatTimeEstimate($artifact/rqm:estimate, $locale)"/></fo:block>
        	</xsl:when>
        	<xsl:otherwise>
        	       <fo:block></fo:block>
        	</xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <!-- Risk -->
    <xsl:template mode="tc-value" match="pref_table:column/pref_table:name[(.='calculatedRisk' or .='communityRisk' or .='systemRisk') 
        ]">
        <xsl:param name="artifact"/>
        
        <fo:block>
            <xsl:call-template name="drawRiskIconSet">
                <xsl:with-param name="iconCount"><xsl:value-of select="$artifact/rqm:risk/rqm:riskLevel"/></xsl:with-param>
                <xsl:with-param name="positiveIcon">risk-selected_obj.gif</xsl:with-param>
                <xsl:with-param name="negativeIcon">risk-unselect_obj.gif</xsl:with-param>
            </xsl:call-template>
        </fo:block>
    </xsl:template>
    
    <!-- Community Risk -->
    <xsl:template mode="tc-value" match="pref_table:column/pref_table:name[.='communityRisk' ]">
        <xsl:param name="artifact"/>
        
        <fo:block>
            <xsl:call-template name="drawRiskIconSet">
                <xsl:with-param name="iconCount"><xsl:value-of select="$artifact/rqm:risk/rqm:commnunityAssessment/rqm:riskLevel"/></xsl:with-param>
                <xsl:with-param name="positiveIcon">risk-selected_obj.gif</xsl:with-param>
                <xsl:with-param name="negativeIcon">risk-unselect_obj.gif</xsl:with-param>
            </xsl:call-template>
        </fo:block>
    </xsl:template>
    
    <!-- Test Script -->
    <xsl:template mode="tc-value" match="pref_table:column/pref_table:name[.='testScripts']">
        <xsl:param name="artifact"/>
        
        <xsl:if test="log:isTraceEnabled()">
            <xsl:value-of select="log:trace(concat('rendering test script column for test case with href=', $artifact/dc:identifier))"/>    
        </xsl:if>
        
        <fo:block>
            <xsl:variable name="testscriptSet" select="$artifact/rqm:testscript"/>
            
            <xsl:if test="$testscriptSet">
                <xsl:choose>
                    <xsl:when test="count($testscriptSet) > 1">
                        <xsl:for-each select="$testscriptSet">  
                            <fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{dc:identifier}">
                                (<xsl:value-of select="rqm:webId"/>)
                            </fo:basic-link>
                            <xsl:if test="last() > position()">
                                <xsl:text>, </xsl:text>
                            </xsl:if>
                        </xsl:for-each>
                    </xsl:when>
                    <xsl:otherwise>
                        <fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{$testscriptSet/dc:identifier}">
                            <xsl:value-of select="concat($testscriptSet/rqm:webId, ': ', $testscriptSet/dc:title)"/>
                        </fo:basic-link>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:if>
        </fo:block>
    </xsl:template>
    
    <!-- Validates Requirement -->
    <xsl:template mode="tc-value" match="pref_table:column/pref_table:name[.='reqlLink' ]">
        <xsl:param name="artifact"/>
        
        <fo:block>
            <!-- Assumption: ReqPro requirements always contains a reqProId attribute. -->
            <!-- Note: Filter ReqPro requirements from the Validates Requirement column (OSLC-style requirements only). -->
            <xsl:variable name="requirementSet" select="$artifact/rqm:requirement[not(@reqProId)]"/>
            
            <xsl:choose>
                <xsl:when test="$requirementSet">
                	<xsl:for-each select="$requirementSet">
	                    <fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{@href}">
	                    	<xsl:value-of select="@summary"/>
	                    </fo:basic-link>
	                            
						<xsl:if test="last() > position()">
	                    	<xsl:text>, </xsl:text>
	                	</xsl:if>
                    </xsl:for-each>
                </xsl:when>
                <xsl:otherwise><fo:block>--</fo:block></xsl:otherwise>
            </xsl:choose>
        </fo:block>
    </xsl:template>
    
    <!-- Tests Development Plan -->
    <xsl:template mode="tc-value" match="pref_table:column/pref_table:name[.='devItemLink' ]">
        <xsl:param name="artifact"/>
        <xsl:variable name="testedItems" select="$artifact/rqm:workitem[@rel='tests']"/>
        
        <xsl:choose>
            <xsl:when test="$testedItems">
                <fo:block>
                    <xsl:for-each select="$testedItems">
                        <fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{@href}">
                            <xsl:apply-templates select="@summary"/>
                        </fo:basic-link>
                        <xsl:if test="not(position()=last())">
                            <xsl:text>, </xsl:text>
                        </xsl:if>
                    </xsl:for-each>    
                </fo:block>
            </xsl:when>
            <xsl:otherwise><fo:block>--</fo:block></xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <!-- TER Count -->
    <xsl:template mode="tc-value" match="pref_table:column/pref_table:name[.='tercount' ]">
        <xsl:param name="artifact"/>
        
        <fo:block text-align="center"><xsl:value-of select="$artifact/prep:terCount"/></fo:block>
    </xsl:template>
    
    <!-- Test Plans -->
    <xsl:template mode="tc-value" match="pref_table:column/pref_table:name[.='parentVTestPlans']">
        <xsl:param name="artifact"/>
        
        <xsl:if test="log:isTraceEnabled()">
            <xsl:value-of select="log:trace(concat('rendering test plan column for test case with href=', $artifact/dc:identifier))"/>    
        </xsl:if>
        
        <fo:block>
            <xsl:variable name="testplanSet" select="$artifact/rqm:testplan"/>
            
            <xsl:if test="$testplanSet">
                <xsl:choose>
                    <xsl:when test="count($testplanSet) > 1">
                        <xsl:for-each select="$testplanSet">
                            <fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{dc:identifier}">
                                (<xsl:value-of select="rqm:webId"/>)
                            </fo:basic-link>
                            
                            <xsl:if test="last() > position()">
                                <xsl:text>, </xsl:text>
                            </xsl:if>
                        </xsl:for-each>
                    </xsl:when>
                    <xsl:otherwise>
                        <fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{$testplanSet/dc:identifier}">
                            <xsl:value-of select="concat($testplanSet/rqm:webId, ': ', $testplanSet/dc:title)"/>
                        </fo:basic-link>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:if>
        </fo:block>
    </xsl:template>
    
    <!-- Copied From -->
    <xsl:template mode="tc-value" match="pref_table:column/pref_table:name[.='copiedFrom']">
        <xsl:param name="artifact"/>
        <xsl:variable name="copiedFrom" select="$artifact/prep:copiedFrom"/>  
        
        <xsl:choose>
            <xsl:when test="$copiedFrom">
                <fo:block>
                    <fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{$copiedFrom/dc:identifier}">
                        <xsl:apply-templates select="$copiedFrom/rqm:webId"/>
                    </fo:basic-link>
                </fo:block>
            </xsl:when>
            <xsl:otherwise><fo:block>--</fo:block></xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <!-- Copy Root -->
    <xsl:template mode="tc-value" match="pref_table:column/pref_table:name[.='copiedRoot']">
        <xsl:param name="artifact"/>
        <xsl:variable name="copiedRoot" select="$artifact/prep:copiedRoot"/>
        
        <xsl:choose>
            <xsl:when test="$copiedRoot">
                <fo:block>
                    <fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{$copiedRoot/dc:identifier}">
                        <xsl:apply-templates select="$copiedRoot/rqm:webId"/>
                    </fo:basic-link>
                </fo:block>
            </xsl:when>
            <xsl:otherwise><fo:block>--</fo:block></xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
</xsl:stylesheet>