<?xml version="1.0" encoding="utf-8"?>
<!-- 
	/****************************************************************************** 
	* Licensed Materials - Property of IBM
	* (c) Copyright IBM Corporation 2010, 2014. All Rights Reserved.
	*  
	* U.S. Government Users Restricted Rights:
	* Use, duplication or disclosure restricted by GSA ADP Schedule
	* Contract with IBM Corp.
	******************************************************************************/
-->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:fo="http://www.w3.org/1999/XSL/Format"
	xmlns:rqm="http://jazz.net/xmlns/alm/qm/v0.1/"
	xmlns:dc="http://purl.org/dc/elements/1.1/"
	xmlns:dcterms="http://purl.org/dc/terms/"
	xmlns:alm="http://jazz.net/xmlns/alm/v0.1/"
	xmlns:prep="http://com.ibm.rqm.print/prep"
	xmlns:pref="http://com.ibm.rqm.export.pdf/preferences"
	xmlns:pref_table="http://com.ibm.rqm.export.pdf/preferences/table"
	xmlns:utils="xalan://com.ibm.rqm.print.render.fop.xml.xslt.extension.UtilsExtension"
    xmlns:time="xalan://com.ibm.rqm.print.render.fop.xml.xslt.extension.TimeUtilsExtension"
    xmlns:log="xalan://com.ibm.rqm.print.render.fop.xml.xslt.extension.LoggingExtension"
	exclude-result-prefixes="xsl rqm dc alm prep utils time">
	
    <xsl:import  href="cache://stylesheets/render/abstract-table-summary.xsl"/>

	<!-- for debugging only -->
	<!--<xsl:include href="cache://stylesheets/render/_config/attribute-sets.xsl"/>-->
	
	<xsl:template name="buildTestScriptTable">
		<xsl:param name="columns"/>
		<xsl:param name="scripts"/>

		<fo:table xsl:use-attribute-sets="artifact-table">
            <xsl:for-each select="$columns">
                <fo:table-column column-width="{concat(pref_table:width, '%')}"/>
            </xsl:for-each>
            
            <fo:table-header xsl:use-attribute-sets="artifact-table-header">
                <xsl:for-each select="$columns">
                    <fo:table-cell xsl:use-attribute-sets="artifact-table-header-cell">
                        <xsl:variable name="header" select="pref_table:name"/>
                        <fo:block>
                        	<xsl:choose>
                        		<xsl:when test="starts-with($header, 'isCategoryColumn')">
                        			<xsl:value-of select="substring-after($header, 'isCategoryColumn')"/>
                        		</xsl:when>
                        	    <xsl:when test="starts-with($header, 'isCustomColumn')">
                        	        <xsl:value-of select="substring-after($header, 'isCustomColumn')"/> 			
                        	    </xsl:when>
                        	    <xsl:when test="$header='id'">
                        	        <fo:block xsl:use-attribute-sets="id-header"><xsl:value-of select="$strings/id"/></fo:block>
                        	    </xsl:when>
								<xsl:when test="$header='Default'">
                        			<xsl:value-of select="$strings/Channel_Default"/>
                        		</xsl:when>
                        		<xsl:otherwise>
                        			<xsl:value-of select="$strings/*[name()=$header]"/>
                        		</xsl:otherwise>
                        	</xsl:choose>
                        </fo:block>
                    </fo:table-cell>
                </xsl:for-each>
            </fo:table-header>

            <fo:table-body>
                <xsl:choose>
                    <xsl:when test="$scripts">
                        <xsl:for-each select="$scripts">
                            <xsl:variable name="script" select="current()"/>

                            <fo:table-row>
                                <xsl:for-each select="$columns">
                                    <xsl:apply-templates mode="script-value" select="current()">
                                        <xsl:with-param name="artifact" select="$script"/>
                                    </xsl:apply-templates>
                                </xsl:for-each>
                            </fo:table-row>
                        </xsl:for-each>
                    </xsl:when>
                    <xsl:otherwise>
                        <!-- inserts emptly columns so that the table header can still appear in the document -->
                        <fo:table-row>
                            <xsl:for-each select="$columns">
                                <fo:table-cell><fo:block/></fo:table-cell>
                            </xsl:for-each>
                        </fo:table-row>
                    </xsl:otherwise>
                </xsl:choose>
            </fo:table-body>
        </fo:table>
    </xsl:template>


    <xsl:template mode="script-value" match="pref_table:column">
        <xsl:param name="artifact"/>
        
        <fo:table-cell xsl:use-attribute-sets="artifact-table-body-cell">
        	<fo:block-container overflow="hidden">
	            <!--  match type-specific value templates locally defined in this file  -->
	            <xsl:apply-templates mode="script-value" select="pref_table:name">
	                <xsl:with-param name="artifact" select="$artifact"/>
	            </xsl:apply-templates>
	            <!--  and now match generic value templates from abstract-table-summary.xsl  -->
	            <xsl:apply-templates select="pref_table:name">
	                <xsl:with-param name="artifact" select="$artifact"/>
	            </xsl:apply-templates>
            </fo:block-container>
        </fo:table-cell>
    </xsl:template>
      
    <!-- ID -->
    <xsl:template mode="script-value"  match="pref_table:column/pref_table:name[.='id']">
        <xsl:param name="artifact"/>
        
        <fo:block xsl:use-attribute-sets="id-cell"><fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{$artifact/dc:identifier}"><xsl:value-of select="$artifact/rqm:webId"/></fo:basic-link></fo:block>
    </xsl:template>
    
    <!-- Name -->
    <xsl:template mode="script-value"  match="pref_table:column/pref_table:name[.='name']">
        <xsl:param name="artifact"/>
        
        <fo:block><fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{$artifact/dc:identifier}"><xsl:apply-templates select="$artifact/dc:title"/></fo:basic-link></fo:block>
    </xsl:template>
    
    <!-- Modified -->
    <xsl:template mode="script-value"  match="pref_table:column/pref_table:name[.='modified']">
        <xsl:param name="artifact"/>
        <xsl:apply-templates mode="ts-modified" select="$artifact"/>
    </xsl:template>
    
    <xsl:template mode="ts-modified" match="rqm:testscript|rqm:remotescript">
        <fo:block><xsl:value-of select="time:formatMediumDate(alm:updated, $locale)"/></fo:block>
    </xsl:template>
    
    <xsl:template mode="ts-modified" match="rqm:executionscript">
        <fo:block><xsl:value-of select="time:formatMediumDate(alm:updated, $locale)"/></fo:block>
    </xsl:template>
    
    <!-- State -->
    <xsl:template mode="script-value"  match="pref_table:column/pref_table:name[.='state']">
        <xsl:param name="artifact"/>
        
        <xsl:variable name="stateId" select="$artifact/alm:state"/>                  
		<xsl:variable name="stateName" select="$strings/*[name()=$stateId]"/>    
        <fo:block>
            <xsl:choose>
            	<xsl:when test="$stateName">
                	<xsl:value-of select="$stateName"/>
				</xsl:when>
                <xsl:otherwise>
                	<xsl:variable name="workflowType" select="'TESTSCRIPT'"/>    
					<xsl:value-of select="utils:getStateName($projectArea, $workflowType, $stateId)"/>
                </xsl:otherwise>
            </xsl:choose>
        </fo:block>                    
    </xsl:template>

    <!-- Locked -->
    <xsl:template mode="script-value"  match="pref_table:column/pref_table:name[.='locked']">
        <xsl:param name="artifact"/>
        <xsl:apply-templates mode="ts-locked" select="$artifact"/>
    </xsl:template>

    <xsl:template mode="ts-locked" match="rqm:testscript|rqm:remotescript|rqm:executionscript">
        <fo:block>
        	<xsl:apply-templates select="rqm:locked"/>
        </fo:block>
    </xsl:template>
    
    <!-- Owner -->
    <xsl:template mode="script-value"  match="pref_table:column/pref_table:name[.='owner']" >
        <xsl:param name="artifact"/>
        <xsl:apply-templates mode="ts-owner" select="$artifact"/>
    </xsl:template>
    
    <xsl:template mode="ts-owner" match="rqm:testscript|rqm:remotescript">
        <fo:block>
            <xsl:choose>
                <xsl:when test="prep:owner='unassigned'">
                    <xsl:value-of select="$strings/Unassigned"/></xsl:when>
                <xsl:otherwise>
                    <xsl:apply-templates select="prep:owner"/></xsl:otherwise>
            </xsl:choose>
        </fo:block>
    </xsl:template>
    
    <xsl:template mode="ts-owner" match="rqm:executionscript">
        <fo:block>
            <xsl:choose>
                <xsl:when test="prep:owner='unassigned'">
                    <xsl:value-of select="$strings/Unassigned"/></xsl:when>
                <xsl:otherwise>
                    <xsl:apply-templates select="prep:owner"/></xsl:otherwise>
            </xsl:choose>
        </fo:block>
    </xsl:template>

    <!-- Data Records -->
    <xsl:template mode="script-value"  match="pref_table:column/pref_table:name[.='recordSelection']">
        <xsl:param name="artifact"/>
        <xsl:variable name="datapool" select="$artifact/rqm:datapool"/>
        <xsl:variable name="recordselectioncriteria" select="$artifact/rqm:recordselectioncriteria"/>
        <fo:block>
            <xsl:choose>
                <xsl:when test="$datapool and $recordselectioncriteria">
					<xsl:value-of select="$recordselectioncriteria/rqm:columnName"/> = <xsl:value-of select="$recordselectioncriteria/rqm:value"/>                    
                </xsl:when>
                <xsl:when test="$datapool">
                	<xsl:value-of select="$strings/recordSelectionAll"/>
				</xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="$strings/Unassigned"/>
                </xsl:otherwise>
            </xsl:choose>
        </fo:block>        
    </xsl:template>
    
	<!-- Created By -->
	<xsl:template mode="script-value"  match="pref_table:column/pref_table:name[.='originator']">
		<xsl:param name="artifact"/>
		<xsl:apply-templates mode="ts-creator" select="$artifact"/>
	</xsl:template>
	   
	<xsl:template mode="ts-creator" match="rqm:testscript|rqm:remotescript|rqm:executionscript">
        <fo:block>
            <xsl:choose>
                <xsl:when test="dc:creator='unassigned'">
                    <xsl:value-of select="$strings/Unassigned"/></xsl:when>
                <xsl:otherwise>
                    <xsl:apply-templates select="dc:creator"/></xsl:otherwise>
            </xsl:choose>
        </fo:block>
    </xsl:template>
     
    <!-- Script Type -->
    <xsl:template mode="script-value"  match="pref_table:column/pref_table:name[.='scriptTypeIds']">
        <xsl:param name="artifact"/>
        <fo:block><xsl:apply-templates mode="ts-scriptType" select="$artifact"/></fo:block>
    </xsl:template>
    
    <xsl:template mode="ts-scriptType" match="rqm:testscript|rqm:remotescript">
        <xsl:variable name="type" select="rqm:scripttype"/>
        <xsl:value-of select="$strings/*[name()=$type]"/>
    </xsl:template>
    
    <xsl:template mode="ts-scriptType" match="rqm:executionscript">
        <xsl:variable name="type" select="dcterms:type"/>
        <xsl:value-of select="$strings/*[name()=$type]"/>
    </xsl:template>
    
    <!-- Validates Requirement -->
    <xsl:template mode="script-value"  match="pref_table:column/pref_table:name[.='reqlLink']">
        <xsl:param name="artifact"/>
        <xsl:variable name="requirements" select="$artifact/rqm:steps//rqm:requirement"/>
        
        <xsl:choose>
            <xsl:when test="$requirements">
            	<xsl:for-each select="$requirements">
            		<fo:block>
	            		<fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{@href}">
	            			<xsl:apply-templates select="@summary"/>
	            		</fo:basic-link>
	            		<xsl:if test="not(position()=last())">, </xsl:if>
            		</fo:block>
				</xsl:for-each>
            </xsl:when>  
            <xsl:otherwise><fo:block>--</fo:block></xsl:otherwise>
        </xsl:choose>
    </xsl:template>
  
    <!-- Channels -->
    <xsl:template mode="script-value"  match="pref_table:column/pref_table:name[.='Channel']">
        <xsl:param name="artifact"/>
        <xsl:variable name="channel" select="$artifact/rqm:channel"/>
        
        <fo:block>
            <xsl:choose>
                <xsl:when test="$channel/dc:identifier">
                    <fo:basic-link xsl:use-attribute-sets="simple-link" 
                        external-destination="{$channel/dc:identifier}"><xsl:value-of select="$channel/dc:title"/>
                    </fo:basic-link>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="$strings/None"/>
                </xsl:otherwise>
            </xsl:choose>
        </fo:block>
    </xsl:template>
    
    <!-- Default -->
    <xsl:template mode="script-value"  match="pref_table:column/pref_table:name[.='Default']">
        <xsl:param name="artifact"/>
        
        <fo:block>
            <xsl:choose>
                <xsl:when test="$artifact/prep:isDefault">
                    <xsl:value-of select="$strings/true"/></xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="$strings/false"/></xsl:otherwise>
            </xsl:choose>
        </fo:block>
    </xsl:template>
    
    <!-- Copied From -->
    <xsl:template mode="script-value" match="pref_table:column/pref_table:name[.='copiedFrom']">
        <xsl:param name="artifact"/>
        <xsl:variable name="copiedFrom" select="$artifact/prep:copiedFrom"/>  
        
        <xsl:choose>
            <xsl:when test="$copiedFrom">
                <fo:block>
                    <fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{$copiedFrom/dc:identifier}">
                        <xsl:apply-templates select="$copiedFrom/rqm:webId"/>
                    </fo:basic-link>
                </fo:block>
            </xsl:when>
            <xsl:otherwise><fo:block>--</fo:block></xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <!-- Copy Root -->
    <xsl:template mode="script-value" match="pref_table:column/pref_table:name[.='copiedRoot']">
        <xsl:param name="artifact"/>
        <xsl:variable name="copiedRoot" select="$artifact/prep:copiedRoot"/>
        
        <xsl:choose>
            <xsl:when test="$copiedRoot">
                <fo:block>
                    <fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{$copiedRoot/dc:identifier}">
                        <xsl:apply-templates select="$copiedRoot/rqm:webId"/>
                    </fo:basic-link>
                </fo:block>
            </xsl:when>
            <xsl:otherwise><fo:block>--</fo:block></xsl:otherwise>
        </xsl:choose>
    </xsl:template>
  
    <!-- Custom Categories (handled by abstract-table-summary.xsl -->
    
    </xsl:stylesheet>