<?xml version="1.0" encoding="utf-8"?>
<!-- 
/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2014. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 -->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:fo="http://www.w3.org/1999/XSL/Format"
	xmlns:rqm="http://jazz.net/xmlns/alm/qm/v0.1/"
	xmlns:dc="http://purl.org/dc/elements/1.1/"
	xmlns:alm="http://jazz.net/xmlns/alm/v0.1/"
	xmlns:prep="http://com.ibm.rqm.print/prep"
	xmlns:pref="http://com.ibm.rqm.export.pdf/preferences"
	xmlns:pref_table="http://com.ibm.rqm.export.pdf/preferences/table"
	xmlns:time="xalan://com.ibm.rqm.print.render.fop.xml.xslt.extension.TimeUtilsExtension"
	xmlns:utils="xalan://com.ibm.rqm.print.render.fop.xml.xslt.extension.UtilsExtension"
	exclude-result-prefixes="xsl rqm dc alm prep time">
	
	<xsl:import  href="cache://stylesheets/render/abstract-table-summary.xsl"/>
	
	<!-- for debugging only -->
	<!--<xsl:include href="cache://stylesheets/render/_config/attribute-sets.xsl"/>
	<xsl:include href="cache://stylesheets/render/_util/utils.xsl"/>-->

	<xsl:template name="buildTestSuiteTable">
		<xsl:param name="suites"/>
		<xsl:param name="columns"/>
		
		<fo:table xsl:use-attribute-sets="artifact-table">
			<xsl:for-each select="$columns">
				<fo:table-column column-width="{concat(pref_table:width, '%')}"/>
			</xsl:for-each>
			
			<fo:table-header xsl:use-attribute-sets="artifact-table-header">
				<xsl:for-each select="$columns">
					<fo:table-cell xsl:use-attribute-sets="artifact-table-header-cell">
						<xsl:variable name="header" select="pref_table:name"/>
						<fo:block>
                        	<xsl:choose>
                        		<xsl:when test="starts-with($header, 'isCategoryColumn')">
                        			<xsl:value-of select="substring-after($header, 'isCategoryColumn')"/>
                        		</xsl:when>
                        		<xsl:when test="starts-with($header, 'isCustomColumn')">
                        			<xsl:value-of select="substring-after($header, 'isCustomColumn')"/> 			
                        		</xsl:when>
                        		<xsl:when test="$header='id'">
                        			<fo:block xsl:use-attribute-sets="id-header"><xsl:value-of select="$strings/*[name()=$header]"/></fo:block>
                        		</xsl:when>
                        		<xsl:when test="$header='priority'">
                        			<fo:block xsl:use-attribute-sets="priority-header"><xsl:value-of select="$strings/Priority"/></fo:block>
                        		</xsl:when>
                        		<xsl:otherwise>
                        			<xsl:value-of select="$strings/*[name()=$header]"/>
                        		</xsl:otherwise>
                        	</xsl:choose>
                        </fo:block>
					</fo:table-cell>
				</xsl:for-each>
			</fo:table-header>
			
			<fo:table-body>
				<xsl:choose>
					<xsl:when test="$suites">
						<xsl:for-each select="$suites">
							<xsl:variable name="testsuite" select="current()"/>
							
							<fo:table-row>
								<xsl:for-each select="$columns">
									<xsl:apply-templates mode="suite-value" select="current()">
										<xsl:with-param name="artifact" select="$testsuite"/>
									</xsl:apply-templates>
								</xsl:for-each>
							</fo:table-row>
						</xsl:for-each>
					</xsl:when>
					<xsl:otherwise>
						<!-- inserts emptly columns so that the table header can still appear in the document -->
						<fo:table-row>
							<xsl:for-each select="$columns">
								<fo:table-cell><fo:block/></fo:table-cell>
							</xsl:for-each>
						</fo:table-row>
					</xsl:otherwise>
				</xsl:choose>
			</fo:table-body>
		</fo:table>
	</xsl:template>

    <xsl:template mode="suite-value" match="pref_table:column">
        <xsl:param name="artifact"/>
        
        <fo:table-cell xsl:use-attribute-sets="artifact-table-body-cell">
        	<fo:block-container overflow="hidden">
	            <!--  match type-specific value templates locally defined in this file  -->
	            <xsl:apply-templates mode="suite-value" select="pref_table:name">
	                <xsl:with-param name="artifact" select="$artifact"/>
	            </xsl:apply-templates>
	            <!--  and now match generic value templates from abstract-table-summary.xsl  -->
	            <xsl:apply-templates select="pref_table:name">
	                <xsl:with-param name="artifact" select="$artifact"/>
	            </xsl:apply-templates>
            </fo:block-container>
        </fo:table-cell>
    </xsl:template>
	
	<!-- ID -->
	<xsl:template mode="suite-value" match="pref_table:column/pref_table:name[.='id']">
		<xsl:param name="artifact"/>
		
		<fo:block xsl:use-attribute-sets="id-cell"><fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{$artifact/dc:identifier}"><xsl:value-of select="$artifact/rqm:webId"/></fo:basic-link></fo:block>
	</xsl:template>
	
	<!-- Name -->
	<xsl:template mode="suite-value" match="pref_table:column/pref_table:name[.='name']">
		<xsl:param name="artifact"/>
		
		<fo:block><fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{$artifact/dc:identifier}"><xsl:apply-templates select="$artifact/dc:title"/></fo:basic-link></fo:block>
	</xsl:template>
	
	<!-- Priority -->
	<xsl:template mode="suite-value" match="pref_table:column/pref_table:name[.='priority']">
		<xsl:param name="artifact"/>
		
		<fo:block xsl:use-attribute-sets="priority-cell">
			<xsl:apply-templates select="$artifact/rqm:priority" />
		</fo:block>
	</xsl:template>
	
	<!-- Locked -->
	<xsl:template mode="suite-value" match="pref_table:column/pref_table:name[.='locked']">
		<xsl:param name="artifact"/>
		
		<fo:block><xsl:apply-templates select="$artifact/rqm:locked"/></fo:block>
	</xsl:template>
	
	<!-- Owner -->
	<xsl:template mode="suite-value" match="pref_table:column/pref_table:name[.='owner']">
		<xsl:param name="artifact"/>
		
		<fo:block><xsl:apply-templates select="$artifact/prep:owner"/></fo:block>
	</xsl:template>
	
	<!-- Created By -->
	<xsl:template mode="suite-value" match="pref_table:column/pref_table:name[.='originator']">
		<xsl:param name="artifact"/>
		
		<fo:block><xsl:apply-templates select="$artifact/prep:creator"/></fo:block>
	</xsl:template>
	
	<!-- State -->
	<xsl:template mode="suite-value" match="pref_table:column/pref_table:name[.='state']">
		<xsl:param name="artifact"/>
		<xsl:variable name="stateId" select="$artifact/alm:state"/>                  
		<xsl:variable name="stateName" select="$strings/*[name()=$stateId]"/>    
        <fo:block>
            <xsl:choose>
            	<xsl:when test="$stateName">
                	<xsl:value-of select="$stateName"/>
				</xsl:when>
                <xsl:otherwise>
                	<xsl:variable name="workflowType" select="'TESTSUITE'"/>    
					<xsl:value-of select="utils:getStateName($projectArea, $workflowType, $stateId)"/>
                </xsl:otherwise>
            </xsl:choose>
        </fo:block>            
	</xsl:template>
	
	<!-- Modified -->
	<xsl:template mode="suite-value" match="pref_table:column/pref_table:name[.='modified']">
		<xsl:param name="artifact"/>
		
		<fo:block><xsl:value-of select="time:formatMediumDate($artifact/alm:updated, $locale)"/></fo:block>
	</xsl:template>
	
	<!-- Weight -->
	<xsl:template mode="suite-value" match="pref_table:column/pref_table:name[.='weight']">
		<xsl:param name="artifact"/>
		
		<fo:block><xsl:value-of select="$artifact/rqm:weight"/></fo:block>
	</xsl:template>
	
	<!-- Risk -->
	<xsl:template mode="suite-value" match="pref_table:column/pref_table:name[.='calculatedRisk']">
		<xsl:param name="artifact"/>
		
		<fo:block>
			<xsl:call-template name="drawRiskIconSet">
				<xsl:with-param name="iconCount"><xsl:value-of select="$artifact/rqm:risk/rqm:riskLevel"/></xsl:with-param>
				<xsl:with-param name="positiveIcon">risk-selected_obj.gif</xsl:with-param>
				<xsl:with-param name="negativeIcon">risk-unselect_obj.gif</xsl:with-param>
			</xsl:call-template>
		</fo:block>
	</xsl:template>
	
	<!-- Community Risk -->
	<xsl:template mode="suite-value" match="pref_table:column/pref_table:name[.='communityRisk']">
		<xsl:param name="artifact"/>
		
		<fo:block>
			<xsl:call-template name="drawRiskIconSet">
				<xsl:with-param name="iconCount"><xsl:value-of select="$artifact/rqm:risk/rqm:commnunityAssessment/rqm:riskLevel"/></xsl:with-param>
				<xsl:with-param name="positiveIcon">risk-selected_obj.gif</xsl:with-param>
				<xsl:with-param name="negativeIcon">risk-unselect_obj.gif</xsl:with-param>
			</xsl:call-template>
		</fo:block>
	</xsl:template>
	
	<!-- Custom Categories (handled by abstract-table-summary.xsl -->
	
	<!-- Executions -->
	<xsl:template mode="suite-value" match="pref_table:column/pref_table:name[.='tsexec']">
		<xsl:param name="artifact"/>
		
		<fo:block><xsl:value-of select="$artifact/prep:tserCount"/></fo:block>		
	</xsl:template>
	
	<!-- Total Estimates -->
	<xsl:template mode="suite-value" match="pref_table:column/pref_table:name[.='tsestimate']">
		<xsl:param name="artifact"/>
		<xsl:variable name="tsestimate" select="$artifact/prep:tsestimate"/>
		
		<fo:block>
			<xsl:value-of select="$strings/Total"/>:
			<fo:inline xsl:use-attribute-sets="value">
				<xsl:choose>
					<xsl:when test="$tsestimate/prep:totalTimeSpent">
						<xsl:value-of select="time:formatTimeEstimate($tsestimate/prep:totalTimeSpent, $locale)"/>
					</xsl:when>
					<xsl:otherwise>0</xsl:otherwise>
				</xsl:choose>
				<xsl:text> / </xsl:text>
				<xsl:choose>
					<xsl:when test="$tsestimate/prep:totalEstimate">
						<xsl:value-of select="time:formatTimeEstimate($tsestimate/prep:totalEstimate, $locale)"/>
					</xsl:when>
					<xsl:otherwise>0</xsl:otherwise>
				</xsl:choose>
			</fo:inline>
		</fo:block>
		<fo:block>
			<xsl:value-of select="$strings/Estimated"/>:
			<fo:inline xsl:use-attribute-sets="value">
				<xsl:choose>
					<xsl:when test="$tsestimate/prep:estimatePercentage">
						<xsl:value-of select="$tsestimate/prep:estimatePercentage"/>
					</xsl:when>
					<xsl:otherwise>0</xsl:otherwise>
				</xsl:choose>
			</fo:inline>
			<xsl:text>%</xsl:text>
		</fo:block>
	</xsl:template>
	
	<!-- Estimate -->
    <xsl:template mode="suite-value" match="pref_table:column/pref_table:name[.='estimate']">
        <xsl:param name="artifact"/>
        <xsl:choose>
        	<xsl:when test="$artifact/rqm:estimate">
        		<fo:block><xsl:value-of select="time:formatTimeEstimate($artifact/rqm:estimate, $locale)"/></fo:block>
        	</xsl:when>
        	<xsl:otherwise>
        	       <fo:block></fo:block>
        	</xsl:otherwise>
        </xsl:choose>
    </xsl:template>
	
	<!-- Points -->
	<xsl:template mode="suite-value" match="pref_table:column/pref_table:name[.='tspoints']">
		<xsl:param name="artifact"/>
		<xsl:variable name="tspoints" select="$artifact/prep:tspoints"/>
		
		<fo:block>
			<xsl:value-of select="$strings/Progress"/>:
			<fo:inline xsl:use-attribute-sets="value">
				<xsl:choose>
					<xsl:when test="$tspoints/prep:attempted">
						<xsl:value-of select="$tspoints/prep:attempted"/>
					</xsl:when>
					<xsl:otherwise>0</xsl:otherwise>
				</xsl:choose>
			</fo:inline>
		</fo:block>
		<fo:block>
			<xsl:value-of select="$strings/Total"/>:
			<fo:inline xsl:use-attribute-sets="value">
				<xsl:choose>
					<xsl:when test="$tspoints/prep:total">
						<xsl:value-of select="$tspoints/prep:total"/>
					</xsl:when>
					<xsl:otherwise>0</xsl:otherwise>
				</xsl:choose>
			</fo:inline>
		</fo:block>
	</xsl:template>
	
	<!-- Attempts -->
	<xsl:template mode="suite-value" match="pref_table:column/pref_table:name[.='tscount']">
		<xsl:param name="artifact"/>
		<xsl:variable name="tscount" select="$artifact/prep:tscount"/>
		
		<fo:block><xsl:value-of select="$strings/Progress"/>:
			<fo:inline xsl:use-attribute-sets="value">
				<xsl:choose>
					<xsl:when test="$tscount/prep:total and $tscount/prep:notStarted">
						<xsl:value-of select="$tscount/prep:total - $tscount/prep:notStarted"/>
					</xsl:when>
					<xsl:otherwise>0</xsl:otherwise>
				</xsl:choose>
			</fo:inline>
		</fo:block>
		<fo:block><xsl:value-of select="$strings/Total"/>:
			<fo:inline xsl:use-attribute-sets="value">
				<xsl:choose>
					<xsl:when test="$tscount/prep:total">
						<xsl:value-of select="$tscount/prep:total"/>
					</xsl:when>
					<xsl:otherwise>0</xsl:otherwise>
				</xsl:choose>
			</fo:inline>
		</fo:block>
	</xsl:template>
	
	<!-- Copied From -->
    <xsl:template mode="suite-value" match="pref_table:column/pref_table:name[.='copiedFrom']">
        <xsl:param name="artifact"/>
        <xsl:variable name="copiedFrom" select="$artifact/prep:copiedFrom"/>  
        
        <xsl:choose>
            <xsl:when test="$copiedFrom">
                <fo:block>
                    <fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{$copiedFrom/dc:identifier}">
                        <xsl:apply-templates select="$copiedFrom/rqm:webId"/>
                    </fo:basic-link>
                </fo:block>
            </xsl:when>
            <xsl:otherwise><fo:block>--</fo:block></xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <!-- Copy Root -->
    <xsl:template mode="suite-value" match="pref_table:column/pref_table:name[.='copiedRoot']">
        <xsl:param name="artifact"/>
        <xsl:variable name="copiedRoot" select="$artifact/prep:copiedRoot"/>
        
        <xsl:choose>
            <xsl:when test="$copiedRoot">
                <fo:block>
                    <fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{$copiedRoot/dc:identifier}">
                        <xsl:apply-templates select="$copiedRoot/rqm:webId"/>
                    </fo:basic-link>
                </fo:block>
            </xsl:when>
            <xsl:otherwise><fo:block>--</fo:block></xsl:otherwise>
        </xsl:choose>
    </xsl:template>
	
</xsl:stylesheet>