<?xml version="1.0" encoding="utf-8"?>
<!-- 
/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2014. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 -->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:fo="http://www.w3.org/1999/XSL/Format"
    xmlns:rqm="http://jazz.net/xmlns/alm/qm/v0.1/"
    xmlns:dc="http://purl.org/dc/elements/1.1/"
    xmlns:alm="http://jazz.net/xmlns/alm/v0.1/"
    xmlns:tsl="http://jazz.net/xmlns/alm/qm/v0.1/tsl/v0.1/"
    xmlns:prep="http://com.ibm.rqm.print/prep"
    xmlns:time="xalan://com.ibm.rqm.print.render.fop.xml.xslt.extension.TimeUtilsExtension"
    xmlns:utils="xalan://com.ibm.rqm.print.render.fop.xml.xslt.extension.UtilsExtension"
    xmlns:log="xalan://com.ibm.rqm.print.render.fop.xml.xslt.extension.LoggingExtension"
    exclude-result-prefixes="xsl rqm">
    
    <xsl:import  href="cache://stylesheets/render/abstract-table-summary.xsl"/>

    <xsl:template name="buildTestSuiteResultsTable">
        <xsl:param name="columns"/>
        
        <fo:table xsl:use-attribute-sets="artifact-table">
            <xsl:for-each select="$columns">
                <fo:table-column column-width="{concat(prep:width, '%')}"/>
                
            </xsl:for-each>
            
            <fo:table-header xsl:use-attribute-sets="artifact-table-header">
                <xsl:for-each select="$columns">
                    <fo:table-cell xsl:use-attribute-sets="artifact-table-header-cell">
                        <xsl:variable name="header" select="prep:name"/>
                        <fo:block>
                        	<xsl:choose>
                        		<xsl:when test="starts-with($header, 'isCategoryColumn')">
                        			<xsl:value-of select="substring-after($header, 'isCategoryColumn')"/>
                        		</xsl:when>
                        	    <xsl:when test="starts-with($header, 'isCustomColumn')">
                        	        <xsl:value-of select="substring-after($header, 'isCustomColumn')"/> 			
                        	    </xsl:when>
                        	    <xsl:when test="$header='id'">
                        	        <fo:block xsl:use-attribute-sets="id-header"><xsl:value-of select="$strings/id"/></fo:block>
                        	    </xsl:when>
                        		<xsl:otherwise>
                        			<xsl:value-of select="$strings/*[name()=translate($header, ' ', '_')]"/>
                        		</xsl:otherwise>
                        	</xsl:choose>
                        </fo:block>
                    </fo:table-cell>
                </xsl:for-each>
            </fo:table-header>
            
            <fo:table-body>
                <xsl:choose>
                    <xsl:when test="rqm:testsuitelog">
                        <xsl:for-each select="rqm:testsuitelog">
                            <xsl:variable name="result" select="current()"/>
                            <fo:table-row>
                                <xsl:for-each select="$columns">
                                
                                    <xsl:apply-templates mode="tsl-value" select="current()">
                                        <xsl:with-param name="artifact" select="$result"/>
                                    </xsl:apply-templates>
                                </xsl:for-each>
                            </fo:table-row>
                        </xsl:for-each>
                    </xsl:when>
                    <xsl:otherwise>
                        <!-- inserts emptly columns so that the table header can still appear in the document -->
                        
                        <fo:table-row>
                            <xsl:for-each select="$columns">
                                <fo:table-cell><fo:block/></fo:table-cell>
                            </xsl:for-each>
                        </fo:table-row>
                    </xsl:otherwise>
                </xsl:choose>
            </fo:table-body>
        </fo:table>
    </xsl:template>
    
    <xsl:template mode="tsl-value" match="prep:column">
        <xsl:param name="artifact"/>
        
        <fo:table-cell xsl:use-attribute-sets="artifact-table-body-cell">
        	<fo:block-container overflow="hidden">
	            <!--  match type-specific value templates locally defined in this file  -->
	            <xsl:apply-templates mode="tsl-value" select="prep:name">
	                <xsl:with-param name="artifact" select="$artifact"/>
	            </xsl:apply-templates>
	            <!--  and now match generic value templates from abstract-table-summary.xsl  -->
	            <xsl:apply-templates select="prep:name">
	                <xsl:with-param name="artifact" select="$artifact"/>
	            </xsl:apply-templates>
            </fo:block-container>
        </fo:table-cell>
    </xsl:template>
    
    <!-- ID -->
    <xsl:template mode="tsl-value" match="prep:column/prep:name[.='id' or .='ID']">
        <xsl:param name="artifact"/>
        
        <fo:block xsl:use-attribute-sets="id-cell">
            <fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{$artifact/dc:identifier}">
                <xsl:value-of select="$artifact/rqm:webId"/>
            </fo:basic-link>
        </fo:block>
    </xsl:template>
    
    <!-- Name -->
    <xsl:template mode="tsl-value" match="prep:column/prep:name[.='name' or .='Name']">
        <xsl:param name="artifact"/>
        
        <fo:block>
            <fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{$artifact/dc:identifier}">
                <xsl:apply-templates select="$artifact/dc:title"/>
            </fo:basic-link>
        </fo:block>
    </xsl:template>
    
    <!-- Test Suite Name -->
    <xsl:template mode="tsl-value" match="prep:column/prep:name[.='testSuiteName']">
        <xsl:param name="artifact"/>
        <xsl:variable name="testsuite" select="$artifact/rqm:testsuite"/>
        
        <xsl:choose>
            <xsl:when test="$testsuite">
                <fo:block>
                    <fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{$testsuite/dc:identifier}">
                        <xsl:apply-templates select="$testsuite/dc:title"/>
                    </fo:basic-link>
                </fo:block>
            </xsl:when>
            <xsl:otherwise><fo:block/></xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <!-- State -->
    <xsl:template mode="tsl-value" match="prep:column/prep:name[.='documentStateIds']">
        <xsl:param name="artifact"/>
        <xsl:if test="$artifact">
	        <xsl:variable name="stateId" select="$artifact/alm:processState"/>    
	                      
			<xsl:variable name="stateName" select="$strings/*[name()=$stateId]"/>    
	        <fo:block>
	            <xsl:choose>
	            	<xsl:when test="$stateName">
	                	<xsl:value-of select="$stateName"/>
					</xsl:when>
	                <xsl:otherwise>
	                	<xsl:variable name="workflowType" select="'TESTSUITERESULT'"/>   
	                	 
						<xsl:value-of select="utils:getStateName($projectArea, $workflowType, $stateId)"/>
	                </xsl:otherwise>
	            </xsl:choose>
	        </fo:block>
	    </xsl:if>            
    </xsl:template>
    
    <!-- Locked -->
    <xsl:template mode="tsl-value" match="prep:column/prep:name[.='locked']">
        <xsl:param name="artifact"/>
        
        <fo:block><xsl:apply-templates select="$artifact/rqm:locked"/></fo:block>
    </xsl:template>
    
    <!-- Owner -->
    <xsl:template mode="tsl-value" match="prep:column/prep:name[.='ownerUUIDs']">
        <xsl:param name="artifact"/>
        
        <fo:block><xsl:apply-templates select="$artifact/prep:owner"/></fo:block>
    </xsl:template>
    
    <!-- Created By -->
    <xsl:template mode="tsl-value" match="prep:column/prep:name[.='originator']">
        <xsl:param name="artifact"/>
        
        <fo:block><xsl:apply-templates select="$artifact/dc:creator"/></fo:block>
    </xsl:template>

    <!-- Completed -->
    <xsl:template mode="tsl-value" match="prep:column/prep:name[.='completed']">
        <xsl:param name="artifact"/>
        <xsl:variable name="endTime" select="$artifact/tsl:endtime"/>
        
        <xsl:choose>
            <xsl:when test="$endTime">
                <fo:block><xsl:value-of select="time:formatMediumDate($artifact/tsl:endtime, $locale)"/></fo:block>        
            </xsl:when>
            <xsl:otherwise><fo:block><xsl:value-of select="$strings/N_A"/></fo:block></xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <!-- Status -->
    <xsl:template mode="tsl-value" match="prep:column/prep:name[.='stateIds' or .='Actual Result']">
        <xsl:param name="artifact"/>
        <xsl:variable name="state" select="$artifact/alm:state"/>
        <xsl:variable name="customStateName" select="$artifact/alm:state/@value"/>
        
        <fo:block>
            <xsl:call-template name="printResultStateIcon">
                <xsl:with-param name="size">45%</xsl:with-param>
                <xsl:with-param name="priority_id"><xsl:value-of select="$state"/></xsl:with-param>
            </xsl:call-template>
            <xsl:text> </xsl:text>
            <xsl:choose>
                <xsl:when test="$customStateName">
                    <xsl:value-of select="$customStateName"/></xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="$strings/*[name()=$state]"/></xsl:otherwise>
            </xsl:choose>
        </fo:block>
    </xsl:template>
    
    <!-- Weight -->
    <xsl:template mode="tsl-value" match="prep:column/prep:name[.='weight']">
        <xsl:param name="artifact"/>
        
        <fo:block><xsl:value-of select="$artifact/tsl:weight"/></fo:block>
    </xsl:template>
    
    <!-- Build -->
    <xsl:template mode="tsl-value" match="prep:column/prep:name[.='buildRecordName']">
        <xsl:param name="artifact"/>
        <xsl:variable name="buildrecord" select="$artifact/rqm:buildrecord"/>
        
        <xsl:choose>
            <xsl:when test="$buildrecord">
                <fo:block>
                    <fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{$buildrecord/dc:identifier}">
                        <xsl:apply-templates select="$buildrecord/dc:title"/>
                    </fo:basic-link>
                </fo:block>
            </xsl:when>
            <xsl:otherwise><fo:block/></xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <!-- Passed Points -->
    <xsl:template mode="tsl-value" match="prep:column/prep:name[.='pointsPass']">
        <xsl:param name="artifact"/>
        <fo:block><xsl:value-of select="$artifact/tsl:pointspassed"/></fo:block>
    </xsl:template>
    
    <!-- Failed Points -->
    <xsl:template mode="tsl-value" match="prep:column/prep:name[.='pointsFail']">
        <xsl:param name="artifact"/>
        <fo:block><xsl:value-of select="$artifact/tsl:pointsfailed"/></fo:block>
    </xsl:template>
    
    <!-- Blocked Points -->
    <xsl:template mode="tsl-value" match="prep:column/prep:name[.='pointsBlocked']">
        <xsl:param name="artifact"/>
        <fo:block><xsl:value-of select="$artifact/tsl:pointsblocked"/></fo:block>
    </xsl:template>
    
    <!-- Inconclusive Points -->
    <xsl:template mode="tsl-value" match="prep:column/prep:name[.='pointsInconclusive']">
        <xsl:param name="artifact"/>
        <fo:block><xsl:value-of select="$artifact/tsl:pointsinconclusive"/></fo:block>
    </xsl:template>
    
    <!-- Attempted Points -->
    <xsl:template mode="tsl-value" match="prep:column/prep:name[.='pointsAttempted']">
        <xsl:param name="artifact"/>
        <fo:block><xsl:value-of select="$artifact/tsl:pointsattempted"/></fo:block>
    </xsl:template>
    
    <!-- Test Plan -->
    <xsl:template mode="tsl-value" match="prep:column/prep:name[.='testPlan']">
        <xsl:param name="artifact"/>
        <xsl:variable name="testplan" select="$artifact/rqm:testplan"/>
        
        <xsl:choose>
            <xsl:when test="$testplan">
                <fo:block>
                    <fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{$testplan/dc:identifier}">
                        <xsl:apply-templates select="$testplan/dc:title"/>
                    </fo:basic-link>
                </fo:block>
            </xsl:when>
            <xsl:otherwise><fo:block/></xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <!-- Iteration -->
	<xsl:template mode="tsl-value" match="prep:column/prep:name[.='testPhases']">
		<xsl:param name="artifact"/>
		<xsl:variable name="iteration" select="$artifact/rqm:testphase"/>
		
		<xsl:choose>
			<xsl:when test="$iteration">
				<fo:block>
					<fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{$iteration/dc:identifier}">
						<xsl:apply-templates select="$iteration/dc:title"/>
					</fo:basic-link>
				</fo:block>
			</xsl:when>
			<xsl:otherwise><fo:block><xsl:value-of select="$strings/Unassigned"/></fo:block></xsl:otherwise>
		</xsl:choose>
	</xsl:template>
    
    <!-- Passed Test Cases -->
    <xsl:template mode="tsl-value" match="prep:column/prep:name[.='passedTestCases']">
        <xsl:param name="artifact"/>
        <fo:block><xsl:value-of select="$artifact/tsl:testcasespassed"/></fo:block>
    </xsl:template>
    
    <!-- Failed Test Cases -->
    <xsl:template mode="tsl-value" match="prep:column/prep:name[.='failedTestCases']">
        <xsl:param name="artifact"/>
        <fo:block><xsl:value-of select="$artifact/tsl:testcasesfailed"/></fo:block>
    </xsl:template>
    
    <!-- Blocked Test Cases -->
    <xsl:template mode="tsl-value" match="prep:column/prep:name[.='blockedTestCases']">
        <xsl:param name="artifact"/>
        <fo:block><xsl:value-of select="$artifact/tsl:testcasesblocked"/></fo:block>
    </xsl:template>
    
    <!-- In Progress Test Cases -->
    <xsl:template mode="tsl-value" match="prep:column/prep:name[.='inProgressTestCases']">
        <xsl:param name="artifact"/>
        <fo:block><xsl:value-of select="$artifact/tsl:testcasesinprogress"/></fo:block>
    </xsl:template>
    
    <!-- Inconclusive Test Cases -->
    <xsl:template mode="tsl-value" match="prep:column/prep:name[.='inconclusiveTestCases']">
        <xsl:param name="artifact"/>
        <fo:block><xsl:value-of select="$artifact/tsl:testcasesinconclusive"/></fo:block>
    </xsl:template>
    
    <!-- Not Started Test Cases -->
    <xsl:template mode="tsl-value" match="prep:column/prep:name[.='notStartedTestCases']">
        <xsl:param name="artifact"/>
        <fo:block><xsl:value-of select="$artifact/tsl:testcasesnotstarted"/></fo:block>
    </xsl:template>
    
    <!-- Total Test Cases -->
    <xsl:template mode="tsl-value" match="prep:column/prep:name[.='totalTestCases']">
        <xsl:param name="artifact"/>
        <fo:block><xsl:value-of select="count($artifact/tsl:suiteelements)"/></fo:block>
    </xsl:template>
    
    <!-- Start Time -->
    <xsl:template mode="tsl-value" match="prep:column/prep:name[.='Start Time']">
        <xsl:param name="artifact"/>
        <fo:block><xsl:value-of select="time:formatMediumDateMediumTime($artifact/tsl:starttime, $locale)"/></fo:block>
    </xsl:template>
    
    <!-- Defects -->
    <xsl:template mode="tsl-value" match="prep:column/prep:name[.='defects']">
        <xsl:param name="artifact"/>
        <xsl:variable name="defects" select="$artifact/rqm:executionresult/rqm:defect"/>			
        
        <fo:block>
            <xsl:choose>
                <xsl:when test="$defects">
                    <xsl:for-each select="$defects">
                        <xsl:if test="position() > 1">, </xsl:if>
                        
                        <fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="$defects/@href">
                            <xsl:apply-templates select="$defects/@summary"/>
                        </fo:basic-link>
                    </xsl:for-each>
                </xsl:when>
                <xsl:otherwise>--</xsl:otherwise>
            </xsl:choose>	
        </fo:block>
    </xsl:template>
    
     <!-- Audit -->
    <xsl:template mode="tsl-value" match="prep:column/prep:name[.='testPlanAtExecutionDocumentStateIds']">
        <xsl:param name="artifact"/>
		<xsl:variable name="testplanstateatexecutionstateid" select="$artifact/tsl:testplanstateatexecution"/>
		<xsl:variable name="testplanstateatexecutionstatename" select="$strings/*[name()=$testplanstateatexecutionstateid]"/>    
        <fo:block>
            <!-- Note: The associated test suite may not be associated to a test plan. -->
        	<xsl:if test="$testplanstateatexecutionstateid">
	            <xsl:choose>
	            	<xsl:when test="$testplanstateatexecutionstatename">
	                	<xsl:value-of select="$testplanstateatexecutionstatename"/>
					</xsl:when>
	                <xsl:otherwise>
	                	<xsl:variable name="workflowType" select="'TESTPLAN'"/>    
						<xsl:value-of select="utils:getStateName($projectArea, $workflowType, $testplanstateatexecutionstateid)"/>
	                </xsl:otherwise>
	            </xsl:choose>
            </xsl:if>
        </fo:block>         
    </xsl:template>
    <xsl:template mode="tsl-value" match="prep:column/prep:name[.='testSuiteAtExecutionDocumentStateIds']">
        <xsl:param name="artifact"/>
		<xsl:variable name="testsuitestateatexecutionstateid" select="$artifact/tsl:testsuitestateatexecution"/>
		<xsl:variable name="testsuitestateatexecutionstatename" select="$strings/*[name()=$testsuitestateatexecutionstateid]"/>    
        <fo:block>
            <xsl:choose>
            	<xsl:when test="$testsuitestateatexecutionstatename">
                	<xsl:value-of select="$testsuitestateatexecutionstatename"/>
				</xsl:when>
                <xsl:otherwise>
                	<xsl:variable name="workflowType" select="'TESTSUITE'"/>    
					<xsl:value-of select="utils:getStateName($projectArea, $workflowType, $testsuitestateatexecutionstateid)"/>
                </xsl:otherwise>
            </xsl:choose>
        </fo:block>         
    </xsl:template>
    <!-- isCurrent -->
    <xsl:template mode="tsl-value" match="prep:column/prep:name[.='isCurrent' ]">
        <xsl:param name="artifact"/>
		<fo:block><xsl:value-of select="$artifact/tsl:isCurrent"/></fo:block>
    </xsl:template>
    
</xsl:stylesheet>